/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2016 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 */
package de.cau.cs.kieler.kgraph.text.ui.random;

import com.google.common.base.Objects;
import de.cau.cs.kieler.kgraph.text.grandom.Configuration;
import de.cau.cs.kieler.kgraph.text.grandom.ConstraintType;
import de.cau.cs.kieler.kgraph.text.grandom.DoubleQuantity;
import de.cau.cs.kieler.kgraph.text.grandom.Edges;
import de.cau.cs.kieler.kgraph.text.grandom.Flow;
import de.cau.cs.kieler.kgraph.text.grandom.FlowType;
import de.cau.cs.kieler.kgraph.text.grandom.Form;
import de.cau.cs.kieler.kgraph.text.grandom.Formats;
import de.cau.cs.kieler.kgraph.text.grandom.GeneratorOptions;
import de.cau.cs.kieler.kgraph.text.grandom.Hierarchy;
import de.cau.cs.kieler.kgraph.text.grandom.Nodes;
import de.cau.cs.kieler.kgraph.text.grandom.Ports;
import de.cau.cs.kieler.kgraph.text.grandom.RandGraph;
import de.cau.cs.kieler.kgraph.text.grandom.RandomGraphGenerator;
import de.cau.cs.kieler.kgraph.text.grandom.Side;
import de.cau.cs.kieler.kgraph.text.grandom.Size;
import de.cau.cs.kieler.kgraph.text.ui.random.GeneratorOptionsUtil;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.elk.core.options.PortConstraints;
import org.eclipse.elk.graph.KNode;
import org.eclipse.elk.graph.properties.IProperty;
import org.eclipse.elk.graph.properties.MapPropertyHolder;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;

@SuppressWarnings("all")
public class GRandomGraphMaker {
  private final Iterable<Configuration> configs;
  
  private final int DEFAULT_NUM_GRAPHS = 1;
  
  private final String DEFAULT_NAME = "random";
  
  private final String DEFAULT_FORMAT = Formats.KGT.toString();
  
  public GRandomGraphMaker(final RandGraph rdg) {
    EList<Configuration> _configs = rdg.getConfigs();
    this.configs = _configs;
  }
  
  public void gen(final IProject project) {
    try {
      for (final Configuration config : this.configs) {
        {
          String _xifexpression = null;
          String _filename = config.getFilename();
          boolean _exists = this.exists(_filename);
          if (_exists) {
            _xifexpression = config.getFilename();
          } else {
            _xifexpression = this.DEFAULT_NAME;
          }
          final String filename = _xifexpression;
          Object _xifexpression_1 = null;
          Formats _format = config.getFormat();
          boolean _exists_1 = this.exists(_format);
          if (_exists_1) {
            _xifexpression_1 = config.getFormat();
          } else {
            _xifexpression_1 = this.DEFAULT_FORMAT;
          }
          final Object format = ((Object)_xifexpression_1);
          final List<KNode> graphs = this.genKNode(config);
          int fileNum = 0;
          for (int i = 0; (i < graphs.size()); i++) {
            {
              while (project.getFile((((filename + Integer.valueOf(fileNum)) + ".") + format)).exists()) {
                fileNum++;
              }
              final IFile f = project.getFile((((filename + Integer.valueOf(fileNum)) + ".") + format));
              KNode _get = graphs.get(i);
              this.serialize(_get, f);
            }
          }
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public List<KNode> genKNode(final Configuration config) {
    final ArrayList<KNode> generated = new ArrayList<KNode>();
    final int samples = config.getSamples();
    int _xifexpression = (int) 0;
    boolean _exists = this.exists(Integer.valueOf(samples));
    if (_exists) {
      _xifexpression = samples;
    } else {
      _xifexpression = this.DEFAULT_NUM_GRAPHS;
    }
    final int nGraphs = _xifexpression;
    final Random r = this.random(config);
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, nGraphs, true);
    for (final Integer i : _doubleDotLessThan) {
      {
        final GeneratorOptions options = this.parseGenOptions(config, r);
        RandomGraphGenerator _randomGraphGenerator = new RandomGraphGenerator(r);
        final KNode n = _randomGraphGenerator.generate(options);
        generated.add(n);
      }
    }
    return generated;
  }
  
  public GeneratorOptions parseGenOptions(final Configuration config, final Random r) {
    GeneratorOptions _xblockexpression = null;
    {
      final GeneratorOptions genOpt = new GeneratorOptions();
      IPreferenceStore _preferenceStore = GeneratorOptionsUtil.getPreferenceStore();
      boolean _exists = this.exists(_preferenceStore);
      if (_exists) {
        GeneratorOptionsUtil.loadPreferences(genOpt);
      }
      this.setGraphType(config, genOpt);
      Nodes _nodes = config.getNodes();
      this.nodes(_nodes, r, genOpt);
      this.edges(config, genOpt, r);
      this.hierarchy(config, genOpt, r);
      DoubleQuantity _fraction = config.getFraction();
      this.setQuantities(genOpt, _fraction, GeneratorOptions.PARTITION_FRAC);
      Integer _maxWidth = config.getMaxWidth();
      this.<Integer>setIfExists(genOpt, _maxWidth, GeneratorOptions.MAX_WIDTH);
      Integer _maxDegree = config.getMaxDegree();
      this.<Integer>setIfExists(genOpt, _maxDegree, GeneratorOptions.MAX_DEGREE);
      _xblockexpression = genOpt;
    }
    return _xblockexpression;
  }
  
  public MapPropertyHolder hierarchy(final Configuration configuration, final GeneratorOptions options, final Random random) {
    MapPropertyHolder _xblockexpression = null;
    {
      final Hierarchy hierarchy = configuration.getHierarchy();
      MapPropertyHolder _xifexpression = null;
      boolean _exists = this.exists(hierarchy);
      if (_exists) {
        MapPropertyHolder _xblockexpression_1 = null;
        {
          final DoubleQuantity hierarch = hierarchy.getNumHierarchNodes();
          DoubleQuantity _crossHierarchRel = hierarchy.getCrossHierarchRel();
          this.setQuantities(options, _crossHierarchRel, GeneratorOptions.EXACT_RELATIVE_HIER);
          boolean _exists_1 = this.exists(hierarch);
          options.<Boolean>setProperty(GeneratorOptions.SMALL_HIERARCHY, Boolean.valueOf(_exists_1));
          DoubleQuantity _levels = hierarchy.getLevels();
          this.setQuantities(options, _levels, GeneratorOptions.MAX_HIERARCHY_LEVEL);
          this.setQuantities(options, hierarch, GeneratorOptions.NUMBER_HIERARCHICAL_NODES);
          DoubleQuantity _edges = hierarchy.getEdges();
          _xblockexpression_1 = this.setQuantities(options, _edges, GeneratorOptions.CROSS_HIER);
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private Random random(final Configuration g) {
    final Integer seed = g.getSeed();
    boolean _equals = Objects.equal(seed, null);
    if (_equals) {
      return new Random();
    } else {
      return new Random((seed).intValue());
    }
  }
  
  private MapPropertyHolder edges(final Configuration config, final GeneratorOptions genOpt, final Random r) {
    MapPropertyHolder _xblockexpression = null;
    {
      final Edges edges = config.getEdges();
      MapPropertyHolder _xifexpression = null;
      boolean _exists = this.exists(edges);
      if (_exists) {
        MapPropertyHolder _xblockexpression_1 = null;
        {
          boolean _isTotal = edges.isTotal();
          if (_isTotal) {
            genOpt.<GeneratorOptions.EdgeDetermination>setProperty(GeneratorOptions.EDGE_DETERMINATION, GeneratorOptions.EdgeDetermination.ABSOLUTE);
            DoubleQuantity _nEdges = edges.getNEdges();
            this.setQuantities(genOpt, _nEdges, GeneratorOptions.EDGES_ABSOLUTE);
          } else {
            boolean _isDensity = edges.isDensity();
            if (_isDensity) {
              DoubleQuantity _nEdges_1 = edges.getNEdges();
              this.setQuantities(genOpt, _nEdges_1, GeneratorOptions.DENSITY);
              genOpt.<GeneratorOptions.EdgeDetermination>setProperty(GeneratorOptions.EDGE_DETERMINATION, GeneratorOptions.EdgeDetermination.DENSITY);
            } else {
              boolean _isRelative = edges.isRelative();
              if (_isRelative) {
                genOpt.<GeneratorOptions.EdgeDetermination>setProperty(GeneratorOptions.EDGE_DETERMINATION, GeneratorOptions.EdgeDetermination.RELATIVE);
                DoubleQuantity _nEdges_2 = edges.getNEdges();
                this.setQuantities(genOpt, _nEdges_2, GeneratorOptions.RELATIVE_EDGES);
              } else {
                genOpt.<GeneratorOptions.EdgeDetermination>setProperty(GeneratorOptions.EDGE_DETERMINATION, GeneratorOptions.EdgeDetermination.OUTGOING);
                DoubleQuantity _nEdges_3 = edges.getNEdges();
                this.setQuantities(genOpt, _nEdges_3, GeneratorOptions.OUTGOING_EDGES);
              }
            }
          }
          boolean _isLabels = edges.isLabels();
          genOpt.<Boolean>setProperty(GeneratorOptions.EDGE_LABELS, Boolean.valueOf(_isLabels));
          boolean _isSelfLoops = edges.isSelfLoops();
          _xblockexpression_1 = genOpt.<Boolean>setProperty(GeneratorOptions.SELF_LOOPS, Boolean.valueOf(_isSelfLoops));
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private MapPropertyHolder nodes(final Nodes nodes, final Random r, final GeneratorOptions genOpt) {
    MapPropertyHolder _xifexpression = null;
    boolean _exists = this.exists(nodes);
    if (_exists) {
      MapPropertyHolder _xblockexpression = null;
      {
        DoubleQuantity _nNodes = nodes.getNNodes();
        this.setQuantities(genOpt, _nNodes, GeneratorOptions.NUMBER_OF_NODES);
        boolean _isLabels = nodes.isLabels();
        genOpt.<Boolean>setProperty(GeneratorOptions.CREATE_NODE_LABELS, Boolean.valueOf(_isLabels));
        this.ports(nodes, genOpt, r);
        this.size(nodes, r, genOpt);
        boolean _isRemoveIsolated = nodes.isRemoveIsolated();
        boolean _not = (!_isRemoveIsolated);
        _xblockexpression = genOpt.<Boolean>setProperty(GeneratorOptions.ISOLATED_NODES, Boolean.valueOf(_not));
      }
      _xifexpression = _xblockexpression;
    }
    return _xifexpression;
  }
  
  private MapPropertyHolder size(final Nodes nodes, final Random r, final GeneratorOptions genOpt) {
    MapPropertyHolder _xblockexpression = null;
    {
      final Size size = nodes.getSize();
      MapPropertyHolder _xifexpression = null;
      boolean _exists = this.exists(size);
      if (_exists) {
        MapPropertyHolder _xblockexpression_1 = null;
        {
          DoubleQuantity _width = size.getWidth();
          this.setQuantities(genOpt, _width, GeneratorOptions.NODE_WIDTH);
          DoubleQuantity _height = size.getHeight();
          _xblockexpression_1 = this.setQuantities(genOpt, _height, GeneratorOptions.NODE_HEIGHT);
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private MapPropertyHolder ports(final Nodes nodes, final GeneratorOptions genOpt, final Random r) {
    MapPropertyHolder _xblockexpression = null;
    {
      final Ports ports = nodes.getPorts();
      MapPropertyHolder _xifexpression = null;
      boolean _exists = this.exists(ports);
      if (_exists) {
        MapPropertyHolder _xblockexpression_1 = null;
        {
          genOpt.<Boolean>setProperty(GeneratorOptions.ENABLE_PORTS, Boolean.valueOf(true));
          boolean _isLabels = ports.isLabels();
          genOpt.<Boolean>setProperty(GeneratorOptions.CREATE_PORT_LABELS, Boolean.valueOf(_isLabels));
          ConstraintType _constraint = ports.getConstraint();
          PortConstraints _constraint_1 = this.getConstraint(_constraint);
          genOpt.<PortConstraints>setProperty(GeneratorOptions.PORT_CONSTRAINTS, _constraint_1);
          DoubleQuantity _reUse = ports.getReUse();
          this.setQuantities(genOpt, _reUse, GeneratorOptions.USE_EXISTING_PORTS_CHANCE);
          final EList<Flow> flows = ports.getFlow();
          boolean _exists_1 = this.exists(flows);
          if (_exists_1) {
            for (final Flow f : flows) {
              FlowType _flowType = f.getFlowType();
              Side _side = f.getSide();
              DoubleQuantity _amount = f.getAmount();
              this.setFlowSide(genOpt, _flowType, _side, _amount, r);
            }
          }
          final Size size = ports.getSize();
          genOpt.<Boolean>setProperty(GeneratorOptions.SET_PORT_SIZE, Boolean.valueOf(true));
          MapPropertyHolder _xifexpression_1 = null;
          boolean _exists_2 = this.exists(size);
          if (_exists_2) {
            MapPropertyHolder _xblockexpression_2 = null;
            {
              DoubleQuantity _height = size.getHeight();
              this.setQuantities(genOpt, _height, GeneratorOptions.PORT_HEIGHT);
              DoubleQuantity _width = size.getWidth();
              _xblockexpression_2 = this.setQuantities(genOpt, _width, GeneratorOptions.PORT_WIDTH);
            }
            _xifexpression_1 = _xblockexpression_2;
          }
          _xblockexpression_1 = _xifexpression_1;
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private MapPropertyHolder setQuantities(final GeneratorOptions genOpt, final DoubleQuantity quant, final IProperty<GeneratorOptions.RandVal> randomValue) {
    MapPropertyHolder _xifexpression = null;
    boolean _exists = this.exists(quant);
    if (_exists) {
      GeneratorOptions.RandVal _randVal = this.toRandVal(quant);
      _xifexpression = genOpt.<GeneratorOptions.RandVal>setProperty(randomValue, _randVal);
    }
    return _xifexpression;
  }
  
  public GeneratorOptions.RandVal toRandVal(final DoubleQuantity quant) {
    GeneratorOptions.RandVal _xifexpression = null;
    boolean _isMinMax = quant.isMinMax();
    if (_isMinMax) {
      Double _min = quant.getMin();
      Double _max = quant.getMax();
      _xifexpression = GeneratorOptions.RandVal.minMax((_min).doubleValue(), (_max).doubleValue());
    } else {
      GeneratorOptions.RandVal _xifexpression_1 = null;
      boolean _isGaussian = quant.isGaussian();
      if (_isGaussian) {
        Double _mean = quant.getMean();
        Double _stddv = quant.getStddv();
        _xifexpression_1 = GeneratorOptions.RandVal.gaussian((_mean).doubleValue(), (_stddv).doubleValue());
      } else {
        Double _quant = quant.getQuant();
        _xifexpression_1 = GeneratorOptions.RandVal.exact((_quant).doubleValue());
      }
      _xifexpression = _xifexpression_1;
    }
    return _xifexpression;
  }
  
  private MapPropertyHolder setFlowSide(final GeneratorOptions options, final FlowType type, final Side side, final DoubleQuantity quant, final Random r) {
    MapPropertyHolder _xblockexpression = null;
    {
      GeneratorOptions.RandVal _randVal = this.toRandVal(quant);
      final int amount = _randVal.intVal(r);
      MapPropertyHolder _switchResult = null;
      if (type != null) {
        switch (type) {
          case INCOMING:
            MapPropertyHolder _switchResult_1 = null;
            if (side != null) {
              switch (side) {
                case EAST:
                  _switchResult_1 = options.<Integer>setProperty(GeneratorOptions.INCOMING_EAST_SIDE, Integer.valueOf(amount));
                  break;
                case NORTH:
                  _switchResult_1 = options.<Integer>setProperty(GeneratorOptions.INCOMING_NORTH_SIDE, Integer.valueOf(amount));
                  break;
                case SOUTH:
                  _switchResult_1 = options.<Integer>setProperty(GeneratorOptions.INCOMING_SOUTH_SIDE, Integer.valueOf(amount));
                  break;
                case WEST:
                  _switchResult_1 = options.<Integer>setProperty(GeneratorOptions.INCOMING_WEST_SIDE, Integer.valueOf(amount));
                  break;
                default:
                  break;
              }
            }
            _switchResult = _switchResult_1;
            break;
          case OUTGOING:
            MapPropertyHolder _switchResult_2 = null;
            if (side != null) {
              switch (side) {
                case EAST:
                  _switchResult_2 = options.<Integer>setProperty(GeneratorOptions.OUTGOING_EAST_SIDE, Integer.valueOf(amount));
                  break;
                case NORTH:
                  _switchResult_2 = options.<Integer>setProperty(GeneratorOptions.OUTGOING_NORTH_SIDE, Integer.valueOf(amount));
                  break;
                case SOUTH:
                  _switchResult_2 = options.<Integer>setProperty(GeneratorOptions.OUTGOING_SOUTH_SIDE, Integer.valueOf(amount));
                  break;
                case WEST:
                  _switchResult_2 = options.<Integer>setProperty(GeneratorOptions.OUTGOING_WEST_SIDE, Integer.valueOf(amount));
                  break;
                default:
                  break;
              }
            }
            _switchResult = _switchResult_2;
            break;
          default:
            break;
        }
      }
      _xblockexpression = _switchResult;
    }
    return _xblockexpression;
  }
  
  private PortConstraints getConstraint(final ConstraintType constraint) {
    if (constraint != null) {
      switch (constraint) {
        case FREE:
          return PortConstraints.FREE;
        case ORDER:
          return PortConstraints.FIXED_ORDER;
        case POSITION:
          return PortConstraints.FIXED_POS;
        case SIDE:
          return PortConstraints.FIXED_SIDE;
        case RATIO:
          return PortConstraints.FIXED_RATIO;
        default:
          break;
      }
    }
    return PortConstraints.UNDEFINED;
  }
  
  private void setGraphType(final Configuration configuration, final GeneratorOptions options) {
    Form _form = configuration.getForm();
    if (_form != null) {
      switch (_form) {
        case ACYCLIC:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.ACYCLIC_NO_TRANSITIVE_EDGES);
          break;
        case BICONNECTED:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.BICONNECTED);
          break;
        case BIPARTITE:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.BIPARTITE);
          break;
        case CUSTOM:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.CUSTOM);
          break;
        case TREES:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.TREE);
          break;
        case TRICONNECTED:
          options.<GeneratorOptions.GraphType>setProperty(GeneratorOptions.GRAPH_TYPE, GeneratorOptions.GraphType.TRICONNECTED);
          break;
        default:
          break;
      }
    }
  }
  
  private void serialize(final KNode graph, final IFile file) throws IOException, CoreException {
    final ResourceSetImpl resourceSet = new ResourceSetImpl();
    URI _locationURI = file.getLocationURI();
    String _string = _locationURI.toString();
    org.eclipse.emf.common.util.URI _createURI = org.eclipse.emf.common.util.URI.createURI(_string);
    final Resource resource = resourceSet.createResource(_createURI);
    EList<EObject> _contents = resource.getContents();
    _contents.add(graph);
    resource.save(Collections.EMPTY_MAP);
    file.refreshLocal(1, null);
  }
  
  private boolean exists(final Object o) {
    return (!Objects.equal(o, null));
  }
  
  private <T extends Object> boolean setIfExists(final GeneratorOptions genOpt, final T value, final IProperty<T> property) {
    boolean _exists = this.exists(value);
    if (_exists) {
      genOpt.<T>setProperty(property, value);
      return true;
    }
    return false;
  }
}
