package de.cau.cs.kieler.kiml.config.text;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import org.eclipse.elk.core.LayoutConfigurator;
import org.eclipse.elk.core.data.LayoutMetaDataService;
import org.eclipse.elk.core.data.LayoutOptionData;
import org.eclipse.elk.core.klayoutdata.KIdentifier;
import org.eclipse.elk.graph.KEdge;
import org.eclipse.elk.graph.KGraphData;
import org.eclipse.elk.graph.KGraphElement;
import org.eclipse.elk.graph.KLabel;
import org.eclipse.elk.graph.KNode;
import org.eclipse.elk.graph.KPort;
import org.eclipse.elk.graph.PersistentEntry;
import org.eclipse.elk.graph.properties.IPropertyHolder;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Pair;

/**
 * Utility class for transformaing textually specified layout configurations
 * to the internal layout properties using KIML's {@link LayoutMetaDataService}.
 * 
 * @author uru
 */
@SuppressWarnings("all")
public final class LayoutConfigTransformer {
  /**
   * @param resource
   * 		a resource containing the textually specified layout configuration
   * @param additionalOptions
   * 		an optional list of layout options that should be added to the
   * 		internal configuration
   * @return a list of {@link VolatileLayoutConfig}s that represent
   * 		   the configurations specified in the passed resource.
   */
  public static List<LayoutConfigurator> from(final Resource resource, final Pair<String, Number>... additionalOptions) {
    final List<LayoutConfigurator> volatileConfigs = Lists.<LayoutConfigurator>newLinkedList();
    final LayoutMetaDataService dataService = LayoutMetaDataService.getInstance();
    EList<EObject> _contents = resource.getContents();
    Iterable<KNode> _filter = Iterables.<KNode>filter(_contents, KNode.class);
    final Consumer<KNode> _function = new Consumer<KNode>() {
      @Override
      public void accept(final KNode root) {
        EList<KGraphData> _data = root.getData();
        Iterable<KIdentifier> _filter = Iterables.<KIdentifier>filter(_data, KIdentifier.class);
        final Function1<KIdentifier, Boolean> _function = new Function1<KIdentifier, Boolean>() {
          @Override
          public Boolean apply(final KIdentifier it) {
            boolean _xblockexpression = false;
            {
              String _id = it.getId();
              InputOutput.<String>println(_id);
              String _id_1 = it.getId();
              boolean _startsWith = _id_1.startsWith("_");
              _xblockexpression = (!_startsWith);
            }
            return Boolean.valueOf(_xblockexpression);
          }
        };
        Iterable<KIdentifier> _filter_1 = IterableExtensions.<KIdentifier>filter(_filter, _function);
        final Consumer<KIdentifier> _function_1 = new Consumer<KIdentifier>() {
          @Override
          public void accept(final KIdentifier cfg) {
            final LayoutConfigurator currentConfig = LayoutConfigTransformer.transformConfig(cfg);
            volatileConfigs.add(currentConfig);
            final Consumer<Pair<String, Number>> _function = new Consumer<Pair<String, Number>>() {
              @Override
              public void accept(final Pair<String, Number> opt) {
                String _key = opt.getKey();
                LayoutOptionData _optionDataBySuffix = dataService.getOptionDataBySuffix(_key);
                final LayoutOptionData optData = ((LayoutOptionData) _optionDataBySuffix);
                boolean _notEquals = (!Objects.equal(optData, null));
                if (_notEquals) {
                  Number _value = opt.getValue();
                  String _string = _value.toString();
                  final Object value = optData.parseValue(_string);
                  boolean _notEquals_1 = (!Objects.equal(value, null));
                  if (_notEquals_1) {
                    IPropertyHolder _configure = currentConfig.configure(KGraphElement.class);
                    _configure.<Object>setProperty(optData, value);
                  }
                }
              }
            };
            ((List<Pair<String, Number>>)Conversions.doWrapArray(additionalOptions)).forEach(_function);
          }
        };
        _filter_1.forEach(_function_1);
      }
    };
    _filter.forEach(_function);
    return volatileConfigs;
  }
  
  public static LayoutConfigurator transformConfig(final KIdentifier cfg) {
    final LayoutConfigurator currentConfig = new LayoutConfigurator();
    EList<PersistentEntry> _persistentEntries = cfg.getPersistentEntries();
    final Consumer<PersistentEntry> _function = new Consumer<PersistentEntry>() {
      @Override
      public void accept(final PersistentEntry entry) {
        LayoutMetaDataService _instance = LayoutMetaDataService.getInstance();
        String _key = entry.getKey();
        LayoutOptionData _optionDataBySuffix = _instance.getOptionDataBySuffix(_key);
        final LayoutOptionData optData = ((LayoutOptionData) _optionDataBySuffix);
        boolean _notEquals = (!Objects.equal(optData, null));
        if (_notEquals) {
          String _value = entry.getValue();
          final Object value = optData.parseValue(_value);
          boolean _notEquals_1 = (!Objects.equal(value, null));
          if (_notEquals_1) {
            boolean _or = false;
            Set<LayoutOptionData.Target> _targets = optData.getTargets();
            boolean _contains = _targets.contains(LayoutOptionData.Target.NODES);
            if (_contains) {
              _or = true;
            } else {
              Set<LayoutOptionData.Target> _targets_1 = optData.getTargets();
              boolean _contains_1 = _targets_1.contains(LayoutOptionData.Target.PARENTS);
              _or = _contains_1;
            }
            if (_or) {
              IPropertyHolder _configure = currentConfig.configure(KNode.class);
              _configure.<Object>setProperty(optData, value);
            }
            Set<LayoutOptionData.Target> _targets_2 = optData.getTargets();
            boolean _contains_2 = _targets_2.contains(LayoutOptionData.Target.EDGES);
            if (_contains_2) {
              IPropertyHolder _configure_1 = currentConfig.configure(KEdge.class);
              _configure_1.<Object>setProperty(optData, value);
            }
            Set<LayoutOptionData.Target> _targets_3 = optData.getTargets();
            boolean _contains_3 = _targets_3.contains(LayoutOptionData.Target.PORTS);
            if (_contains_3) {
              IPropertyHolder _configure_2 = currentConfig.configure(KPort.class);
              _configure_2.<Object>setProperty(optData, value);
            }
            Set<LayoutOptionData.Target> _targets_4 = optData.getTargets();
            boolean _contains_4 = _targets_4.contains(LayoutOptionData.Target.LABELS);
            if (_contains_4) {
              IPropertyHolder _configure_3 = currentConfig.configure(KLabel.class);
              _configure_3.<Object>setProperty(optData, value);
            }
          }
        }
      }
    };
    _persistentEntries.forEach(_function);
    return currentConfig;
  }
}
