/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2014 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.kiml.formats.gml;

import org.eclipse.xtend.lib.annotations.Data;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * Abstract class for GML Elements. Every element
 * consists of a key and either a value or a collection of
 * further elements.
 * All valued elements have to implement a method that
 * returns the value in form of a String.
 */
@Data
@SuppressWarnings("all")
public abstract class Element {
  private final Element container;
  
  private final String key;
  
  public abstract String getValue();
  
  /**
   * Two element instances with the same key and
   * the same container are still different objects.
   */
  @Override
  public int hashCode() {
    return super.hashCode();
  }
  
  /**
   * Two element instances with the same key and
   * the same container are still different objects.
   */
  @Override
  public boolean equals(final Object obj) {
    return super.equals(obj);
  }
  
  public Element(final Element container, final String key) {
    super();
    this.container = container;
    this.key = key;
  }
  
  @Override
  @Pure
  public String toString() {
    ToStringBuilder b = new ToStringBuilder(this);
    b.add("container", this.container);
    b.add("key", this.key);
    return b.toString();
  }
  
  @Pure
  public Element getContainer() {
    return this.container;
  }
  
  @Pure
  public String getKey() {
    return this.key;
  }
}
