/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2014 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.kiml.formats.gml;

import de.cau.cs.kieler.kiml.formats.gml.Element;
import org.eclipse.xtend.lib.annotations.Data;
import org.eclipse.xtext.xbase.lib.Pure;
import org.eclipse.xtext.xbase.lib.util.ToStringBuilder;

/**
 * A NumberElement has either an integer
 * or double as value.
 */
@Data
@SuppressWarnings("all")
public class NumberElement extends Element {
  private final Number number;
  
  @Override
  public String getValue() {
    return this.number.toString();
  }
  
  public NumberElement(final Element container, final String key, final Number number) {
    super(container, key);
    this.number = number;
  }
  
  @Override
  @Pure
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + ((this.number== null) ? 0 : this.number.hashCode());
    return result;
  }
  
  @Override
  @Pure
  public boolean equals(final Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    if (!super.equals(obj))
      return false;
    NumberElement other = (NumberElement) obj;
    if (this.number == null) {
      if (other.number != null)
        return false;
    } else if (!this.number.equals(other.number))
      return false;
    return true;
  }
  
  @Override
  @Pure
  public String toString() {
    String result = new ToStringBuilder(this)
    	.addAllFields()
    	.toString();
    return result;
  }
  
  @Pure
  public Number getNumber() {
    return this.number;
  }
}
