/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.kiml.grana.text.formatting;

import com.google.inject.Inject;
import de.cau.cs.kieler.kiml.grana.text.services.GranaGrammarAccess;
import java.util.List;
import org.eclipse.xtext.Assignment;
import org.eclipse.xtext.Keyword;
import org.eclipse.xtext.ParserRule;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.TerminalRule;
import org.eclipse.xtext.formatting.impl.AbstractDeclarativeFormatter;
import org.eclipse.xtext.formatting.impl.FormattingConfig;
import org.eclipse.xtext.util.Pair;
import org.eclipse.xtext.xbase.lib.Extension;

/**
 * Formatter for textually specified Grana analyses.
 * 
 * @author uru
 */
@SuppressWarnings("all")
public class GranaFormatter extends AbstractDeclarativeFormatter {
  @Inject
  @Extension
  private GranaGrammarAccess _granaGrammarAccess;
  
  @Override
  protected void configureFormatting(final FormattingConfig c) {
    c.setAutoLinewrap(120);
    FormattingConfig.LinewrapLocator _setLinewrap = c.setLinewrap(0, 1, 2);
    TerminalRule _sL_COMMENTRule = this._granaGrammarAccess.getSL_COMMENTRule();
    _setLinewrap.before(_sL_COMMENTRule);
    FormattingConfig.LinewrapLocator _setLinewrap_1 = c.setLinewrap(0, 1, 2);
    TerminalRule _mL_COMMENTRule = this._granaGrammarAccess.getML_COMMENTRule();
    _setLinewrap_1.before(_mL_COMMENTRule);
    FormattingConfig.LinewrapLocator _setLinewrap_2 = c.setLinewrap(0, 1, 1);
    TerminalRule _mL_COMMENTRule_1 = this._granaGrammarAccess.getML_COMMENTRule();
    _setLinewrap_2.after(_mL_COMMENTRule_1);
    List<Pair<Keyword, Keyword>> _findKeywordPairs = this._granaGrammarAccess.findKeywordPairs("{", "}");
    for (final Pair<Keyword, Keyword> pair : _findKeywordPairs) {
      {
        Keyword _first = pair.getFirst();
        Keyword _second = pair.getSecond();
        c.setIndentation(_first, _second);
        FormattingConfig.LinewrapLocator _setLinewrap_3 = c.setLinewrap(1);
        Keyword _first_1 = pair.getFirst();
        _setLinewrap_3.after(_first_1);
        FormattingConfig.LinewrapLocator _setLinewrap_4 = c.setLinewrap(1);
        Keyword _second_1 = pair.getSecond();
        _setLinewrap_4.before(_second_1);
        FormattingConfig.LinewrapLocator _setLinewrap_5 = c.setLinewrap(1);
        Keyword _second_2 = pair.getSecond();
        _setLinewrap_5.after(_second_2);
      }
    }
    List<Pair<Keyword, Keyword>> _findKeywordPairs_1 = this._granaGrammarAccess.findKeywordPairs("[", "]");
    for (final Pair<Keyword, Keyword> pair_1 : _findKeywordPairs_1) {
      {
        FormattingConfig.NoSpaceLocator _setNoSpace = c.setNoSpace();
        Keyword _first = pair_1.getFirst();
        _setNoSpace.after(_first);
        FormattingConfig.NoSpaceLocator _setNoSpace_1 = c.setNoSpace();
        Keyword _second = pair_1.getSecond();
        _setNoSpace_1.before(_second);
      }
    }
    List<Pair<Keyword, Keyword>> _findKeywordPairs_2 = this._granaGrammarAccess.findKeywordPairs("(", ")");
    for (final Pair<Keyword, Keyword> pair_2 : _findKeywordPairs_2) {
      {
        FormattingConfig.NoSpaceLocator _setNoSpace = c.setNoSpace();
        Keyword _first = pair_2.getFirst();
        _setNoSpace.after(_first);
        FormattingConfig.NoSpaceLocator _setNoSpace_1 = c.setNoSpace();
        Keyword _second = pair_2.getSecond();
        _setNoSpace_1.before(_second);
      }
    }
    List<Keyword> _findKeywords = this._granaGrammarAccess.findKeywords(",");
    for (final Keyword comma : _findKeywords) {
      {
        FormattingConfig.NoLinewrapLocator _setNoLinewrap = c.setNoLinewrap();
        _setNoLinewrap.before(comma);
        FormattingConfig.NoSpaceLocator _setNoSpace = c.setNoSpace();
        _setNoSpace.before(comma);
      }
    }
    List<Keyword> _findKeywords_1 = this._granaGrammarAccess.findKeywords(":");
    for (final Keyword colon : _findKeywords_1) {
      {
        FormattingConfig.NoLinewrapLocator _setNoLinewrap = c.setNoLinewrap();
        _setNoLinewrap.before(colon);
        FormattingConfig.NoSpaceLocator _setNoSpace = c.setNoSpace();
        _setNoSpace.before(colon);
      }
    }
    List<Keyword> _findKeywords_2 = this._granaGrammarAccess.findKeywords("*");
    for (final Keyword star : _findKeywords_2) {
      {
        FormattingConfig.NoLinewrapLocator _setNoLinewrap = c.setNoLinewrap();
        _setNoLinewrap.after(star);
        FormattingConfig.NoSpaceLocator _setNoSpace = c.setNoSpace();
        _setNoSpace.after(star);
      }
    }
    FormattingConfig.LinewrapLocator _setLinewrap_3 = c.setLinewrap(2, 2, 2);
    ParserRule _jobRule = this._granaGrammarAccess.getJobRule();
    _setLinewrap_3.before(_jobRule);
    FormattingConfig.LinewrapLocator _setLinewrap_4 = c.setLinewrap();
    ParserRule _resourceRule = this._granaGrammarAccess.getResourceRule();
    _setLinewrap_4.after(_resourceRule);
    FormattingConfig.LinewrapLocator _setLinewrap_5 = c.setLinewrap();
    ParserRule _localResourceRule = this._granaGrammarAccess.getLocalResourceRule();
    _setLinewrap_5.after(_localResourceRule);
    FormattingConfig.LinewrapLocator _setLinewrap_6 = c.setLinewrap();
    ParserRule _resourceReferenceRule = this._granaGrammarAccess.getResourceReferenceRule();
    _setLinewrap_6.after(_resourceReferenceRule);
    FormattingConfig.LinewrapLocator _setLinewrap_7 = c.setLinewrap();
    ParserRule _localOutputRule = this._granaGrammarAccess.getLocalOutputRule();
    _setLinewrap_7.after(_localOutputRule);
    FormattingConfig.LinewrapLocator _setLinewrap_8 = c.setLinewrap();
    ParserRule _outputReferenceRule = this._granaGrammarAccess.getOutputReferenceRule();
    _setLinewrap_8.after(_outputReferenceRule);
    FormattingConfig.LinewrapLocator _setLinewrap_9 = c.setLinewrap();
    ParserRule _analysisRule = this._granaGrammarAccess.getAnalysisRule();
    _setLinewrap_9.after(_analysisRule);
    FormattingConfig.LinewrapLocator _setLinewrap_10 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RegularJobElements _regularJobAccess = this._granaGrammarAccess.getRegularJobAccess();
    Assignment _nameAssignment_1 = _regularJobAccess.getNameAssignment_1();
    _setLinewrap_10.after(_nameAssignment_1);
    FormattingConfig.LinewrapLocator _setLinewrap_11 = c.setLinewrap();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_1 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _layoutBeforeAnalysisLayoutBeforeAnalysisKeyword_2_0 = _regularJobAccess_1.getLayoutBeforeAnalysisLayoutBeforeAnalysisKeyword_2_0();
    _setLinewrap_11.after(_layoutBeforeAnalysisLayoutBeforeAnalysisKeyword_2_0);
    FormattingConfig.LinewrapLocator _setLinewrap_12 = c.setLinewrap();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_2 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _measureExecutionTimeMeasureExecutionTimeKeyword_3_0 = _regularJobAccess_2.getMeasureExecutionTimeMeasureExecutionTimeKeyword_3_0();
    _setLinewrap_12.after(_measureExecutionTimeMeasureExecutionTimeKeyword_3_0);
    FormattingConfig.LinewrapLocator _setLinewrap_13 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RegularJobElements _regularJobAccess_3 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _resourcesKeyword_4 = _regularJobAccess_3.getResourcesKeyword_4();
    _setLinewrap_13.after(_resourcesKeyword_4);
    FormattingConfig.LinewrapLocator _setLinewrap_14 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RegularJobElements _regularJobAccess_4 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _layoutoptionsKeyword_6 = _regularJobAccess_4.getLayoutoptionsKeyword_6();
    _setLinewrap_14.after(_layoutoptionsKeyword_6);
    FormattingConfig.LinewrapLocator _setLinewrap_15 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RegularJobElements _regularJobAccess_5 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _analysesKeyword_8 = _regularJobAccess_5.getAnalysesKeyword_8();
    _setLinewrap_15.after(_analysesKeyword_8);
    FormattingConfig.LinewrapLocator _setLinewrap_16 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess = this._granaGrammarAccess.getRangeJobAccess();
    Assignment _nameAssignment_1_1 = _rangeJobAccess.getNameAssignment_1();
    _setLinewrap_16.after(_nameAssignment_1_1);
    FormattingConfig.LinewrapLocator _setLinewrap_17 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_1 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _resourcesKeyword_2 = _rangeJobAccess_1.getResourcesKeyword_2();
    _setLinewrap_17.after(_resourcesKeyword_2);
    FormattingConfig.LinewrapLocator _setLinewrap_18 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_2 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _layoutoptionsKeyword_4 = _rangeJobAccess_2.getLayoutoptionsKeyword_4();
    _setLinewrap_18.after(_layoutoptionsKeyword_4);
    FormattingConfig.LinewrapLocator _setLinewrap_19 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_3 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _analysesKeyword_6 = _rangeJobAccess_3.getAnalysesKeyword_6();
    _setLinewrap_19.after(_analysesKeyword_6);
    FormattingConfig.LinewrapLocator _setLinewrap_20 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_4 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeoptionKeyword_8 = _rangeJobAccess_4.getRangeoptionKeyword_8();
    _setLinewrap_20.after(_rangeoptionKeyword_8);
    FormattingConfig.LinewrapLocator _setLinewrap_21 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_5 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeanalysisKeyword_11 = _rangeJobAccess_5.getRangeanalysisKeyword_11();
    _setLinewrap_21.after(_rangeanalysisKeyword_11);
    FormattingConfig.LinewrapLocator _setLinewrap_22 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_6 = this._granaGrammarAccess.getRangeJobAccess();
    Assignment _rangeOptionAssignment_9 = _rangeJobAccess_6.getRangeOptionAssignment_9();
    _setLinewrap_22.after(_rangeOptionAssignment_9);
    FormattingConfig.LinewrapLocator _setLinewrap_23 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_7 = this._granaGrammarAccess.getRangeJobAccess();
    RuleCall _rangeAnalysisComponentNATURALTerminalRuleCall_13_1_0 = _rangeJobAccess_7.getRangeAnalysisComponentNATURALTerminalRuleCall_13_1_0();
    _setLinewrap_23.after(_rangeAnalysisComponentNATURALTerminalRuleCall_13_1_0);
    FormattingConfig.LinewrapLocator _setLinewrap_24 = c.setLinewrap(1, 1, 2);
    GranaGrammarAccess.RangeElements _rangeAccess = this._granaGrammarAccess.getRangeAccess();
    ParserRule _rule = _rangeAccess.getRule();
    _setLinewrap_24.after(_rule);
    FormattingConfig.LinewrapLocator _setLinewrap_25 = c.setLinewrap();
    GranaGrammarAccess.GranaElements _granaAccess = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalResourcesKeyword_0_0 = _granaAccess.getGlobalResourcesKeyword_0_0();
    _setLinewrap_25.after(_globalResourcesKeyword_0_0);
    FormattingConfig.LinewrapLocator _setLinewrap_26 = c.setLinewrap();
    GranaGrammarAccess.GranaElements _granaAccess_1 = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalOutputsKeyword_1_0 = _granaAccess_1.getGlobalOutputsKeyword_1_0();
    _setLinewrap_26.after(_globalOutputsKeyword_1_0);
    FormattingConfig.LinewrapLocator _setLinewrap_27 = c.setLinewrap();
    ParserRule _globalResourceRefRule = this._granaGrammarAccess.getGlobalResourceRefRule();
    _setLinewrap_27.after(_globalResourceRefRule);
    FormattingConfig.LinewrapLocator _setLinewrap_28 = c.setLinewrap();
    ParserRule _globalOutputRefRule = this._granaGrammarAccess.getGlobalOutputRefRule();
    _setLinewrap_28.after(_globalOutputRefRule);
    FormattingConfig.LinewrapLocator _setLinewrap_29 = c.setLinewrap(2, 2, 2);
    GranaGrammarAccess.GranaElements _granaAccess_2 = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalOutputsKeyword_1_0_1 = _granaAccess_2.getGlobalOutputsKeyword_1_0();
    _setLinewrap_29.before(_globalOutputsKeyword_1_0_1);
    FormattingConfig.LinewrapLocator _setLinewrap_30 = c.setLinewrap(2, 2, 2);
    GranaGrammarAccess.GranaElements _granaAccess_3 = this._granaGrammarAccess.getGranaAccess();
    Keyword _executeKeyword_2_0 = _granaAccess_3.getExecuteKeyword_2_0();
    _setLinewrap_30.before(_executeKeyword_2_0);
    FormattingConfig.LinewrapLocator _setLinewrap_31 = c.setLinewrap();
    GranaGrammarAccess.PersistentEntryElements _persistentEntryAccess = this._granaGrammarAccess.getPersistentEntryAccess();
    Assignment _valueAssignment_2 = _persistentEntryAccess.getValueAssignment_2();
    _setLinewrap_31.after(_valueAssignment_2);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement = c.setIndentationIncrement();
    GranaGrammarAccess.GranaElements _granaAccess_4 = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalResourcesKeyword_0_0_1 = _granaAccess_4.getGlobalResourcesKeyword_0_0();
    _setIndentationIncrement.after(_globalResourcesKeyword_0_0_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_1 = c.setIndentationIncrement();
    GranaGrammarAccess.GranaElements _granaAccess_5 = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalOutputsKeyword_1_0_2 = _granaAccess_5.getGlobalOutputsKeyword_1_0();
    _setIndentationIncrement_1.after(_globalOutputsKeyword_1_0_2);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_2 = c.setIndentationIncrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_6 = this._granaGrammarAccess.getRegularJobAccess();
    Assignment _nameAssignment_1_2 = _regularJobAccess_6.getNameAssignment_1();
    _setIndentationIncrement_2.after(_nameAssignment_1_2);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_3 = c.setIndentationIncrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_7 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _resourcesKeyword_4_1 = _regularJobAccess_7.getResourcesKeyword_4();
    _setIndentationIncrement_3.after(_resourcesKeyword_4_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_4 = c.setIndentationIncrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_8 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _analysesKeyword_8_1 = _regularJobAccess_8.getAnalysesKeyword_8();
    _setIndentationIncrement_4.after(_analysesKeyword_8_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_5 = c.setIndentationIncrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_9 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _layoutoptionsKeyword_6_1 = _regularJobAccess_9.getLayoutoptionsKeyword_6();
    _setIndentationIncrement_5.after(_layoutoptionsKeyword_6_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_6 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_8 = this._granaGrammarAccess.getRangeJobAccess();
    Assignment _nameAssignment_1_3 = _rangeJobAccess_8.getNameAssignment_1();
    _setIndentationIncrement_6.after(_nameAssignment_1_3);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_7 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_9 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _resourcesKeyword_2_1 = _rangeJobAccess_9.getResourcesKeyword_2();
    _setIndentationIncrement_7.after(_resourcesKeyword_2_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_8 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_10 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _analysesKeyword_6_1 = _rangeJobAccess_10.getAnalysesKeyword_6();
    _setIndentationIncrement_8.after(_analysesKeyword_6_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_9 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_11 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _layoutoptionsKeyword_4_1 = _rangeJobAccess_11.getLayoutoptionsKeyword_4();
    _setIndentationIncrement_9.after(_layoutoptionsKeyword_4_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_10 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_12 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeoptionKeyword_8_1 = _rangeJobAccess_12.getRangeoptionKeyword_8();
    _setIndentationIncrement_10.after(_rangeoptionKeyword_8_1);
    FormattingConfig.IndentationLocatorStart _setIndentationIncrement_11 = c.setIndentationIncrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_13 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeanalysisKeyword_11_1 = _rangeJobAccess_13.getRangeanalysisKeyword_11();
    _setIndentationIncrement_11.after(_rangeanalysisKeyword_11_1);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement = c.setIndentationDecrement();
    GranaGrammarAccess.GranaElements _granaAccess_6 = this._granaGrammarAccess.getGranaAccess();
    Keyword _globalOutputsKeyword_1_0_3 = _granaAccess_6.getGlobalOutputsKeyword_1_0();
    _setIndentationDecrement.before(_globalOutputsKeyword_1_0_3);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_1 = c.setIndentationDecrement();
    GranaGrammarAccess.GranaElements _granaAccess_7 = this._granaGrammarAccess.getGranaAccess();
    Keyword _executeKeyword_2_0_1 = _granaAccess_7.getExecuteKeyword_2_0();
    _setIndentationDecrement_1.before(_executeKeyword_2_0_1);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_2 = c.setIndentationDecrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_10 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _layoutoptionsKeyword_6_2 = _regularJobAccess_10.getLayoutoptionsKeyword_6();
    _setIndentationDecrement_2.before(_layoutoptionsKeyword_6_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_3 = c.setIndentationDecrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_11 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _analysesKeyword_8_2 = _regularJobAccess_11.getAnalysesKeyword_8();
    _setIndentationDecrement_3.before(_analysesKeyword_8_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_4 = c.setIndentationDecrement();
    GranaGrammarAccess.RegularJobElements _regularJobAccess_12 = this._granaGrammarAccess.getRegularJobAccess();
    Keyword _outputKeyword_10 = _regularJobAccess_12.getOutputKeyword_10();
    _setIndentationDecrement_4.before(_outputKeyword_10);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_5 = c.setIndentationDecrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_14 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _layoutoptionsKeyword_4_2 = _rangeJobAccess_14.getLayoutoptionsKeyword_4();
    _setIndentationDecrement_5.before(_layoutoptionsKeyword_4_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_6 = c.setIndentationDecrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_15 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _analysesKeyword_6_2 = _rangeJobAccess_15.getAnalysesKeyword_6();
    _setIndentationDecrement_6.before(_analysesKeyword_6_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_7 = c.setIndentationDecrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_16 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _outputKeyword_14 = _rangeJobAccess_16.getOutputKeyword_14();
    _setIndentationDecrement_7.before(_outputKeyword_14);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_8 = c.setIndentationDecrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_17 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeoptionKeyword_8_2 = _rangeJobAccess_17.getRangeoptionKeyword_8();
    _setIndentationDecrement_8.before(_rangeoptionKeyword_8_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_9 = c.setIndentationDecrement();
    GranaGrammarAccess.RangeJobElements _rangeJobAccess_18 = this._granaGrammarAccess.getRangeJobAccess();
    Keyword _rangeanalysisKeyword_11_2 = _rangeJobAccess_18.getRangeanalysisKeyword_11();
    _setIndentationDecrement_9.before(_rangeanalysisKeyword_11_2);
    FormattingConfig.IndentationLocatorEnd _setIndentationDecrement_10 = c.setIndentationDecrement();
    ParserRule _outputRule = this._granaGrammarAccess.getOutputRule();
    _setIndentationDecrement_10.after(_outputRule);
  }
}
