/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.kiml.grana.text.ui.contentassist;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import de.cau.cs.kieler.kiml.grana.AnalysisData;
import de.cau.cs.kieler.kiml.grana.AnalysisService;
import de.cau.cs.kieler.kiml.grana.text.grana.Analysis;
import de.cau.cs.kieler.kiml.grana.text.grana.RangeJob;
import de.cau.cs.kieler.kiml.grana.text.services.GranaGrammarAccess;
import de.cau.cs.kieler.kiml.grana.text.ui.contentassist.AbstractGranaProposalProvider;
import java.io.File;
import java.util.Collection;
import java.util.List;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.elk.core.data.LayoutAlgorithmData;
import org.eclipse.elk.core.data.LayoutMetaDataService;
import org.eclipse.elk.core.data.LayoutOptionData;
import org.eclipse.elk.core.options.CoreOptions;
import org.eclipse.elk.core.ui.LayoutOptionLabelProvider;
import org.eclipse.elk.core.util.Pair;
import org.eclipse.elk.graph.PersistentEntry;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.swt.graphics.Image;
import org.eclipse.xtext.Assignment;
import org.eclipse.xtext.ParserRule;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.conversion.IValueConverterService;
import org.eclipse.xtext.ui.editor.contentassist.ConfigurableCompletionProposal;
import org.eclipse.xtext.ui.editor.contentassist.ContentAssistContext;
import org.eclipse.xtext.ui.editor.contentassist.ICompletionProposalAcceptor;
import org.eclipse.xtext.ui.editor.contentassist.IContentProposalPriorities;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;

/**
 * see http://www.eclipse.org/Xtext/documentation.html#contentAssist on how to customize content assistant
 * 
 * @author uru
 */
@SuppressWarnings("all")
public class GranaProposalProvider extends AbstractGranaProposalProvider {
  @Inject
  private GranaGrammarAccess grammarAccess;
  
  /**
   * Proposals for graph analyses.
   */
  @Override
  public void completeAnalysis_Name(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    super.completeAnalysis_Name(model, assignment, context, acceptor);
    AnalysisService _instance = AnalysisService.getInstance();
    Collection<AnalysisData> _analyses = _instance.getAnalyses();
    for (final AnalysisData a : _analyses) {
      String _id = a.getId();
      String _prefix = context.getPrefix();
      boolean _isValidProposal = this.isValidProposal(_id, _prefix, context);
      if (_isValidProposal) {
        String _name = a.getName();
        final StyledString displayString = new StyledString(_name);
        String _id_1 = a.getId();
        IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
        int _defaultPriority = _priorityHelper.getDefaultPriority();
        int _plus = (_defaultPriority + 1);
        ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(_id_1, displayString, null, _plus, context);
        acceptor.accept(_doCreateProposal);
      }
    }
  }
  
  /**
   * Proposals for layout option keys.
   */
  @Override
  public void completePersistentEntry_Key(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    this.handleLayoutOptionProposal(model, assignment, context, acceptor);
  }
  
  @Override
  public void completeRangeJob_RangeOption(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    this.handleLayoutOptionProposal(model, assignment, context, acceptor);
  }
  
  private void handleLayoutOptionProposal(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    LayoutMetaDataService _instance = LayoutMetaDataService.getInstance();
    Collection<LayoutOptionData> _optionData = _instance.getOptionData();
    for (final LayoutOptionData optionData : _optionData) {
      {
        String _string = optionData.toString();
        StyledString.Styler _xifexpression = null;
        LayoutOptionData.Visibility _visibility = optionData.getVisibility();
        boolean _equals = Objects.equal(_visibility, LayoutOptionData.Visibility.ADVANCED);
        if (_equals) {
          _xifexpression = StyledString.COUNTER_STYLER;
        } else {
          _xifexpression = null;
        }
        final StyledString displayString = new StyledString(_string, _xifexpression);
        String _id = optionData.getId();
        String _plus = (" (" + _id);
        String _plus_1 = (_plus + ")");
        displayString.append(_plus_1, StyledString.QUALIFIER_STYLER);
        IValueConverterService _valueConverter = this.getValueConverter();
        String _id_1 = optionData.getId();
        ParserRule _qualifiedIDRule = this.grammarAccess.getQualifiedIDRule();
        String _name = _qualifiedIDRule.getName();
        final String proposal = _valueConverter.toString(_id_1, _name);
        final LayoutOptionLabelProvider labelProvider = new LayoutOptionLabelProvider(optionData);
        Object _default = optionData.getDefault();
        final Image image = labelProvider.getImage(_default);
        String _id_2 = optionData.getId();
        this.handleKeyProposal(context, acceptor, _id_2, proposal, displayString, image);
      }
    }
  }
  
  /**
   * Copied from {@link KGraphProposalProvider}.
   */
  private void handleKeyProposal(final ContentAssistContext context, final ICompletionProposalAcceptor acceptor, final String id, final String proposal, final StyledString displayString, final Image image) {
    String _prefix = context.getPrefix();
    boolean _isValidProposal = this.isValidProposal(proposal, _prefix, context);
    if (_isValidProposal) {
      IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
      int _defaultPriority = _priorityHelper.getDefaultPriority();
      ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(proposal, displayString, image, _defaultPriority, context);
      acceptor.accept(_doCreateProposal);
    } else {
      final int lastDotIndex = id.lastIndexOf(".");
      if ((lastDotIndex >= 0)) {
        String _substring = id.substring(0, (lastDotIndex + 1));
        final StringBuilder prefix = new StringBuilder(_substring);
        String _prefix_1 = context.getPrefix();
        prefix.append(_prefix_1);
        int i = 0;
        while (((i < proposal.length()) && (i < prefix.length()))) {
          {
            char _charAt = proposal.charAt(i);
            char _charAt_1 = prefix.charAt(i);
            boolean _notEquals = (_charAt != _charAt_1);
            if (_notEquals) {
              char _charAt_2 = proposal.charAt(i);
              boolean _equals = Objects.equal(Character.valueOf(_charAt_2), "^");
              if (_equals) {
                prefix.insert(i, "^");
              } else {
                int _length = proposal.length();
                i = _length;
              }
            }
            i = (i + 1);
          }
        }
        String _string = prefix.toString();
        boolean _isValidProposal_1 = this.isValidProposal(proposal, _string, context);
        if (_isValidProposal_1) {
          IContentProposalPriorities _priorityHelper_1 = this.getPriorityHelper();
          int _defaultPriority_1 = _priorityHelper_1.getDefaultPriority();
          ConfigurableCompletionProposal _doCreateProposal_1 = this.doCreateProposal(proposal, displayString, image, _defaultPriority_1, context);
          acceptor.accept(_doCreateProposal_1);
        }
      }
    }
  }
  
  @Override
  public void completePersistentEntry_Value(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    super.completePersistentEntry_Value(model, assignment, context, acceptor);
    this.valueProposal(context, acceptor);
  }
  
  /**
   * Copied from {@link KGraphProposalProvider}.
   * 
   * Computes the annotation value proposals based on a foregoing layout parameter.
   * 
   * @param context Xtext API
   * @param acceptor Xtext API
   * 
   * @author sgu, chsch
   */
  private void valueProposal(final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    EObject _currentModel = context.getCurrentModel();
    if ((_currentModel instanceof PersistentEntry)) {
      EObject _currentModel_1 = context.getCurrentModel();
      final String annotationName = ((PersistentEntry) _currentModel_1).getKey();
      boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(annotationName);
      boolean _not = (!_isNullOrEmpty);
      if (_not) {
        final LayoutMetaDataService layoutServices = LayoutMetaDataService.getInstance();
        LayoutOptionData optionData = layoutServices.getOptionData(annotationName);
        boolean _equals = Objects.equal(optionData, null);
        if (_equals) {
          LayoutOptionData _optionData = layoutServices.getOptionData(("de.cau.cs.kieler." + annotationName));
          optionData = _optionData;
        }
        LayoutOptionData.Type _xifexpression = null;
        boolean _notEquals = (!Objects.equal(optionData, null));
        if (_notEquals) {
          _xifexpression = optionData.getType();
        } else {
          _xifexpression = LayoutOptionData.Type.UNDEFINED;
        }
        final LayoutOptionData.Type theType = _xifexpression;
        String proposal = null;
        boolean _matched = false;
        if (!_matched) {
          if (Objects.equal(theType, LayoutOptionData.Type.BOOLEAN)) {
            _matched=true;
          }
          if (!_matched) {
            if (Objects.equal(theType, LayoutOptionData.Type.ENUM)) {
              _matched=true;
            }
          }
          if (!_matched) {
            if (Objects.equal(theType, LayoutOptionData.Type.ENUMSET)) {
              _matched=true;
            }
          }
          if (_matched) {
            String[] _choices = optionData.getChoices();
            int _length = _choices.length;
            ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, _length, true);
            for (final Integer j : _doubleDotLessThan) {
              {
                String[] _choices_1 = optionData.getChoices();
                String _get = _choices_1[(j).intValue()];
                proposal = _get;
                ICompletionProposal _createCompletionProposal = this.createCompletionProposal(proposal, context);
                acceptor.accept(_createCompletionProposal);
              }
            }
          }
        }
        if (!_matched) {
          boolean _and = false;
          boolean _equals_1 = Objects.equal(theType, LayoutOptionData.Type.STRING);
          if (!_equals_1) {
            _and = false;
          } else {
            String _id = CoreOptions.ALGORITHM.getId();
            boolean _equals_2 = annotationName.equals(_id);
            _and = _equals_2;
          }
          if (_and) {
            _matched=true;
            String displayString = null;
            Collection<LayoutAlgorithmData> _algorithmData = layoutServices.getAlgorithmData();
            for (final LayoutAlgorithmData data : _algorithmData) {
              {
                String _id_1 = data.getId();
                String _plus = ("\"" + _id_1);
                String _plus_1 = (_plus + "\"");
                proposal = _plus_1;
                String _name = data.getName();
                displayString = _name;
                ICompletionProposal _createCompletionProposal = this.createCompletionProposal(proposal, displayString, null, context);
                acceptor.accept(_createCompletionProposal);
              }
            }
          }
        }
        if (!_matched) {
          if (Objects.equal(theType, LayoutOptionData.Type.STRING)) {
            _matched=true;
          }
          if (!_matched) {
            if (Objects.equal(theType, LayoutOptionData.Type.FLOAT)) {
              _matched=true;
            }
          }
          if (!_matched) {
            if (Objects.equal(theType, LayoutOptionData.Type.INT)) {
              _matched=true;
            }
          }
          if (!_matched) {
            if (Objects.equal(theType, LayoutOptionData.Type.OBJECT)) {
              _matched=true;
            }
          }
          if (_matched) {
            if (theType != null) {
              switch (theType) {
                case STRING:
                  proposal = "\"\"";
                  break;
                case FLOAT:
                  proposal = "0.0";
                  break;
                case INT:
                  proposal = "0";
                  break;
                case OBJECT:
                  try {
                    Class<?> _optionClass = optionData.getOptionClass();
                    Object _newInstance = _optionClass.newInstance();
                    String _string = _newInstance.toString();
                    String _plus = ("\"" + _string);
                    String _plus_1 = (_plus + "\"");
                    proposal = _plus_1;
                  } catch (final Throwable _t) {
                    if (_t instanceof InstantiationException) {
                      final InstantiationException e = (InstantiationException)_t;
                      proposal = "\"\"";
                    } else if (_t instanceof IllegalAccessException) {
                      final IllegalAccessException e_1 = (IllegalAccessException)_t;
                      proposal = "\"\"";
                    } else {
                      throw Exceptions.sneakyThrow(_t);
                    }
                  }
                  break;
                default:
                  proposal = "";
                  break;
              }
            } else {
              proposal = "";
            }
            LayoutOptionData.Type _type = optionData.getType();
            String _string_1 = _type.toString();
            ICompletionProposal _createCompletionProposal = this.createCompletionProposal(proposal, _string_1, null, context);
            acceptor.accept(_createCompletionProposal);
          }
        }
      }
    }
  }
  
  private final String FS_PREFIX = "file://";
  
  private final String SUG_WORKSPACEROOT = "\"/";
  
  private final StyledString DSP_WORKSPACE_ROOT = new StyledString("Workspace Root");
  
  private final String SUG_FILESYSTEM = ("\"" + this.FS_PREFIX);
  
  private final StyledString DSP_FILESYSTEM = new StyledString("Filesystem");
  
  @Override
  public void completeLocalResource_Path(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    super.completeLocalResource_Path(model, assignment, context, acceptor);
    String _prefix = context.getPrefix();
    final String stripped = _prefix.replaceAll("^\"|\"$", "");
    boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(stripped);
    if (_isNullOrEmpty) {
      IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
      int _defaultPriority = _priorityHelper.getDefaultPriority();
      int _plus = (_defaultPriority + 1);
      ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(this.SUG_WORKSPACEROOT, this.DSP_WORKSPACE_ROOT, 
        null, _plus, context);
      acceptor.accept(_doCreateProposal);
      IContentProposalPriorities _priorityHelper_1 = this.getPriorityHelper();
      int _defaultPriority_1 = _priorityHelper_1.getDefaultPriority();
      int _plus_1 = (_defaultPriority_1 + 1);
      ConfigurableCompletionProposal _doCreateProposal_1 = this.doCreateProposal(this.SUG_FILESYSTEM, this.DSP_FILESYSTEM, 
        null, _plus_1, context);
      acceptor.accept(_doCreateProposal_1);
      return;
    }
    final boolean isFs = stripped.startsWith(this.FS_PREFIX);
    if (isFs) {
      this.filesystemSuggestions(context, acceptor, false);
    } else {
      this.workspaceSuggestions(context, acceptor, false);
    }
  }
  
  @Override
  public void complete_LocalOutput(final EObject model, final RuleCall ruleCall, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    super.complete_LocalOutput(model, ruleCall, context, acceptor);
    String _prefix = context.getPrefix();
    final String stripped = _prefix.replaceAll("^\"|\"$", "");
    boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(stripped);
    if (_isNullOrEmpty) {
      IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
      int _defaultPriority = _priorityHelper.getDefaultPriority();
      int _plus = (_defaultPriority + 1);
      ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(this.SUG_WORKSPACEROOT, this.DSP_WORKSPACE_ROOT, 
        null, _plus, context);
      acceptor.accept(_doCreateProposal);
      IContentProposalPriorities _priorityHelper_1 = this.getPriorityHelper();
      int _defaultPriority_1 = _priorityHelper_1.getDefaultPriority();
      int _plus_1 = (_defaultPriority_1 + 1);
      ConfigurableCompletionProposal _doCreateProposal_1 = this.doCreateProposal(this.SUG_FILESYSTEM, this.DSP_FILESYSTEM, 
        null, _plus_1, context);
      acceptor.accept(_doCreateProposal_1);
      return;
    }
    final boolean isFs = stripped.startsWith(this.FS_PREFIX);
    if (isFs) {
      this.filesystemSuggestions(context, acceptor, true);
    } else {
      this.workspaceSuggestions(context, acceptor, true);
    }
  }
  
  private void filesystemSuggestions(final ContentAssistContext context, final ICompletionProposalAcceptor acceptor, final boolean acceptFiles) {
    String _prefix = context.getPrefix();
    final String stripped = _prefix.replaceAll("^\"|\"$", "");
    int _length = this.FS_PREFIX.length();
    final String woFile = stripped.substring(_length);
    final int sepIndex = woFile.lastIndexOf("/");
    boolean _or = false;
    int _length_1 = woFile.length();
    boolean _equals = (_length_1 == 0);
    if (_equals) {
      _or = true;
    } else {
      _or = (sepIndex == (-1));
    }
    if (_or) {
      File[] _listRoots = File.listRoots();
      for (final File root : _listRoots) {
        {
          String _path = root.getPath();
          final String suggestion = _path.replaceAll("\\\\", "/");
          final StyledString displayString = new StyledString(suggestion);
          String quotedSuggestion = (("\"" + this.FS_PREFIX) + suggestion);
          IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
          int _defaultPriority = _priorityHelper.getDefaultPriority();
          int _plus = (_defaultPriority + 0);
          ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(quotedSuggestion, displayString, 
            null, _plus, context);
          acceptor.accept(_doCreateProposal);
        }
      }
      return;
    }
    final String woFilePrefix = woFile.substring(0, (sepIndex + 1));
    final File file = new File(woFilePrefix);
    boolean _and = false;
    boolean _exists = file.exists();
    if (!_exists) {
      _and = false;
    } else {
      boolean _isDirectory = file.isDirectory();
      _and = _isDirectory;
    }
    if (_and) {
      File[] _listFiles = file.listFiles();
      final Function1<File, Boolean> _function = new Function1<File, Boolean>() {
        @Override
        public Boolean apply(final File it) {
          boolean _or = false;
          if (acceptFiles) {
            _or = true;
          } else {
            boolean _isDirectory = it.isDirectory();
            _or = _isDirectory;
          }
          return Boolean.valueOf(_or);
        }
      };
      Iterable<File> _filter = IterableExtensions.<File>filter(((Iterable<File>)Conversions.doWrapArray(_listFiles)), _function);
      for (final File f : _filter) {
        {
          String _path = f.getPath();
          final String suggestion = _path.replaceAll("\\\\", "/");
          boolean _isValidProposal = this.isValidProposal(suggestion, woFile, context);
          if (_isValidProposal) {
            final StyledString displayString = new StyledString(suggestion);
            String quotedSuggestion = (("\"" + this.FS_PREFIX) + suggestion);
            int priority = 0;
            boolean _isDirectory_1 = f.isDirectory();
            if (_isDirectory_1) {
              quotedSuggestion = (quotedSuggestion + "/");
              priority = 1;
            }
            IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
            int _defaultPriority = _priorityHelper.getDefaultPriority();
            int _plus = (_defaultPriority + priority);
            ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(quotedSuggestion, displayString, 
              null, _plus, context);
            acceptor.accept(_doCreateProposal);
          }
        }
      }
    }
  }
  
  private void workspaceSuggestions(final ContentAssistContext context, final ICompletionProposalAcceptor acceptor, final boolean acceptFiles) {
    try {
      IWorkspace _workspace = ResourcesPlugin.getWorkspace();
      final IWorkspaceRoot wsRoot = _workspace.getRoot();
      String _prefix = context.getPrefix();
      final String stripped = _prefix.replaceAll("^\"|\"$", "");
      final int sepIndex = stripped.lastIndexOf("/");
      String _xifexpression = null;
      boolean _and = false;
      if (!(sepIndex != (-1))) {
        _and = false;
      } else {
        int _length = stripped.length();
        boolean _greaterThan = (_length > 0);
        _and = _greaterThan;
      }
      if (_and) {
        _xifexpression = stripped.substring(0, (sepIndex + 1));
      } else {
        _xifexpression = "/";
      }
      final String prefix = _xifexpression;
      final IResource container = wsRoot.findMember(prefix);
      boolean _and_1 = false;
      boolean _notEquals = (!Objects.equal(container, null));
      if (!_notEquals) {
        _and_1 = false;
      } else {
        _and_1 = (container instanceof IContainer);
      }
      if (_and_1) {
        IResource[] _members = ((IContainer) container).members();
        for (final IResource member : _members) {
          boolean _or = false;
          if (acceptFiles) {
            _or = true;
          } else {
            _or = (member instanceof IContainer);
          }
          if (_or) {
            IPath _fullPath = member.getFullPath();
            final String suggestion = _fullPath.toString();
            boolean _isValidProposal = this.isValidProposal(suggestion, stripped, context);
            if (_isValidProposal) {
              final StyledString displayString = new StyledString(suggestion);
              String quotedSuggestion = ("\"" + suggestion);
              int priority = 0;
              if ((member instanceof IContainer)) {
                quotedSuggestion = (quotedSuggestion + "/");
                priority = 1;
              }
              IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
              int _defaultPriority = _priorityHelper.getDefaultPriority();
              int _plus = (_defaultPriority + priority);
              ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(quotedSuggestion, displayString, 
                null, _plus, context);
              acceptor.accept(_doCreateProposal);
            }
          }
        }
      }
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  @Override
  public void completeRangeJob_RangeAnalysisComponent(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    super.completeRangeJob_RangeAnalysisComponent(model, assignment, context, acceptor);
    if ((model instanceof RangeJob)) {
      final Analysis analysis = ((RangeJob) model).getRangeAnalysis();
      boolean _notEquals = (!Objects.equal(analysis, null));
      if (_notEquals) {
        AnalysisService _instance = AnalysisService.getInstance();
        String _name = analysis.getName();
        final AnalysisData analysisData = _instance.getAnalysis(_name);
        boolean _notEquals_1 = (!Objects.equal(analysisData, null));
        if (_notEquals_1) {
          List<Pair<String, String>> _components = analysisData.getComponents();
          InputOutput.<List<Pair<String, String>>>println(_components);
          int i = 0;
          List<Pair<String, String>> _components_1 = analysisData.getComponents();
          for (final Pair<String, String> c : _components_1) {
            {
              final String suggestion = (Integer.valueOf(i) + "");
              String _plus = (Integer.valueOf(i) + " - ");
              String _first = c.getFirst();
              String _plus_1 = (_plus + _first);
              String _plus_2 = (_plus_1 + " ");
              String _second = c.getSecond();
              String _plus_3 = (_plus_2 + _second);
              final StyledString displayString = new StyledString(_plus_3);
              IContentProposalPriorities _priorityHelper = this.getPriorityHelper();
              int _defaultPriority = _priorityHelper.getDefaultPriority();
              ConfigurableCompletionProposal _doCreateProposal = this.doCreateProposal(suggestion, displayString, null, _defaultPriority, context);
              acceptor.accept(_doCreateProposal);
              i++;
            }
          }
        }
      }
    }
  }
}
