package de.cau.cs.kieler.klighd.examples.dependencies;

import com.google.common.collect.ImmutableList;
import com.google.inject.Inject;
import de.cau.cs.kieler.klighd.SynthesisOption;
import de.cau.cs.kieler.klighd.examples.dependencies.ProjectDependencyInformation;
import de.cau.cs.kieler.klighd.krendering.KAreaPlacementData;
import de.cau.cs.kieler.klighd.krendering.KColor;
import de.cau.cs.kieler.klighd.krendering.KPolyline;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KColorExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KEdgeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KNodeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KPolylineExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import java.util.List;
import java.util.function.Consumer;
import org.eclipse.core.resources.IProject;
import org.eclipse.elk.alg.layered.properties.LayeredOptions;
import org.eclipse.elk.core.options.CoreOptions;
import org.eclipse.elk.core.options.EdgeRouting;
import org.eclipse.elk.core.util.Pair;
import org.eclipse.elk.graph.KEdge;
import org.eclipse.elk.graph.KNode;
import org.eclipse.elk.graph.properties.IProperty;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Very basic synthesis that presents a dependency graph for the passed project information.
 * 
 * @author uru
 */
@SuppressWarnings("all")
public class ProjectDependencySynthesis extends AbstractDiagramSynthesis<ProjectDependencyInformation> {
  @Inject
  @Extension
  private KNodeExtensions _kNodeExtensions;
  
  @Inject
  @Extension
  private KEdgeExtensions _kEdgeExtensions;
  
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  @Inject
  @Extension
  private KPolylineExtensions _kPolylineExtensions;
  
  @Inject
  @Extension
  private KColorExtensions _kColorExtensions;
  
  private final static SynthesisOption EXT_DEPS = SynthesisOption.createCheckOption("External Dependencies", Boolean.valueOf(false));
  
  @Override
  public List<SynthesisOption> getDisplayedSynthesisOptions() {
    return ImmutableList.<SynthesisOption>of(ProjectDependencySynthesis.EXT_DEPS);
  }
  
  @Override
  public List<Pair<IProperty<?>, List<?>>> getDisplayedLayoutOptions() {
    Pair<IProperty<?>, List<?>> _specifyLayoutOption = this.specifyLayoutOption(CoreOptions.ALGORITHM, null);
    EdgeRouting[] _values = EdgeRouting.values();
    Iterable<EdgeRouting> _drop = IterableExtensions.<EdgeRouting>drop(((Iterable<EdgeRouting>)Conversions.doWrapArray(_values)), 1);
    final Function1<EdgeRouting, String> _function = new Function1<EdgeRouting, String>() {
      @Override
      public String apply(final EdgeRouting it) {
        return it.name();
      }
    };
    List<EdgeRouting> _sortBy = IterableExtensions.<EdgeRouting, String>sortBy(_drop, _function);
    Pair<IProperty<?>, List<?>> _specifyLayoutOption_1 = this.specifyLayoutOption(CoreOptions.EDGE_ROUTING, _sortBy);
    Pair<IProperty<?>, List<?>> _specifyLayoutOption_2 = this.specifyLayoutOption(LayeredOptions.MERGE_EDGES, null);
    return ImmutableList.<Pair<IProperty<?>, List<?>>>of(_specifyLayoutOption, _specifyLayoutOption_1, _specifyLayoutOption_2);
  }
  
  @Override
  public KNode transform(final ProjectDependencyInformation model) {
    KNode _createNode = this._kNodeExtensions.createNode(model);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode root) {
        Iterable<IProject> _projects = model.getProjects();
        final Consumer<IProject> _function = new Consumer<IProject>() {
          @Override
          public void accept(final IProject project) {
            KNode _createNode = ProjectDependencySynthesis.this._kNodeExtensions.createNode(project);
            KNode _associateWith = ProjectDependencySynthesis.this.<KNode>associateWith(_createNode, project);
            final Procedure1<KNode> _function = new Procedure1<KNode>() {
              @Override
              public void apply(final KNode projectNode) {
                KRectangle _addRectangle = ProjectDependencySynthesis.this._kRenderingExtensions.addRectangle(projectNode);
                final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
                  @Override
                  public void apply(final KRectangle rect) {
                    String _name = project.getName();
                    KText _addText = ProjectDependencySynthesis.this._kContainerRenderingExtensions.addText(rect, _name);
                    ProjectDependencySynthesis.this.addPadding(_addText, 5);
                    ProjectDependencySynthesis.this._kRenderingExtensions.<KRectangle>addSingleClickAction(rect, "de.cau.cs.kieler.klighd.examples.dependencies.hightlightEdges");
                    String _name_1 = project.getName();
                    boolean _contains = _name_1.contains("de.cau.cs.kieler");
                    if (_contains) {
                      KColor _color = ProjectDependencySynthesis.this._kColorExtensions.getColor("lightgray");
                      ProjectDependencySynthesis.this._kRenderingExtensions.setBackground(rect, _color);
                    }
                  }
                };
                ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
                EList<KNode> _children = root.getChildren();
                _children.add(projectNode);
              }
            };
            ObjectExtensions.<KNode>operator_doubleArrow(_associateWith, _function);
          }
        };
        _projects.forEach(_function);
        Iterable<IProject> _projects_1 = model.getProjects();
        final Consumer<IProject> _function_1 = new Consumer<IProject>() {
          @Override
          public void accept(final IProject project) {
            final org.eclipse.xtext.xbase.lib.Pair<List<IProject>, List<String>> deps = model.getDependencies(project);
            InputOutput.<org.eclipse.xtext.xbase.lib.Pair<List<IProject>, List<String>>>println(deps);
            List<IProject> _key = deps.getKey();
            final Consumer<IProject> _function = new Consumer<IProject>() {
              @Override
              public void accept(final IProject dep) {
                KEdge _createEdge = ProjectDependencySynthesis.this._kEdgeExtensions.createEdge();
                final Procedure1<KEdge> _function = new Procedure1<KEdge>() {
                  @Override
                  public void apply(final KEdge edge) {
                    KNode _node = ProjectDependencySynthesis.this._kNodeExtensions.getNode(project);
                    edge.setSource(_node);
                    KNode _node_1 = ProjectDependencySynthesis.this._kNodeExtensions.getNode(dep);
                    edge.setTarget(_node_1);
                    KPolyline _addPolyline = ProjectDependencySynthesis.this._kEdgeExtensions.addPolyline(edge);
                    final Procedure1<KPolyline> _function = new Procedure1<KPolyline>() {
                      @Override
                      public void apply(final KPolyline pl) {
                        ProjectDependencySynthesis.this._kPolylineExtensions.addHeadArrowDecorator(pl);
                        ProjectDependencySynthesis.this._kPolylineExtensions.addJunctionPointDecorator(pl);
                      }
                    };
                    ObjectExtensions.<KPolyline>operator_doubleArrow(_addPolyline, _function);
                  }
                };
                ObjectExtensions.<KEdge>operator_doubleArrow(_createEdge, _function);
              }
            };
            _key.forEach(_function);
            boolean _booleanValue = ProjectDependencySynthesis.this.getBooleanValue(ProjectDependencySynthesis.EXT_DEPS);
            if (_booleanValue) {
              List<String> _value = deps.getValue();
              final Consumer<String> _function_1 = new Consumer<String>() {
                @Override
                public void accept(final String dep) {
                  KNode _createNode = ProjectDependencySynthesis.this._kNodeExtensions.createNode(dep);
                  KNode _associateWith = ProjectDependencySynthesis.this.<KNode>associateWith(_createNode, dep);
                  final Procedure1<KNode> _function = new Procedure1<KNode>() {
                    @Override
                    public void apply(final KNode extProject) {
                      KRectangle _addRectangle = ProjectDependencySynthesis.this._kRenderingExtensions.addRectangle(extProject);
                      final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
                        @Override
                        public void apply(final KRectangle rect) {
                          KText _addText = ProjectDependencySynthesis.this._kContainerRenderingExtensions.addText(rect, dep);
                          ProjectDependencySynthesis.this.addPadding(_addText, 5);
                          ProjectDependencySynthesis.this._kRenderingExtensions.<KRectangle>addSingleClickAction(rect, "de.cau.cs.kieler.klighd.examples.dependencies.hightlightEdges");
                          boolean _contains = dep.contains("de.cau.cs.kieler");
                          if (_contains) {
                            KColor _color = ProjectDependencySynthesis.this._kColorExtensions.getColor("lightgray");
                            ProjectDependencySynthesis.this._kRenderingExtensions.setBackground(rect, _color);
                          }
                          boolean _or = false;
                          boolean _contains_1 = dep.contains("gmf");
                          if (_contains_1) {
                            _or = true;
                          } else {
                            boolean _contains_2 = dep.contains("kivi");
                            _or = _contains_2;
                          }
                          if (_or) {
                            KColor _color_1 = ProjectDependencySynthesis.this._kColorExtensions.getColor("red");
                            ProjectDependencySynthesis.this._kRenderingExtensions.setBackground(rect, _color_1);
                          }
                          boolean _contains_3 = dep.contains("ui");
                          if (_contains_3) {
                            KColor _color_2 = ProjectDependencySynthesis.this._kColorExtensions.getColor("brown");
                            ProjectDependencySynthesis.this._kRenderingExtensions.setBackground(rect, _color_2);
                          }
                        }
                      };
                      ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
                      EList<KNode> _children = root.getChildren();
                      _children.add(extProject);
                    }
                  };
                  ObjectExtensions.<KNode>operator_doubleArrow(_associateWith, _function);
                  KEdge _createEdge = ProjectDependencySynthesis.this._kEdgeExtensions.createEdge();
                  final Procedure1<KEdge> _function_1 = new Procedure1<KEdge>() {
                    @Override
                    public void apply(final KEdge edge) {
                      KNode _node = ProjectDependencySynthesis.this._kNodeExtensions.getNode(project);
                      edge.setSource(_node);
                      KNode _node_1 = ProjectDependencySynthesis.this._kNodeExtensions.getNode(dep);
                      edge.setTarget(_node_1);
                      KPolyline _addPolyline = ProjectDependencySynthesis.this._kEdgeExtensions.addPolyline(edge);
                      final Procedure1<KPolyline> _function = new Procedure1<KPolyline>() {
                        @Override
                        public void apply(final KPolyline pl) {
                          ProjectDependencySynthesis.this._kPolylineExtensions.addHeadArrowDecorator(pl);
                          ProjectDependencySynthesis.this._kPolylineExtensions.addJunctionPointDecorator(pl);
                        }
                      };
                      ObjectExtensions.<KPolyline>operator_doubleArrow(_addPolyline, _function);
                    }
                  };
                  ObjectExtensions.<KEdge>operator_doubleArrow(_createEdge, _function_1);
                }
              };
              _value.forEach(_function_1);
            }
          }
        };
        _projects_1.forEach(_function_1);
      }
    };
    return ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
  }
  
  public KRendering addPadding(final KRendering r, final int size) {
    KAreaPlacementData _setAreaPlacementData = this._kRenderingExtensions.setAreaPlacementData(r);
    KAreaPlacementData _from = this._kRenderingExtensions.from(_setAreaPlacementData, this._kRenderingExtensions.LEFT, size, 0, this._kRenderingExtensions.TOP, size, 0);
    return this._kRenderingExtensions.to(_from, this._kRenderingExtensions.RIGHT, size, 0, this._kRenderingExtensions.BOTTOM, size, 0);
  }
}
