package de.cau.cs.kieler.klighd.examples.ecore;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import de.cau.cs.kieler.klighd.examples.ecore.EModelElementCollection;
import de.cau.cs.kieler.klighd.ui.view.DiagramView;
import de.cau.cs.kieler.klighd.ui.view.controller.AbstractViewUpdateController;
import de.cau.cs.kieler.klighd.ui.view.controllers.EditorSaveAdapter;
import de.cau.cs.kieler.klighd.ui.view.model.MessageModel;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.List;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.presentation.EcoreEditor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

/**
 * View controller for ecore modeling editor.
 * 
 * @author als
 */
@SuppressWarnings("all")
public class EcoreDiagramSynthesisController extends AbstractViewUpdateController implements EditorSaveAdapter.EditorSafeListener {
  /**
   * Controller ID.
   */
  private final static String ID = "de.cau.cs.kieler.klighd.examples.ecore.EcoreDiagramSynthesisController";
  
  /**
   * The save adapter for the editor.
   */
  private final EditorSaveAdapter saveAdapter;
  
  private final List<EModelElement> selectedModelElements = CollectionLiterals.<EModelElement>newLinkedList();
  
  private Class<?> navItemClass;
  
  private Method eObjectGetter;
  
  private EcoreEditor editor;
  
  /**
   * Default Constructor.
   */
  public EcoreDiagramSynthesisController() {
    EditorSaveAdapter _editorSaveAdapter = new EditorSaveAdapter(this);
    this.saveAdapter = _editorSaveAdapter;
    try {
      Class<?> _forName = Class.forName("org.eclipse.emf.ecoretools.diagram.navigator.EcoreDomainNavigatorItem");
      this.navItemClass = _forName;
      Method _method = this.navItemClass.getMethod("getEObject");
      this.eObjectGetter = _method;
    } catch (final Throwable _t) {
      if (_t instanceof Throwable) {
        final Throwable t = (Throwable)_t;
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public String getID() {
    return EcoreDiagramSynthesisController.ID;
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public void onActivate(final IEditorPart editor) {
    if ((editor instanceof EcoreEditor)) {
      Object _readModel = this.readModel(((EcoreEditor)editor));
      this.updateModel(_readModel);
      this.saveAdapter.activate(editor);
      ((EcoreEditor)editor).addSelectionChangedListener(this);
      this.editor = ((EcoreEditor)editor);
    } else {
      throw new IllegalArgumentException("Incompatible Editor");
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public void onDeactivate() {
    this.saveAdapter.deactivate();
    this.editor.removeSelectionChangedListener(this);
    this.editor = null;
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public void selectionChanged(final SelectionChangedEvent event) {
    Object _source = event.getSource();
    if ((_source instanceof EcoreEditor)) {
      Object _readModel = this.readModel(this.editor);
      this.updateModel(_readModel);
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public void onEditorSaved(final IEditorPart editor) {
    DiagramView _diagramView = this.getDiagramView();
    boolean _isLinkedWithActiveEditor = _diagramView.isLinkedWithActiveEditor();
    if (_isLinkedWithActiveEditor) {
      Object _readModel = this.readModel(this.editor);
      this.updateModel(_readModel);
    }
  }
  
  /**
   * {@inheritDoc}
   */
  @Override
  public void refresh() {
    Object _readModel = this.readModel(this.editor);
    this.updateModel(_readModel);
  }
  
  /**
   * Reads the selected model elements from given EcoreTreeEditor.
   * 
   * @param editor
   *            IEditorPart containing model
   * @return EObject model
   */
  private Object readModel(final EcoreEditor editor) {
    List _xifexpression = null;
    ISelection _selection = editor.getSelection();
    if ((_selection instanceof IStructuredSelection)) {
      ISelection _selection_1 = editor.getSelection();
      _xifexpression = ((IStructuredSelection) _selection_1).toList();
    } else {
      _xifexpression = CollectionLiterals.<Object>emptyList();
    }
    final List selectionElements = _xifexpression;
    boolean _isEmpty = selectionElements.isEmpty();
    if (_isEmpty) {
      return null;
    }
    EModelElementCollection modelElements = null;
    IPath editorInputPath = null;
    IEditorInput _editorInput = editor.getEditorInput();
    boolean _isInstance = FileEditorInput.class.isInstance(_editorInput);
    if (_isInstance) {
      IEditorInput _editorInput_1 = editor.getEditorInput();
      final FileEditorInput fileEditorInput = FileEditorInput.class.cast(_editorInput_1);
      boolean _and = false;
      IFile _file = fileEditorInput.getFile();
      boolean _notEquals = (!Objects.equal(_file, null));
      if (!_notEquals) {
        _and = false;
      } else {
        IFile _file_1 = fileEditorInput.getFile();
        URI _locationURI = _file_1.getLocationURI();
        boolean _notEquals_1 = (!Objects.equal(_locationURI, null));
        _and = _notEquals_1;
      }
      if (_and) {
        IPath _path = fileEditorInput.getPath();
        editorInputPath = _path;
      }
    }
    boolean _notEquals_2 = (!Objects.equal(editorInputPath, null));
    if (_notEquals_2) {
      Iterable<EModelElement> _filter = Iterables.<EModelElement>filter(selectionElements, EModelElement.class);
      List<EModelElement> _list = IterableExtensions.<EModelElement>toList(_filter);
      EModelElementCollection _of = EModelElementCollection.of(_list);
      modelElements = _of;
    }
    boolean _isNullOrEmpty = IterableExtensions.isNullOrEmpty(modelElements);
    if (_isNullOrEmpty) {
      List<EModelElement> elements = null;
      final Function1<Object, Boolean> _function = new Function1<Object, Boolean>() {
        @Override
        public Boolean apply(final Object it) {
          boolean _or = false;
          boolean _isInstance = EPackage.class.isInstance(it);
          if (_isInstance) {
            _or = true;
          } else {
            boolean _isInstance_1 = EClass.class.isInstance(it);
            _or = _isInstance_1;
          }
          return Boolean.valueOf(_or);
        }
      };
      boolean _forall = IterableExtensions.<Object>forall(selectionElements, _function);
      if (_forall) {
        Iterable<EModelElement> _filter_1 = Iterables.<EModelElement>filter(selectionElements, EModelElement.class);
        List<EModelElement> _list_1 = IterableExtensions.<EModelElement>toList(_filter_1);
        elements = _list_1;
      } else {
        boolean _and_1 = false;
        boolean _notEquals_3 = (!Objects.equal(this.navItemClass, null));
        if (!_notEquals_3) {
          _and_1 = false;
        } else {
          boolean _notEquals_4 = (!Objects.equal(this.eObjectGetter, null));
          _and_1 = _notEquals_4;
        }
        if (_and_1) {
          final Function1<Object, Boolean> _function_1 = new Function1<Object, Boolean>() {
            @Override
            public Boolean apply(final Object it) {
              return Boolean.valueOf(EcoreDiagramSynthesisController.this.navItemClass.isInstance(it));
            }
          };
          boolean _forall_1 = IterableExtensions.<Object>forall(selectionElements, _function_1);
          if (_forall_1) {
            Iterable<?> _filter_2 = Iterables.filter(selectionElements, this.navItemClass);
            final Function1<Object, Object> _function_2 = new Function1<Object, Object>() {
              @Override
              public Object apply(final Object it) {
                try {
                  return EcoreDiagramSynthesisController.this.eObjectGetter.invoke(it);
                } catch (Throwable _e) {
                  throw Exceptions.sneakyThrow(_e);
                }
              }
            };
            Iterable<Object> _map = IterableExtensions.map(_filter_2, _function_2);
            Iterable<EModelElement> _filter_3 = Iterables.<EModelElement>filter(_map, EModelElement.class);
            List<EModelElement> _list_2 = IterableExtensions.<EModelElement>toList(_filter_3);
            elements = _list_2;
          }
        }
      }
      boolean _notEquals_5 = (!Objects.equal(elements, null));
      if (_notEquals_5) {
        this.selectedModelElements.retainAll(elements);
        elements.removeAll(this.selectedModelElements);
        Iterables.<EModelElement>addAll(this.selectedModelElements, elements);
        EModelElementCollection _of_1 = EModelElementCollection.of(this.selectedModelElements);
        modelElements = _of_1;
      }
    }
    boolean _isNullOrEmpty_1 = IterableExtensions.isNullOrEmpty(modelElements);
    if (_isNullOrEmpty_1) {
      return new MessageModel("No model element selected");
    } else {
      return modelElements;
    }
  }
}
