package de.cau.cs.kieler.klighd.examples.uml2;

import com.google.common.collect.Iterables;
import de.cau.cs.kieler.klighd.KlighdConstants;
import de.cau.cs.kieler.klighd.krendering.KColor;
import de.cau.cs.kieler.klighd.krendering.KEllipse;
import de.cau.cs.kieler.klighd.krendering.KPolyline;
import de.cau.cs.kieler.klighd.krendering.KPosition;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KColorExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KEdgeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KLabelExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KNodeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import java.util.List;
import java.util.function.Consumer;
import javax.inject.Inject;
import org.eclipse.elk.alg.layered.p4nodes.NodePlacementStrategy;
import org.eclipse.elk.alg.layered.properties.LayeredOptions;
import org.eclipse.elk.core.options.CoreOptions;
import org.eclipse.elk.graph.KEdge;
import org.eclipse.elk.graph.KLabel;
import org.eclipse.elk.graph.KNode;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UseCase;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * This exemplary diagram synthesis implementation generates KGraph/KRendering model
 * based on given UML2 models containing actors, use cases and related associations.
 * 
 * @author chsch
 */
@SuppressWarnings("all")
public class UML2UseCaseDiagramSynthesis extends AbstractDiagramSynthesis<Model> {
  @Inject
  @Extension
  private KNodeExtensions _kNodeExtensions;
  
  @Inject
  @Extension
  private KEdgeExtensions _kEdgeExtensions;
  
  @Inject
  @Extension
  private KLabelExtensions _kLabelExtensions;
  
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  @Inject
  @Extension
  private KColorExtensions _kColorExtensions;
  
  @Override
  public KNode transform(final Model model) {
    KNode _createNode = this._kNodeExtensions.createNode(model);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode it) {
        UML2UseCaseDiagramSynthesis.this._kNodeExtensions.<Float>addLayoutParam(it, CoreOptions.SPACING_NODE, Float.valueOf(40f));
        UML2UseCaseDiagramSynthesis.this._kNodeExtensions.<NodePlacementStrategy>addLayoutParam(it, LayeredOptions.NODE_PLACEMENT_STRATEGY, NodePlacementStrategy.LINEAR_SEGMENTS);
        EList<KNode> _children = it.getChildren();
        EList<PackageableElement> _packagedElements = model.getPackagedElements();
        final Function1<PackageableElement, KNode> _function = new Function1<PackageableElement, KNode>() {
          @Override
          public KNode apply(final PackageableElement it) {
            KNode _switchResult = null;
            boolean _matched = false;
            if (!_matched) {
              if (it instanceof Actor) {
                _matched=true;
                _switchResult = UML2UseCaseDiagramSynthesis.this.createActorNode(((Actor)it));
              }
            }
            if (!_matched) {
              if (it instanceof UseCase) {
                _matched=true;
                _switchResult = UML2UseCaseDiagramSynthesis.this.createUseCaseNode(((UseCase)it));
              }
            }
            if (!_matched) {
              _switchResult = null;
            }
            return _switchResult;
          }
        };
        List<KNode> _map = ListExtensions.<PackageableElement, KNode>map(_packagedElements, _function);
        Iterable<KNode> _filterNull = IterableExtensions.<KNode>filterNull(_map);
        Iterables.<KNode>addAll(_children, _filterNull);
        EList<PackageableElement> _packagedElements_1 = model.getPackagedElements();
        Iterable<Association> _filter = Iterables.<Association>filter(_packagedElements_1, Association.class);
        final Consumer<Association> _function_1 = new Consumer<Association>() {
          @Override
          public void accept(final Association it) {
            UML2UseCaseDiagramSynthesis.this.createAssociationEdge(it);
          }
        };
        _filter.forEach(_function_1);
      }
    };
    return ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
  }
  
  public KNode createActorNode(final Actor actor) {
    KNode _createNode = this._kNodeExtensions.createNode(actor);
    KNode _associateWith = this.<KNode>associateWith(_createNode, actor);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode it) {
        UML2UseCaseDiagramSynthesis.this._kNodeExtensions.setNodeSize(it, 60, 100);
        String _name = actor.getName();
        KLabel _addOutsideBottomCenteredNodeLabel = UML2UseCaseDiagramSynthesis.this._kLabelExtensions.addOutsideBottomCenteredNodeLabel(it, _name, 
          KlighdConstants.DEFAULT_FONT_SIZE, KlighdConstants.DEFAULT_FONT_NAME);
        UML2UseCaseDiagramSynthesis.this.<KLabel>associateWith(_addOutsideBottomCenteredNodeLabel, actor);
        KRectangle _addRectangle = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.addRectangle(it);
        final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
          @Override
          public void apply(final KRectangle it) {
            UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.setInvisible(it, true);
            KPolyline _createActorFigureBody = UML2UseCaseDiagramSynthesis.this.createActorFigureBody();
            UML2UseCaseDiagramSynthesis.this._kContainerRenderingExtensions.<KPolyline>addChild(it, _createActorFigureBody);
            KEllipse _addEllipse = UML2UseCaseDiagramSynthesis.this._kContainerRenderingExtensions.addEllipse(it);
            KPosition _createKPosition = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 0, 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 0, 0);
            KEllipse _setPointPlacementData = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.<KEllipse>setPointPlacementData(_addEllipse, _createKPosition, 
              UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.H_CENTRAL, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.V_TOP, 0, 0, 35, 35);
            KColor _color = UML2UseCaseDiagramSynthesis.this._kColorExtensions.getColor("white");
            UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.setBackground(_setPointPlacementData, _color);
          }
        };
        ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
      }
    };
    return ObjectExtensions.<KNode>operator_doubleArrow(_associateWith, _function);
  }
  
  public KNode createUseCaseNode(final UseCase useCase) {
    KNode _createNode = this._kNodeExtensions.createNode(useCase);
    KNode _associateWith = this.<KNode>associateWith(_createNode, useCase);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode it) {
        KEllipse _addEllipse = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.addEllipse(it);
        final Procedure1<KEllipse> _function = new Procedure1<KEllipse>() {
          @Override
          public void apply(final KEllipse it) {
            KColor _color = UML2UseCaseDiagramSynthesis.this._kColorExtensions.getColor("darkGray");
            UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.setForeground(it, _color);
            String _name = useCase.getName();
            KText _addText = UML2UseCaseDiagramSynthesis.this._kContainerRenderingExtensions.addText(it, _name);
            KText _associateWith = UML2UseCaseDiagramSynthesis.this.<KText>associateWith(_addText, useCase);
            final Procedure1<KText> _function = new Procedure1<KText>() {
              @Override
              public void apply(final KText it) {
                UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.<KText>setSurroundingSpace(it, 10, 0.1f);
                KColor _color = UML2UseCaseDiagramSynthesis.this._kColorExtensions.getColor("white");
                UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.setBackground(it, _color);
              }
            };
            ObjectExtensions.<KText>operator_doubleArrow(_associateWith, _function);
          }
        };
        ObjectExtensions.<KEllipse>operator_doubleArrow(_addEllipse, _function);
      }
    };
    return ObjectExtensions.<KNode>operator_doubleArrow(_associateWith, _function);
  }
  
  public KEdge createAssociationEdge(final Association asso) {
    KEdge _createEdge = this._kEdgeExtensions.createEdge(asso);
    KEdge _associateWith = this.<KEdge>associateWith(_createEdge, asso);
    final Procedure1<KEdge> _function = new Procedure1<KEdge>() {
      @Override
      public void apply(final KEdge it) {
        EList<Property> _ownedEnds = asso.getOwnedEnds();
        Property _head = IterableExtensions.<Property>head(_ownedEnds);
        Type _type = null;
        if (_head!=null) {
          _type=_head.getType();
        }
        KNode _node = UML2UseCaseDiagramSynthesis.this._kNodeExtensions.getNode(_type);
        it.setSource(_node);
        EList<Property> _ownedEnds_1 = asso.getOwnedEnds();
        Property _last = IterableExtensions.<Property>last(_ownedEnds_1);
        Type _type_1 = null;
        if (_last!=null) {
          _type_1=_last.getType();
        }
        KNode _node_1 = UML2UseCaseDiagramSynthesis.this._kNodeExtensions.getNode(_type_1);
        it.setTarget(_node_1);
        UML2UseCaseDiagramSynthesis.this._kEdgeExtensions.addPolyline(it);
      }
    };
    return ObjectExtensions.<KEdge>operator_doubleArrow(_associateWith, _function);
  }
  
  public KPolyline createActorFigureBody() {
    KPolyline _createKPolyline = AbstractDiagramSynthesis.RENDERING_FACTORY.createKPolyline();
    final Procedure1<KPolyline> _function = new Procedure1<KPolyline>() {
      @Override
      public void apply(final KPolyline it) {
        EList<KPosition> _points = it.getPoints();
        KPosition _createKPosition = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 34, 0);
        _points.add(_createKPosition);
        EList<KPosition> _points_1 = it.getPoints();
        KPosition _createKPosition_1 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-2), 0.5f);
        _points_1.add(_createKPosition_1);
        EList<KPosition> _points_2 = it.getPoints();
        KPosition _createKPosition_2 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 0, 0f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-2), 0.5f);
        _points_2.add(_createKPosition_2);
        EList<KPosition> _points_3 = it.getPoints();
        KPosition _createKPosition_3 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 0, 0f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 2, 0.5f);
        _points_3.add(_createKPosition_3);
        EList<KPosition> _points_4 = it.getPoints();
        KPosition _createKPosition_4 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 2, 0.5f);
        _points_4.add(_createKPosition_4);
        EList<KPosition> _points_5 = it.getPoints();
        KPosition _createKPosition_5 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-1), 0.7f);
        _points_5.add(_createKPosition_5);
        EList<KPosition> _points_6 = it.getPoints();
        KPosition _createKPosition_6 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 0, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-3), 1);
        _points_6.add(_createKPosition_6);
        EList<KPosition> _points_7 = it.getPoints();
        KPosition _createKPosition_7 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 3, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 0, 1);
        _points_7.add(_createKPosition_7);
        EList<KPosition> _points_8 = it.getPoints();
        KPosition _createKPosition_8 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.LEFT, 0, 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 2, 0.7f);
        _points_8.add(_createKPosition_8);
        EList<KPosition> _points_9 = it.getPoints();
        KPosition _createKPosition_9 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, 3, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 0, 1);
        _points_9.add(_createKPosition_9);
        EList<KPosition> _points_10 = it.getPoints();
        KPosition _createKPosition_10 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, 0, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-3), 1);
        _points_10.add(_createKPosition_10);
        EList<KPosition> _points_11 = it.getPoints();
        KPosition _createKPosition_11 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-1), 0.7f);
        _points_11.add(_createKPosition_11);
        EList<KPosition> _points_12 = it.getPoints();
        KPosition _createKPosition_12 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 2, 0.5f);
        _points_12.add(_createKPosition_12);
        EList<KPosition> _points_13 = it.getPoints();
        KPosition _createKPosition_13 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, 0, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 2, 0.5f);
        _points_13.add(_createKPosition_13);
        EList<KPosition> _points_14 = it.getPoints();
        KPosition _createKPosition_14 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, 0, 0, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-2), 0.5f);
        _points_14.add(_createKPosition_14);
        EList<KPosition> _points_15 = it.getPoints();
        KPosition _createKPosition_15 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, (-2), 0.5f);
        _points_15.add(_createKPosition_15);
        EList<KPosition> _points_16 = it.getPoints();
        KPosition _createKPosition_16 = UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.createKPosition(UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.RIGHT, (-2), 0.5f, UML2UseCaseDiagramSynthesis.this._kRenderingExtensions.TOP, 34, 0);
        _points_16.add(_createKPosition_16);
      }
    };
    return ObjectExtensions.<KPolyline>operator_doubleArrow(_createKPolyline, _function);
  }
}
