/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2012 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.klighd.krendering.extensions;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import de.cau.cs.kieler.klighd.krendering.Colors;
import de.cau.cs.kieler.klighd.krendering.HorizontalAlignment;
import de.cau.cs.kieler.klighd.krendering.KAction;
import de.cau.cs.kieler.klighd.krendering.KAreaPlacementData;
import de.cau.cs.kieler.klighd.krendering.KBackground;
import de.cau.cs.kieler.klighd.krendering.KColor;
import de.cau.cs.kieler.klighd.krendering.KColoring;
import de.cau.cs.kieler.klighd.krendering.KContainerRendering;
import de.cau.cs.kieler.klighd.krendering.KDecoratorPlacementData;
import de.cau.cs.kieler.klighd.krendering.KEllipse;
import de.cau.cs.kieler.klighd.krendering.KFontBold;
import de.cau.cs.kieler.klighd.krendering.KFontItalic;
import de.cau.cs.kieler.klighd.krendering.KFontName;
import de.cau.cs.kieler.klighd.krendering.KFontSize;
import de.cau.cs.kieler.klighd.krendering.KForeground;
import de.cau.cs.kieler.klighd.krendering.KGridPlacement;
import de.cau.cs.kieler.klighd.krendering.KGridPlacementData;
import de.cau.cs.kieler.klighd.krendering.KHorizontalAlignment;
import de.cau.cs.kieler.klighd.krendering.KInvisibility;
import de.cau.cs.kieler.klighd.krendering.KLineCap;
import de.cau.cs.kieler.klighd.krendering.KLineJoin;
import de.cau.cs.kieler.klighd.krendering.KLineStyle;
import de.cau.cs.kieler.klighd.krendering.KLineWidth;
import de.cau.cs.kieler.klighd.krendering.KPlacementData;
import de.cau.cs.kieler.klighd.krendering.KPointPlacementData;
import de.cau.cs.kieler.klighd.krendering.KPolygon;
import de.cau.cs.kieler.klighd.krendering.KPolyline;
import de.cau.cs.kieler.klighd.krendering.KPosition;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.klighd.krendering.KRenderingFactory;
import de.cau.cs.kieler.klighd.krendering.KRenderingPackage;
import de.cau.cs.kieler.klighd.krendering.KRotation;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.KShadow;
import de.cau.cs.kieler.klighd.krendering.KStyle;
import de.cau.cs.kieler.klighd.krendering.KStyleHolder;
import de.cau.cs.kieler.klighd.krendering.KStyleRef;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.KTextStrikeout;
import de.cau.cs.kieler.klighd.krendering.KTextUnderline;
import de.cau.cs.kieler.klighd.krendering.KVerticalAlignment;
import de.cau.cs.kieler.klighd.krendering.KXPosition;
import de.cau.cs.kieler.klighd.krendering.KYPosition;
import de.cau.cs.kieler.klighd.krendering.LineCap;
import de.cau.cs.kieler.klighd.krendering.LineJoin;
import de.cau.cs.kieler.klighd.krendering.LineStyle;
import de.cau.cs.kieler.klighd.krendering.Trigger;
import de.cau.cs.kieler.klighd.krendering.Underline;
import de.cau.cs.kieler.klighd.krendering.VerticalAlignment;
import de.cau.cs.kieler.klighd.krendering.extensions.PositionReferenceX;
import de.cau.cs.kieler.klighd.krendering.extensions.PositionReferenceY;
import java.util.Arrays;
import java.util.List;
import org.eclipse.elk.graph.KGraphData;
import org.eclipse.elk.graph.KGraphElement;
import org.eclipse.elk.graph.KLabel;
import org.eclipse.elk.graph.KLabeledGraphElement;
import org.eclipse.elk.graph.KNode;
import org.eclipse.elk.graph.KPort;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * This class contains lots of convenient helper functions for configuring KRendering-based view models,
 * and it does not claim to be complete ;-).<br>
 * In order to be consistent with the further extension classes the extension methods are non-static
 * ones requiring this class to be instantiated. Since this class doesn't declare any fields (i.e.
 * required memory) the instantiation should not be a problem. The instantiation may be done directly
 * by calling 'new KRenderingExtensions()' or by delegating that to a dependency injection framework.<br>
 * <br>
 * NOTE: Do NOT introduce <i>create extensions</i> or other continuous memory in that class!
 * 
 * @containsExtensions
 */
@SuppressWarnings("all")
public class KRenderingExtensions {
  @Extension
  private KRenderingFactory _kRenderingFactory = KRenderingFactory.eINSTANCE;
  
  /**
   * Reveals the KRendering of a KGraphElement, e.g. for the purpose of highlighting.
   */
  public KRendering getKRendering(final KGraphElement kge) {
    return kge.<KRendering>getData(KRendering.class);
  }
  
  public KContainerRendering getKContainerRendering(final KGraphElement kge) {
    return kge.<KContainerRendering>getData(KContainerRendering.class);
  }
  
  protected KContainerRendering _addInvisibleContainerRendering(final KNode node) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        KRenderingExtensions.this.setInvisible(it, true);
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  protected KContainerRendering _addInvisibleContainerRendering(final KPort port) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        KRenderingExtensions.this.setInvisible(it, true);
        EList<KGraphData> _data = port.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  protected KContainerRendering _addInvisibleContainerRendering(final KLabel label) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        KRenderingExtensions.this.setInvisible(it, true);
        EList<KGraphData> _data = label.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  protected KEllipse _addEllipse(final KNode node) {
    KEllipse _createKEllipse = this._kRenderingFactory.createKEllipse();
    final Procedure1<KEllipse> _function = new Procedure1<KEllipse>() {
      @Override
      public void apply(final KEllipse it) {
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KEllipse>operator_doubleArrow(_createKEllipse, _function);
  }
  
  protected KEllipse _addEllipse(final KPort port) {
    KEllipse _createKEllipse = this._kRenderingFactory.createKEllipse();
    final Procedure1<KEllipse> _function = new Procedure1<KEllipse>() {
      @Override
      public void apply(final KEllipse it) {
        EList<KGraphData> _data = port.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KEllipse>operator_doubleArrow(_createKEllipse, _function);
  }
  
  protected KEllipse _addEllipse(final KLabel label) {
    KEllipse _createKEllipse = this._kRenderingFactory.createKEllipse();
    final Procedure1<KEllipse> _function = new Procedure1<KEllipse>() {
      @Override
      public void apply(final KEllipse it) {
        EList<KGraphData> _data = label.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KEllipse>operator_doubleArrow(_createKEllipse, _function);
  }
  
  protected KPolygon _addPolygon(final KNode node) {
    KPolygon _createKPolygon = this._kRenderingFactory.createKPolygon();
    final Procedure1<KPolygon> _function = new Procedure1<KPolygon>() {
      @Override
      public void apply(final KPolygon it) {
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KPolygon>operator_doubleArrow(_createKPolygon, _function);
  }
  
  protected KPolygon _addPolygon(final KPort port) {
    KPolygon _createKPolygon = this._kRenderingFactory.createKPolygon();
    final Procedure1<KPolygon> _function = new Procedure1<KPolygon>() {
      @Override
      public void apply(final KPolygon it) {
        EList<KGraphData> _data = port.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KPolygon>operator_doubleArrow(_createKPolygon, _function);
  }
  
  /**
   * Adds a rectangle to the node.
   * @example
   * node.addRectangle => [
   *   it.background = "green".color
   * ]
   * 
   * @extensionCategory composition
   */
  protected KRectangle _addRectangle(final KNode node) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  protected KRectangle _addRectangle(final KPort port) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        EList<KGraphData> _data = port.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  protected KRectangle _addRectangle(final KLabel label) {
    KRectangle _createKRectangle = this._kRenderingFactory.createKRectangle();
    final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
      @Override
      public void apply(final KRectangle it) {
        EList<KGraphData> _data = label.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRectangle>operator_doubleArrow(_createKRectangle, _function);
  }
  
  /**
   * @extensionCategory composition
   */
  protected KRoundedRectangle _addRoundedRectangle(final KNode node, final float cWidth, final float cHeight) {
    KRoundedRectangle _createKRoundedRectangle = this._kRenderingFactory.createKRoundedRectangle();
    final Procedure1<KRoundedRectangle> _function = new Procedure1<KRoundedRectangle>() {
      @Override
      public void apply(final KRoundedRectangle it) {
        it.setCornerWidth(cWidth);
        it.setCornerHeight(cHeight);
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_createKRoundedRectangle, _function);
  }
  
  protected KRoundedRectangle _addRoundedRectangle(final KLabel label, final float cWidth, final float cHeight) {
    KRoundedRectangle _createKRoundedRectangle = this._kRenderingFactory.createKRoundedRectangle();
    final Procedure1<KRoundedRectangle> _function = new Procedure1<KRoundedRectangle>() {
      @Override
      public void apply(final KRoundedRectangle it) {
        it.setCornerWidth(cWidth);
        it.setCornerHeight(cHeight);
        EList<KGraphData> _data = label.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_createKRoundedRectangle, _function);
  }
  
  protected KRoundedRectangle _addRoundedRectangle(final KNode node, final float cWidth, final float cHeight, final float lineWidth) {
    KRoundedRectangle _createKRoundedRectangle = this._kRenderingFactory.createKRoundedRectangle();
    final Procedure1<KRoundedRectangle> _function = new Procedure1<KRoundedRectangle>() {
      @Override
      public void apply(final KRoundedRectangle it) {
        it.setCornerWidth(cWidth);
        it.setCornerHeight(cHeight);
        KRenderingExtensions.this.<KRoundedRectangle>setLineWidth(it, lineWidth);
        EList<KGraphData> _data = node.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_createKRoundedRectangle, _function);
  }
  
  protected KRoundedRectangle _addRoundedRectangle(final KLabel label, final float cWidth, final float cHeight, final float lineWidth) {
    KRoundedRectangle _createKRoundedRectangle = this._kRenderingFactory.createKRoundedRectangle();
    final Procedure1<KRoundedRectangle> _function = new Procedure1<KRoundedRectangle>() {
      @Override
      public void apply(final KRoundedRectangle it) {
        it.setCornerWidth(cWidth);
        it.setCornerHeight(cHeight);
        KRenderingExtensions.this.<KRoundedRectangle>setLineWidth(it, lineWidth);
        EList<KGraphData> _data = label.getData();
        _data.add(it);
      }
    };
    return ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_createKRoundedRectangle, _function);
  }
  
  public KRoundedRectangle setCornerSize(final KRoundedRectangle rect, final float cWidth, final float cHeight) {
    final Procedure1<KRoundedRectangle> _function = new Procedure1<KRoundedRectangle>() {
      @Override
      public void apply(final KRoundedRectangle it) {
        it.setCornerWidth(cWidth);
        it.setCornerHeight(cHeight);
      }
    };
    return ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(rect, _function);
  }
  
  public KText addText(final KNode node, final String text) {
    KText _createKText = this._kRenderingFactory.createKText();
    final Procedure1<KText> _function = new Procedure1<KText>() {
      @Override
      public void apply(final KText it) {
        EList<KGraphData> _data = node.getData();
        _data.add(it);
        it.setText(text);
      }
    };
    return ObjectExtensions.<KText>operator_doubleArrow(_createKText, _function);
  }
  
  public <T extends KRendering> T addAction(final T rendering, final Trigger trigger, final String actionId) {
    EList<KAction> _actions = rendering.getActions();
    KAction _createKAction = this._kRenderingFactory.createKAction();
    final Procedure1<KAction> _function = new Procedure1<KAction>() {
      @Override
      public void apply(final KAction it) {
        it.setTrigger(trigger);
        it.setActionId(actionId);
      }
    };
    KAction _doubleArrow = ObjectExtensions.<KAction>operator_doubleArrow(_createKAction, _function);
    _actions.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addAction(final T rendering, final Trigger trigger, final String actionId, final boolean altPressed, final boolean ctrlCmdPressed, final boolean shiftPressed) {
    EList<KAction> _actions = rendering.getActions();
    KAction _createKAction = this._kRenderingFactory.createKAction();
    final Procedure1<KAction> _function = new Procedure1<KAction>() {
      @Override
      public void apply(final KAction it) {
        it.setTrigger(trigger);
        it.setActionId(actionId);
        it.setAltPressed(altPressed);
        it.setCtrlCmdPressed(ctrlCmdPressed);
        it.setShiftPressed(shiftPressed);
      }
    };
    KAction _doubleArrow = ObjectExtensions.<KAction>operator_doubleArrow(_createKAction, _function);
    _actions.add(_doubleArrow);
    return rendering;
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a single (!) click on <code>rendering</code>. The action triggered with a delay if and
   * only if no subsequent click occurs within the system wide double click time.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addSingleClickAction(final T rendering, final String actionId) {
    return this.<T>addAction(rendering, Trigger.SINGLECLICK, actionId);
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a single (!) click on <code>rendering</code>. The action triggered with a delay if and
   * only if no subsequent click occurs within the system wide double click time, and if the
   * required modifier keys are pressed.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addSingleClickAction(final T rendering, final String actionId, final boolean altPressed, final boolean ctrlCmdPressed, final boolean shiftPressed) {
    return this.<T>addAction(rendering, Trigger.SINGLECLICK, actionId, altPressed, ctrlCmdPressed, shiftPressed);
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a double (or tripple, ...) click on <code>rendering</code>.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addDoubleClickAction(final T rendering, final String actionId) {
    return this.<T>addAction(rendering, Trigger.DOUBLECLICK, actionId);
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a double (or tripple, ...) click on <code>rendering</code> if the required modifier
   * keys are pressed.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addDoubleClickAction(final T rendering, final String actionId, final boolean altPressed, final boolean ctrlCmdPressed, final boolean shiftPressed) {
    return this.<T>addAction(rendering, Trigger.DOUBLECLICK, actionId, altPressed, ctrlCmdPressed, shiftPressed);
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a single click on <code>rendering</code> regardless of whether more clicks follow
   * within the system wide double click time. In contrast to #addSingleClickAction, there's
   * no delay between the event receipt and the action execution.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addSingleOrMultiClickAction(final T rendering, final String actionId) {
    return this.<T>addAction(rendering, Trigger.SINGLE_OR_MULTICLICK, actionId);
  }
  
  /**
   * Configures the action being registered in KLighD with <code>actionId</code> to be executed
   * after a single click on <code>rendering</code> regardless of whether more clicks follow
   * within the system wide double click time, if the required modifier keys are pressed.
   * In contrast to #addSingleClickAction, there's no delay between the event receipt and the
   * action execution.
   * 
   * @extensionCategory actions
   */
  public <T extends KRendering> T addSingleOrMultiClickAction(final T rendering, final String actionId, final boolean altPressed, final boolean ctrlCmdPressed, final boolean shiftPressed) {
    return this.<T>addAction(rendering, Trigger.SINGLE_OR_MULTICLICK, actionId, altPressed, ctrlCmdPressed, shiftPressed);
  }
  
  public <T extends KRendering> T with(final T rendering, final KPlacementData pd) {
    rendering.setPlacementData(pd);
    return rendering;
  }
  
  public <T extends KRendering> T with(final T rendering, final KStyle style) {
    EList<KStyle> _styles = rendering.getStyles();
    _styles.add(style);
    return rendering;
  }
  
  public <T extends KRendering> T withCopyOf(final T rendering, final KStyle style) {
    EList<KStyle> _styles = rendering.getStyles();
    KStyle _copy = EcoreUtil.<KStyle>copy(style);
    _styles.add(_copy);
    return rendering;
  }
  
  private final static Function1<? super KStyle, ? extends Boolean> IS_SELECTION = new Function1<KStyle, Boolean>() {
    @Override
    public Boolean apply(final KStyle it) {
      return Boolean.valueOf(it.isSelection());
    }
  };
  
  private final static Procedure1<? super KStyle> SET_SELECTION = new Procedure1<KStyle>() {
    @Override
    public void apply(final KStyle it) {
      it.setSelection(true);
    }
  };
  
  public <T extends KRendering> T setStyleRef(final T rendering, final KStyleHolder styleHolder) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyleRef> _filter = Iterables.<KStyleRef>filter(_styles_1, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionStyleRef(final T rendering, final KStyleHolder styleHolder) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KStyleRef> _filter_1 = Iterables.<KStyleRef>filter(_filter, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter_1);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setStyleRef(final T rendering, final KStyleHolder styleHolder, final Class<KStyle>... styleTypes) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyleRef> _filter = Iterables.<KStyleRef>filter(_styles_1, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<? extends Class<KStyle>>)Conversions.doWrapArray(styleTypes)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionStyleRef(final T rendering, final KStyleHolder styleHolder, final Class<KStyle>... styleTypes) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KStyleRef> _filter_1 = Iterables.<KStyleRef>filter(_filter, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter_1);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<? extends Class<KStyle>>)Conversions.doWrapArray(styleTypes)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setStyleRef(final T rendering, final KStyleHolder styleHolder, final EClass... styleTypes) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyleRef> _filter = Iterables.<KStyleRef>filter(_styles_1, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        final Function1<EClass, Class<?>> _function = new Function1<EClass, Class<?>>() {
          @Override
          public Class<?> apply(final EClass it) {
            return it.getInstanceClass();
          }
        };
        List<Class<?>> _map = ListExtensions.<EClass, Class<?>>map(((List<EClass>)Conversions.doWrapArray(styleTypes)), _function);
        final Function1<Class<?>, Boolean> _function_1 = new Function1<Class<?>, Boolean>() {
          @Override
          public Boolean apply(final Class<?> it) {
            return Boolean.valueOf(KStyle.class.isAssignableFrom(it));
          }
        };
        Iterable<Class<?>> _filter = IterableExtensions.<Class<?>>filter(_map, _function_1);
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<Class<KStyle>>) ((Iterable<?>) _filter)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionStyleRef(final T rendering, final KStyleHolder styleHolder, final EClass... styleTypes) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KStyleRef> _filter_1 = Iterables.<KStyleRef>filter(_filter, KStyleRef.class);
    List<KStyleRef> _list = IterableExtensions.<KStyleRef>toList(_filter_1);
    _styles.removeAll(_list);
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles_2 = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        final Function1<EClass, Class<?>> _function = new Function1<EClass, Class<?>>() {
          @Override
          public Class<?> apply(final EClass it) {
            return it.getInstanceClass();
          }
        };
        List<Class<?>> _map = ListExtensions.<EClass, Class<?>>map(((List<EClass>)Conversions.doWrapArray(styleTypes)), _function);
        final Function1<Class<?>, Boolean> _function_1 = new Function1<Class<?>, Boolean>() {
          @Override
          public Boolean apply(final Class<?> it) {
            return Boolean.valueOf(KStyle.class.isAssignableFrom(it));
          }
        };
        Iterable<Class<?>> _filter = IterableExtensions.<Class<?>>filter(_map, _function_1);
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<Class<KStyle>>) ((Iterable<?>) _filter)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addStyleRef(final T rendering, final KStyleHolder styleHolder) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addSelectionStyleRef(final T rendering, final KStyleHolder styleHolder) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addStyleRef(final T rendering, final KStyleHolder styleHolder, final Class<KStyle>... styleTypes) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<? extends Class<KStyle>>)Conversions.doWrapArray(styleTypes)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addSelectionStyleRef(final T rendering, final KStyleHolder styleHolder, final Class<KStyle>... styleTypes) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<? extends Class<KStyle>>)Conversions.doWrapArray(styleTypes)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addStyleRef(final T rendering, final KStyleHolder styleHolder, final EClass... styleTypes) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        final Function1<EClass, Class<?>> _function = new Function1<EClass, Class<?>>() {
          @Override
          public Class<?> apply(final EClass it) {
            return it.getInstanceClass();
          }
        };
        List<Class<?>> _map = ListExtensions.<EClass, Class<?>>map(((List<EClass>)Conversions.doWrapArray(styleTypes)), _function);
        final Function1<Class<?>, Boolean> _function_1 = new Function1<Class<?>, Boolean>() {
          @Override
          public Boolean apply(final Class<?> it) {
            return Boolean.valueOf(KStyle.class.isAssignableFrom(it));
          }
        };
        Iterable<Class<?>> _filter = IterableExtensions.<Class<?>>filter(_map, _function_1);
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<Class<KStyle>>) ((Iterable<?>) _filter)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T addSelectionStyleRef(final T rendering, final KStyleHolder styleHolder, final EClass... styleTypes) {
    boolean _equals = Objects.equal(styleHolder, null);
    if (_equals) {
      return rendering;
    }
    EList<KStyle> _styles = rendering.getStyles();
    KStyleRef _createKStyleRef = this._kRenderingFactory.createKStyleRef();
    final Procedure1<KStyleRef> _function = new Procedure1<KStyleRef>() {
      @Override
      public void apply(final KStyleRef it) {
        it.setSelection(true);
        it.setStyleHolder(styleHolder);
        EList<Class<KStyle>> _referencedTypes = it.getReferencedTypes();
        final Function1<EClass, Class<?>> _function = new Function1<EClass, Class<?>>() {
          @Override
          public Class<?> apply(final EClass it) {
            return it.getInstanceClass();
          }
        };
        List<Class<?>> _map = ListExtensions.<EClass, Class<?>>map(((List<EClass>)Conversions.doWrapArray(styleTypes)), _function);
        final Function1<Class<?>, Boolean> _function_1 = new Function1<Class<?>, Boolean>() {
          @Override
          public Boolean apply(final Class<?> it) {
            return Boolean.valueOf(KStyle.class.isAssignableFrom(it));
          }
        };
        Iterable<Class<?>> _filter = IterableExtensions.<Class<?>>filter(_map, _function_1);
        Iterables.<Class<KStyle>>addAll(_referencedTypes, ((Iterable<Class<KStyle>>) ((Iterable<?>) _filter)));
      }
    };
    KStyleRef _doubleArrow = ObjectExtensions.<KStyleRef>operator_doubleArrow(_createKStyleRef, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public KInvisibility getInvisible(final KRendering rendering) {
    KInvisibility _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KInvisibility> _filter = Iterables.<KInvisibility>filter(_styles, KInvisibility.class);
    KInvisibility _last = IterableExtensions.<KInvisibility>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KInvisibility _createKInvisibility = this._kRenderingFactory.createKInvisibility();
      _elvis = _createKInvisibility;
    }
    return _elvis;
  }
  
  public boolean getInvisibleValue(final KRendering rendering) {
    KInvisibility _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KInvisibility> _filter = Iterables.<KInvisibility>filter(_styles, KInvisibility.class);
    KInvisibility _last = IterableExtensions.<KInvisibility>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KInvisibility _createKInvisibility = this._kRenderingFactory.createKInvisibility();
      _elvis = _createKInvisibility;
    }
    return _elvis.isInvisible();
  }
  
  public <T extends KRendering> T setInvisible(final T rendering, final boolean invisible) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KInvisibility> _filter = Iterables.<KInvisibility>filter(_styles_1, KInvisibility.class);
    List<KInvisibility> _list = IterableExtensions.<KInvisibility>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KInvisibility _createKInvisibility = this._kRenderingFactory.createKInvisibility();
    final Procedure1<KInvisibility> _function = new Procedure1<KInvisibility>() {
      @Override
      public void apply(final KInvisibility it) {
        it.setInvisible(invisible);
      }
    };
    KInvisibility _doubleArrow = ObjectExtensions.<KInvisibility>operator_doubleArrow(_createKInvisibility, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionInvisible(final T rendering, final boolean invisible) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KInvisibility> _filter_1 = Iterables.<KInvisibility>filter(_filter, KInvisibility.class);
    List<KInvisibility> _list = IterableExtensions.<KInvisibility>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KInvisibility _createKInvisibility = this._kRenderingFactory.createKInvisibility();
    final Procedure1<KInvisibility> _function = new Procedure1<KInvisibility>() {
      @Override
      public void apply(final KInvisibility it) {
        it.setSelection(true);
        it.setInvisible(invisible);
      }
    };
    KInvisibility _doubleArrow = ObjectExtensions.<KInvisibility>operator_doubleArrow(_createKInvisibility, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KLineWidth getLineWidth(final KRendering rendering) {
    KLineWidth _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineWidth> _filter = Iterables.<KLineWidth>filter(_styles, KLineWidth.class);
    KLineWidth _last = IterableExtensions.<KLineWidth>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineWidth _createKLineWidth = this._kRenderingFactory.createKLineWidth();
      final Procedure1<KLineWidth> _function = new Procedure1<KLineWidth>() {
        @Override
        public void apply(final KLineWidth it) {
          it.setLineWidth(1f);
        }
      };
      KLineWidth _doubleArrow = ObjectExtensions.<KLineWidth>operator_doubleArrow(_createKLineWidth, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public float getLineWidthValue(final KRendering rendering) {
    KLineWidth _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineWidth> _filter = Iterables.<KLineWidth>filter(_styles, KLineWidth.class);
    KLineWidth _last = IterableExtensions.<KLineWidth>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineWidth _createKLineWidth = this._kRenderingFactory.createKLineWidth();
      final Procedure1<KLineWidth> _function = new Procedure1<KLineWidth>() {
        @Override
        public void apply(final KLineWidth it) {
          it.setLineWidth(1f);
        }
      };
      KLineWidth _doubleArrow = ObjectExtensions.<KLineWidth>operator_doubleArrow(_createKLineWidth, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getLineWidth();
  }
  
  /**
   * @extensionCategory style
   */
  public <T extends KRendering> T setLineWidth(final T rendering, final float width) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KLineWidth> _filter = Iterables.<KLineWidth>filter(_styles_1, KLineWidth.class);
    List<KLineWidth> _list = IterableExtensions.<KLineWidth>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineWidth _createKLineWidth = this._kRenderingFactory.createKLineWidth();
    final Procedure1<KLineWidth> _function = new Procedure1<KLineWidth>() {
      @Override
      public void apply(final KLineWidth it) {
        it.setLineWidth(width);
      }
    };
    KLineWidth _doubleArrow = ObjectExtensions.<KLineWidth>operator_doubleArrow(_createKLineWidth, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionLineWidth(final T rendering, final float width) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KLineWidth> _filter_1 = Iterables.<KLineWidth>filter(_filter, KLineWidth.class);
    List<KLineWidth> _list = IterableExtensions.<KLineWidth>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineWidth _createKLineWidth = this._kRenderingFactory.createKLineWidth();
    final Procedure1<KLineWidth> _function = new Procedure1<KLineWidth>() {
      @Override
      public void apply(final KLineWidth it) {
        it.setSelection(true);
        it.setLineWidth(width);
      }
    };
    KLineWidth _doubleArrow = ObjectExtensions.<KLineWidth>operator_doubleArrow(_createKLineWidth, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  /**
   * @extensionCategory style
   */
  public KLineStyle getLineStyle(final KRendering rendering) {
    KLineStyle _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineStyle> _filter = Iterables.<KLineStyle>filter(_styles, KLineStyle.class);
    KLineStyle _last = IterableExtensions.<KLineStyle>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineStyle _createKLineStyle = this._kRenderingFactory.createKLineStyle();
      final Procedure1<KLineStyle> _function = new Procedure1<KLineStyle>() {
        @Override
        public void apply(final KLineStyle it) {
          it.setLineStyle(LineStyle.SOLID);
        }
      };
      KLineStyle _doubleArrow = ObjectExtensions.<KLineStyle>operator_doubleArrow(_createKLineStyle, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public LineStyle getLineStyleValue(final KRendering rendering) {
    KLineStyle _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineStyle> _filter = Iterables.<KLineStyle>filter(_styles, KLineStyle.class);
    KLineStyle _last = IterableExtensions.<KLineStyle>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineStyle _createKLineStyle = this._kRenderingFactory.createKLineStyle();
      final Procedure1<KLineStyle> _function = new Procedure1<KLineStyle>() {
        @Override
        public void apply(final KLineStyle it) {
          it.setLineStyle(LineStyle.SOLID);
        }
      };
      KLineStyle _doubleArrow = ObjectExtensions.<KLineStyle>operator_doubleArrow(_createKLineStyle, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getLineStyle();
  }
  
  public <T extends KRendering> T setLineStyle(final T rendering, final LineStyle style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KLineStyle> _filter = Iterables.<KLineStyle>filter(_styles_1, KLineStyle.class);
    List<KLineStyle> _list = IterableExtensions.<KLineStyle>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineStyle _createKLineStyle = this._kRenderingFactory.createKLineStyle();
    final Procedure1<KLineStyle> _function = new Procedure1<KLineStyle>() {
      @Override
      public void apply(final KLineStyle it) {
        it.setLineStyle(style);
      }
    };
    KLineStyle _doubleArrow = ObjectExtensions.<KLineStyle>operator_doubleArrow(_createKLineStyle, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionLineStyle(final T rendering, final LineStyle style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KLineStyle> _filter_1 = Iterables.<KLineStyle>filter(_filter, KLineStyle.class);
    List<KLineStyle> _list = IterableExtensions.<KLineStyle>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineStyle _createKLineStyle = this._kRenderingFactory.createKLineStyle();
    final Procedure1<KLineStyle> _function = new Procedure1<KLineStyle>() {
      @Override
      public void apply(final KLineStyle it) {
        it.setSelection(true);
        it.setLineStyle(style);
      }
    };
    KLineStyle _doubleArrow = ObjectExtensions.<KLineStyle>operator_doubleArrow(_createKLineStyle, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KLineCap getLineCap(final KRendering rendering) {
    KLineCap _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineCap> _filter = Iterables.<KLineCap>filter(_styles, KLineCap.class);
    KLineCap _last = IterableExtensions.<KLineCap>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineCap _createKLineCap = this._kRenderingFactory.createKLineCap();
      final Procedure1<KLineCap> _function = new Procedure1<KLineCap>() {
        @Override
        public void apply(final KLineCap it) {
          it.setLineCap(LineCap.CAP_FLAT);
        }
      };
      KLineCap _doubleArrow = ObjectExtensions.<KLineCap>operator_doubleArrow(_createKLineCap, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public LineCap getLineCapValue(final KRendering rendering) {
    KLineCap _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineCap> _filter = Iterables.<KLineCap>filter(_styles, KLineCap.class);
    KLineCap _last = IterableExtensions.<KLineCap>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineCap _createKLineCap = this._kRenderingFactory.createKLineCap();
      final Procedure1<KLineCap> _function = new Procedure1<KLineCap>() {
        @Override
        public void apply(final KLineCap it) {
          it.setLineCap(LineCap.CAP_FLAT);
        }
      };
      KLineCap _doubleArrow = ObjectExtensions.<KLineCap>operator_doubleArrow(_createKLineCap, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getLineCap();
  }
  
  public <T extends KRendering> T setLineCap(final T rendering, final LineCap style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KLineCap> _filter = Iterables.<KLineCap>filter(_styles_1, KLineCap.class);
    List<KLineCap> _list = IterableExtensions.<KLineCap>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineCap _createKLineCap = this._kRenderingFactory.createKLineCap();
    final Procedure1<KLineCap> _function = new Procedure1<KLineCap>() {
      @Override
      public void apply(final KLineCap it) {
        it.setLineCap(style);
      }
    };
    KLineCap _doubleArrow = ObjectExtensions.<KLineCap>operator_doubleArrow(_createKLineCap, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionLineCap(final T rendering, final LineCap style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KLineCap> _filter_1 = Iterables.<KLineCap>filter(_filter, KLineCap.class);
    List<KLineCap> _list = IterableExtensions.<KLineCap>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineCap _createKLineCap = this._kRenderingFactory.createKLineCap();
    final Procedure1<KLineCap> _function = new Procedure1<KLineCap>() {
      @Override
      public void apply(final KLineCap it) {
        it.setSelection(true);
        it.setLineCap(style);
      }
    };
    KLineCap _doubleArrow = ObjectExtensions.<KLineCap>operator_doubleArrow(_createKLineCap, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KLineJoin getLineJoin(final KRendering rendering) {
    KLineJoin _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineJoin> _filter = Iterables.<KLineJoin>filter(_styles, KLineJoin.class);
    KLineJoin _last = IterableExtensions.<KLineJoin>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
      final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
        @Override
        public void apply(final KLineJoin it) {
          it.setLineJoin(LineJoin.JOIN_MITER);
        }
      };
      KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public LineJoin getLineJoinValue(final KRendering rendering) {
    KLineJoin _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KLineJoin> _filter = Iterables.<KLineJoin>filter(_styles, KLineJoin.class);
    KLineJoin _last = IterableExtensions.<KLineJoin>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
      final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
        @Override
        public void apply(final KLineJoin it) {
          it.setLineJoin(LineJoin.JOIN_MITER);
        }
      };
      KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getLineJoin();
  }
  
  public <T extends KRendering> T setLineJoin(final T rendering, final LineJoin style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KLineJoin> _filter = Iterables.<KLineJoin>filter(_styles_1, KLineJoin.class);
    List<KLineJoin> _list = IterableExtensions.<KLineJoin>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
    final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
      @Override
      public void apply(final KLineJoin it) {
        it.setLineJoin(style);
      }
    };
    KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionLineJoin(final T rendering, final LineJoin style) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KLineJoin> _filter_1 = Iterables.<KLineJoin>filter(_filter, KLineJoin.class);
    List<KLineJoin> _list = IterableExtensions.<KLineJoin>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
    final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
      @Override
      public void apply(final KLineJoin it) {
        it.setSelection(true);
        it.setLineJoin(style);
      }
    };
    KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setLineJoin(final T rendering, final LineJoin style, final float miterLimit) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KLineJoin> _filter = Iterables.<KLineJoin>filter(_styles_1, KLineJoin.class);
    List<KLineJoin> _list = IterableExtensions.<KLineJoin>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
    final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
      @Override
      public void apply(final KLineJoin it) {
        it.setLineJoin(style);
        it.setMiterLimit(miterLimit);
      }
    };
    KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionLineJoin(final T rendering, final LineJoin style, final float miterLimit) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KLineJoin> _filter_1 = Iterables.<KLineJoin>filter(_filter, KLineJoin.class);
    List<KLineJoin> _list = IterableExtensions.<KLineJoin>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KLineJoin _createKLineJoin = this._kRenderingFactory.createKLineJoin();
    final Procedure1<KLineJoin> _function = new Procedure1<KLineJoin>() {
      @Override
      public void apply(final KLineJoin it) {
        it.setSelection(true);
        it.setLineJoin(style);
        it.setMiterLimit(miterLimit);
      }
    };
    KLineJoin _doubleArrow = ObjectExtensions.<KLineJoin>operator_doubleArrow(_createKLineJoin, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KRotation getRotation(final KRendering rendering) {
    KRotation _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KRotation> _filter = Iterables.<KRotation>filter(_styles, KRotation.class);
    KRotation _last = IterableExtensions.<KRotation>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KRotation _createKRotation = this._kRenderingFactory.createKRotation();
      _elvis = _createKRotation;
    }
    return _elvis;
  }
  
  public float getRotationValue(final KRendering rendering) {
    KRotation _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KRotation> _filter = Iterables.<KRotation>filter(_styles, KRotation.class);
    KRotation _last = IterableExtensions.<KRotation>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KRotation _createKRotation = this._kRenderingFactory.createKRotation();
      _elvis = _createKRotation;
    }
    return _elvis.getRotation();
  }
  
  public <T extends KRendering> T setRotation(final T rendering, final Float rotation) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KRotation> _filter = Iterables.<KRotation>filter(_styles_1, KRotation.class);
    List<KRotation> _list = IterableExtensions.<KRotation>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KRotation _createKRotation = this._kRenderingFactory.createKRotation();
    final Procedure1<KRotation> _function = new Procedure1<KRotation>() {
      @Override
      public void apply(final KRotation it) {
        it.setRotation((rotation).floatValue());
      }
    };
    KRotation _doubleArrow = ObjectExtensions.<KRotation>operator_doubleArrow(_createKRotation, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionRotation(final T rendering, final Float rotation) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KRotation> _filter_1 = Iterables.<KRotation>filter(_filter, KRotation.class);
    List<KRotation> _list = IterableExtensions.<KRotation>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KRotation _createKRotation = this._kRenderingFactory.createKRotation();
    final Procedure1<KRotation> _function = new Procedure1<KRotation>() {
      @Override
      public void apply(final KRotation it) {
        it.setSelection(true);
        it.setRotation((rotation).floatValue());
      }
    };
    KRotation _doubleArrow = ObjectExtensions.<KRotation>operator_doubleArrow(_createKRotation, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KBackground getBackground(final KRendering rendering) {
    KBackground _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KBackground> _filter = null;
    if (_styles!=null) {
      _filter=Iterables.<KBackground>filter(_styles, KBackground.class);
    }
    KBackground _last = IterableExtensions.<KBackground>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KBackground _createKBackground = this._kRenderingFactory.createKBackground();
      final Procedure1<KBackground> _function = new Procedure1<KBackground>() {
        @Override
        public void apply(final KBackground it) {
          KColor _createKColor = KRenderingExtensions.this._kRenderingFactory.createKColor();
          it.setColor(_createKColor);
        }
      };
      KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_createKBackground, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public <T extends KRendering> T setBackground(final T rendering, final KColoring<?> coloring) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _color = coloring.getColor();
    KColor _copy = EcoreUtil.<KColor>copy(_color);
    int _alpha = coloring.getAlpha();
    KBackground _setColor2 = _createKBackground.setColor2(_copy, _alpha);
    KColor _targetColor = coloring.getTargetColor();
    KColor _copy_1 = EcoreUtil.<KColor>copy(_targetColor);
    int _targetAlpha = coloring.getTargetAlpha();
    KBackground _setTargetColor2 = _setColor2.setTargetColor2(_copy_1, _targetAlpha);
    float _gradientAngle = coloring.getGradientAngle();
    KBackground _setGradientAngle2 = _setTargetColor2.setGradientAngle2(_gradientAngle);
    final Procedure1<KBackground> _function = new Procedure1<KBackground>() {
      @Override
      public void apply(final KBackground it) {
        String _modifierId = coloring.getModifierId();
        it.setModifierId(_modifierId);
        boolean _isPropagateToChildren = coloring.isPropagateToChildren();
        it.setPropagateToChildren(_isPropagateToChildren);
      }
    };
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setGradientAngle2, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackground(final T rendering, final KColoring<?> coloring) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _color = coloring.getColor();
    KColor _copy = EcoreUtil.<KColor>copy(_color);
    int _alpha = coloring.getAlpha();
    KBackground _setColor2 = _createKBackground.setColor2(_copy, _alpha);
    KColor _targetColor = coloring.getTargetColor();
    KColor _copy_1 = EcoreUtil.<KColor>copy(_targetColor);
    int _targetAlpha = coloring.getTargetAlpha();
    KBackground _setTargetColor2 = _setColor2.setTargetColor2(_copy_1, _targetAlpha);
    float _gradientAngle = coloring.getGradientAngle();
    KBackground _setGradientAngle2 = _setTargetColor2.setGradientAngle2(_gradientAngle);
    final Procedure1<KBackground> _function = new Procedure1<KBackground>() {
      @Override
      public void apply(final KBackground it) {
        String _modifierId = coloring.getModifierId();
        it.setModifierId(_modifierId);
        boolean _isPropagateToChildren = coloring.isPropagateToChildren();
        it.setPropagateToChildren(_isPropagateToChildren);
        it.setSelection(true);
      }
    };
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setGradientAngle2, _function);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackground(final T rendering, final KColor color) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _copy = EcoreUtil.<KColor>copy(color);
    KBackground _setColor2 = _createKBackground.setColor2(_copy);
    _styles_2.add(_setColor2);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackground(final T rendering, final KColor color) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _copy = EcoreUtil.<KColor>copy(color);
    KBackground _setColor2 = _createKBackground.setColor2(_copy);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor2, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackground(final T rendering, final KColor color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _copy = EcoreUtil.<KColor>copy(color);
    KBackground _setColor2 = _createKBackground.setColor2(_copy, alpha);
    _styles_2.add(_setColor2);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackground(final T rendering, final KColor color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KColor _copy = EcoreUtil.<KColor>copy(color);
    KBackground _setColor2 = _createKBackground.setColor2(_copy, alpha);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor2, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackground(final T rendering, final Colors color) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(color);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackground(final T rendering, final Colors color) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(color);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackground(final T rendering, final Colors color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(color, alpha);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackground(final T rendering, final Colors color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(color, alpha);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundColor(final T rendering, final int red, final int green, final int blue) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(red, green, blue);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundColor(final T rendering, final int red, final int green, final int blue) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(red, green, blue);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundColor(final T rendering, final int red, final int green, final int blue, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(red, green, blue, alpha);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundColor(final T rendering, final int red, final int green, final int blue, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColor = _createKBackground.setColor(red, green, blue, alpha);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundGradient(final T rendering, final KColor color1, final KColor color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, color2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundGradient(final T rendering, final KColor color1, final KColor color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, color2, angle);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundGradient(final T rendering, final KColor color1, final int alpha1, final KColor color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, alpha1, color2, alpha2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundGradient(final T rendering, final KColor color1, final int alpha1, final KColor color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, alpha1, color2, alpha2, angle);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundGradient(final T rendering, final Colors color1, final Colors color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, color2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundGradient(final T rendering, final Colors color1, final Colors color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, color2, angle);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundGradient(final T rendering, final Colors color1, final int alpha1, final Colors color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KBackground> _filter = Iterables.<KBackground>filter(_styles_1, KBackground.class);
    List<KBackground> _list = IterableExtensions.<KBackground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, alpha1, color2, alpha2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionBackgroundGradient(final T rendering, final Colors color1, final int alpha1, final Colors color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KBackground _createKBackground = this._kRenderingFactory.createKBackground();
    KBackground _setColors = _createKBackground.setColors(color1, alpha1, color2, alpha2, angle);
    KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setBackgroundInvisible(final T rendering, final boolean invisible) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KBackground _elvis = null;
        EList<KStyle> _styles = it.getStyles();
        Iterable<KBackground> _filter = null;
        if (_styles!=null) {
          _filter=Iterables.<KBackground>filter(_styles, KBackground.class);
        }
        KBackground _last = IterableExtensions.<KBackground>last(_filter);
        if (_last != null) {
          _elvis = _last;
        } else {
          KBackground _createKBackground = KRenderingExtensions.this._kRenderingFactory.createKBackground();
          final Procedure1<KBackground> _function = new Procedure1<KBackground>() {
            @Override
            public void apply(final KBackground it) {
              EList<KStyle> _styles = rendering.getStyles();
              _styles.add(it);
              KColor _createKColor = KRenderingExtensions.this._kRenderingFactory.createKColor();
              it.setColor(_createKColor);
            }
          };
          KBackground _doubleArrow = ObjectExtensions.<KBackground>operator_doubleArrow(_createKBackground, _function);
          _elvis = _doubleArrow;
        }
        final Procedure1<KBackground> _function_1 = new Procedure1<KBackground>() {
          @Override
          public void apply(final KBackground it) {
            Integer _xifexpression = null;
            if (invisible) {
              _xifexpression = Integer.valueOf(0);
            } else {
              KRenderingPackage _kRenderingPackage = KRenderingExtensions.this._kRenderingFactory.getKRenderingPackage();
              EAttribute _kColoring_Alpha = _kRenderingPackage.getKColoring_Alpha();
              Object _defaultValue = _kColoring_Alpha.getDefaultValue();
              _xifexpression = ((Integer) _defaultValue);
            }
            it.setAlpha((_xifexpression).intValue());
          }
        };
        ObjectExtensions.<KBackground>operator_doubleArrow(_elvis, _function_1);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  public KForeground getForeground(final KRendering rendering) {
    KForeground _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles, KForeground.class);
    KForeground _last = IterableExtensions.<KForeground>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KForeground _createKForeground = this._kRenderingFactory.createKForeground();
      final Procedure1<KForeground> _function = new Procedure1<KForeground>() {
        @Override
        public void apply(final KForeground it) {
          KColor _createKColor = KRenderingExtensions.this._kRenderingFactory.createKColor();
          it.setColor(_createKColor);
        }
      };
      KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_createKForeground, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public <T extends KRendering> T setForeground(final T rendering, final KColoring<?> coloring) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KColor _color = coloring.getColor();
    KColor _copy = EcoreUtil.<KColor>copy(_color);
    int _alpha = coloring.getAlpha();
    KColor _targetColor = coloring.getTargetColor();
    KColor _copy_1 = EcoreUtil.<KColor>copy(_targetColor);
    int _targetAlpha = coloring.getTargetAlpha();
    float _gradientAngle = coloring.getGradientAngle();
    KForeground _setColors = _createKForeground.setColors(_copy, _alpha, _copy_1, _targetAlpha, _gradientAngle);
    final Procedure1<KForeground> _function = new Procedure1<KForeground>() {
      @Override
      public void apply(final KForeground it) {
        String _modifierId = coloring.getModifierId();
        it.setModifierId(_modifierId);
        boolean _isPropagateToChildren = coloring.isPropagateToChildren();
        it.setPropagateToChildren(_isPropagateToChildren);
      }
    };
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColors, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForeground(final T rendering, final KColor color) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor2 = _createKForeground.setColor2(color);
    _styles_2.add(_setColor2);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForeground(final T rendering, final KColor color) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor2 = _createKForeground.setColor2(color);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor2, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForeground(final T rendering, final KColor color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor2 = _createKForeground.setColor2(color, alpha);
    _styles_2.add(_setColor2);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForeground(final T rendering, final KColor color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor2 = _createKForeground.setColor2(color, alpha);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor2, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForeground(final T rendering, final Colors color) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(color);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForeground(final T rendering, final Colors color) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(color);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForeground(final T rendering, final Colors color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(color, alpha);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForeground(final T rendering, final Colors color, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(color, alpha);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundColor(final T rendering, final int red, final int green, final int blue) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(red, green, blue);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundColor(final T rendering, final int red, final int green, final int blue) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(red, green, blue);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundColor(final T rendering, final int red, final int green, final int blue, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(red, green, blue, alpha);
    _styles_2.add(_setColor);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundColor(final T rendering, final int red, final int green, final int blue, final int alpha) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColor = _createKForeground.setColor(red, green, blue, alpha);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColor, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundGradient(final T rendering, final KColor color1, final KColor color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, color2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundGradient(final T rendering, final KColor color1, final KColor color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, color2, angle);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundGradient(final T rendering, final KColor color1, final int alpha1, final KColor color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, alpha1, color2, alpha2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundGradient(final T rendering, final KColor color1, final int alpha1, final KColor color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, alpha1, color2, alpha2, angle);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundGradient(final T rendering, final Colors color1, final Colors color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, color2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundGradient(final T rendering, final Colors color1, final Colors color2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, color2, angle);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundGradient(final T rendering, final Colors color1, final int alpha1, final Colors color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KForeground> _filter = Iterables.<KForeground>filter(_styles_1, KForeground.class);
    List<KForeground> _list = IterableExtensions.<KForeground>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, alpha1, color2, alpha2, angle);
    _styles_2.add(_setColors);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionForegroundGradient(final T rendering, final Colors color1, final int alpha1, final Colors color2, final int alpha2, final float angle) {
    EList<KStyle> _styles = rendering.getStyles();
    KForeground _createKForeground = this._kRenderingFactory.createKForeground();
    KForeground _setColors = _createKForeground.setColors(color1, alpha1, color2, alpha2, angle);
    KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_setColors, KRenderingExtensions.SET_SELECTION);
    _styles.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setForegroundInvisible(final T rendering, final boolean invisible) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KForeground _elvis = null;
        EList<KStyle> _styles = it.getStyles();
        Iterable<KForeground> _filter = null;
        if (_styles!=null) {
          _filter=Iterables.<KForeground>filter(_styles, KForeground.class);
        }
        KForeground _last = IterableExtensions.<KForeground>last(_filter);
        if (_last != null) {
          _elvis = _last;
        } else {
          KForeground _createKForeground = KRenderingExtensions.this._kRenderingFactory.createKForeground();
          final Procedure1<KForeground> _function = new Procedure1<KForeground>() {
            @Override
            public void apply(final KForeground it) {
              EList<KStyle> _styles = rendering.getStyles();
              _styles.add(it);
              KColor _createKColor = KRenderingExtensions.this._kRenderingFactory.createKColor();
              it.setColor(_createKColor);
            }
          };
          KForeground _doubleArrow = ObjectExtensions.<KForeground>operator_doubleArrow(_createKForeground, _function);
          _elvis = _doubleArrow;
        }
        final Procedure1<KForeground> _function_1 = new Procedure1<KForeground>() {
          @Override
          public void apply(final KForeground it) {
            Integer _xifexpression = null;
            if (invisible) {
              _xifexpression = Integer.valueOf(0);
            } else {
              KRenderingPackage _kRenderingPackage = KRenderingExtensions.this._kRenderingFactory.getKRenderingPackage();
              EAttribute _kColoring_Alpha = _kRenderingPackage.getKColoring_Alpha();
              Object _defaultValue = _kColoring_Alpha.getDefaultValue();
              _xifexpression = ((Integer) _defaultValue);
            }
            it.setAlpha((_xifexpression).intValue());
          }
        };
        ObjectExtensions.<KForeground>operator_doubleArrow(_elvis, _function_1);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  private final int defaultShadowSize = 4;
  
  public KShadow getShadow(final KRendering rendering) {
    KShadow _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KShadow> _filter = Iterables.<KShadow>filter(_styles, KShadow.class);
    KShadow _last = IterableExtensions.<KShadow>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      _elvis = null;
    }
    return _elvis;
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final Colors color) {
    KColor _createKColor = this._kRenderingFactory.createKColor();
    KColor _setColor = _createKColor.setColor(color);
    return this.<T>setShadow(rendering, _setColor);
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final Colors color, final float size) {
    KColor _createKColor = this._kRenderingFactory.createKColor();
    KColor _setColor = _createKColor.setColor(color);
    return this.<T>setShadow(rendering, _setColor, size);
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final Colors color, final float xOffset, final float yOffset) {
    KColor _createKColor = this._kRenderingFactory.createKColor();
    KColor _setColor = _createKColor.setColor(color);
    return this.<T>setShadow(rendering, _setColor, xOffset, yOffset);
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final KColor color) {
    return this.<T>setShadow(rendering, color, this.defaultShadowSize);
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final KColor color, final float size) {
    return this.<T>setShadow(rendering, color, size, size);
  }
  
  public <T extends KRendering> T setShadow(final T rendering, final KColor color, final float xOffset, final float yOffset) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KShadow> _filter = Iterables.<KShadow>filter(_styles_1, KShadow.class);
    List<KShadow> _list = IterableExtensions.<KShadow>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KShadow _createKShadow = this._kRenderingFactory.createKShadow();
    final Procedure1<KShadow> _function = new Procedure1<KShadow>() {
      @Override
      public void apply(final KShadow it) {
        EList<KStyle> _styles = rendering.getStyles();
        _styles.add(it);
        it.setColor(color);
        it.setXOffset(xOffset);
        it.setYOffset(yOffset);
      }
    };
    KShadow _doubleArrow = ObjectExtensions.<KShadow>operator_doubleArrow(_createKShadow, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KFontSize getFontSize(final KRendering rendering) {
    KFontSize _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontSize> _filter = Iterables.<KFontSize>filter(_styles, KFontSize.class);
    KFontSize _last = IterableExtensions.<KFontSize>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontSize _createKFontSize = this._kRenderingFactory.createKFontSize();
      final Procedure1<KFontSize> _function = new Procedure1<KFontSize>() {
        @Override
        public void apply(final KFontSize it) {
          it.setSize(10);
        }
      };
      KFontSize _doubleArrow = ObjectExtensions.<KFontSize>operator_doubleArrow(_createKFontSize, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public int getFontSizeValue(final KRendering rendering) {
    KFontSize _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontSize> _filter = Iterables.<KFontSize>filter(_styles, KFontSize.class);
    KFontSize _last = IterableExtensions.<KFontSize>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontSize _createKFontSize = this._kRenderingFactory.createKFontSize();
      final Procedure1<KFontSize> _function = new Procedure1<KFontSize>() {
        @Override
        public void apply(final KFontSize it) {
          it.setSize(10);
        }
      };
      KFontSize _doubleArrow = ObjectExtensions.<KFontSize>operator_doubleArrow(_createKFontSize, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getSize();
  }
  
  public <T extends KRendering> T setFontSize(final T rendering, final int size) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KFontSize> _filter = Iterables.<KFontSize>filter(_styles_1, KFontSize.class);
    List<KFontSize> _list = IterableExtensions.<KFontSize>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KFontSize _createKFontSize = this._kRenderingFactory.createKFontSize();
    final Procedure1<KFontSize> _function = new Procedure1<KFontSize>() {
      @Override
      public void apply(final KFontSize it) {
        it.setSize(size);
      }
    };
    KFontSize _doubleArrow = ObjectExtensions.<KFontSize>operator_doubleArrow(_createKFontSize, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KFontName getFontName(final KRendering rendering) {
    KFontName _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontName> _filter = Iterables.<KFontName>filter(_styles, KFontName.class);
    KFontName _last = IterableExtensions.<KFontName>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontName _createKFontName = this._kRenderingFactory.createKFontName();
      final Procedure1<KFontName> _function = new Procedure1<KFontName>() {
        @Override
        public void apply(final KFontName it) {
          it.setName("Arial");
        }
      };
      KFontName _doubleArrow = ObjectExtensions.<KFontName>operator_doubleArrow(_createKFontName, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public String getFontNameValue(final KRendering rendering) {
    KFontName _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontName> _filter = Iterables.<KFontName>filter(_styles, KFontName.class);
    KFontName _last = IterableExtensions.<KFontName>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontName _createKFontName = this._kRenderingFactory.createKFontName();
      final Procedure1<KFontName> _function = new Procedure1<KFontName>() {
        @Override
        public void apply(final KFontName it) {
          it.setName("Arial");
        }
      };
      KFontName _doubleArrow = ObjectExtensions.<KFontName>operator_doubleArrow(_createKFontName, _function);
      _elvis = _doubleArrow;
    }
    return _elvis.getName();
  }
  
  public <T extends KRendering> T setFontName(final T rendering, final String name) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KFontName> _filter = Iterables.<KFontName>filter(_styles_1, KFontName.class);
    List<KFontName> _list = IterableExtensions.<KFontName>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KFontName _createKFontName = this._kRenderingFactory.createKFontName();
    final Procedure1<KFontName> _function = new Procedure1<KFontName>() {
      @Override
      public void apply(final KFontName it) {
        it.setName(name);
      }
    };
    KFontName _doubleArrow = ObjectExtensions.<KFontName>operator_doubleArrow(_createKFontName, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KFontBold getFontBold(final KRendering rendering) {
    KFontBold _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontBold> _filter = Iterables.<KFontBold>filter(_styles, KFontBold.class);
    KFontBold _last = IterableExtensions.<KFontBold>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontBold _createKFontBold = this._kRenderingFactory.createKFontBold();
      _elvis = _createKFontBold;
    }
    return _elvis;
  }
  
  public boolean getFontBoldValue(final KRendering rendering) {
    KFontBold _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontBold> _filter = Iterables.<KFontBold>filter(_styles, KFontBold.class);
    KFontBold _last = IterableExtensions.<KFontBold>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontBold _createKFontBold = this._kRenderingFactory.createKFontBold();
      _elvis = _createKFontBold;
    }
    return _elvis.isBold();
  }
  
  public <T extends KRendering> T setFontBold(final T rendering, final boolean bold) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KFontBold> _filter = Iterables.<KFontBold>filter(_styles_1, KFontBold.class);
    List<KFontBold> _list = IterableExtensions.<KFontBold>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KFontBold _createKFontBold = this._kRenderingFactory.createKFontBold();
    final Procedure1<KFontBold> _function = new Procedure1<KFontBold>() {
      @Override
      public void apply(final KFontBold it) {
        it.setBold(bold);
      }
    };
    KFontBold _doubleArrow = ObjectExtensions.<KFontBold>operator_doubleArrow(_createKFontBold, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionFontBold(final T rendering, final boolean bold) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KFontBold> _filter_1 = Iterables.<KFontBold>filter(_filter, KFontBold.class);
    List<KFontBold> _list = IterableExtensions.<KFontBold>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KFontBold _createKFontBold = this._kRenderingFactory.createKFontBold();
    final Procedure1<KFontBold> _function = new Procedure1<KFontBold>() {
      @Override
      public void apply(final KFontBold it) {
        it.setSelection(true);
        it.setBold(bold);
      }
    };
    KFontBold _doubleArrow = ObjectExtensions.<KFontBold>operator_doubleArrow(_createKFontBold, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KFontItalic getFontItalic(final KRendering rendering) {
    KFontItalic _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontItalic> _filter = Iterables.<KFontItalic>filter(_styles, KFontItalic.class);
    KFontItalic _last = IterableExtensions.<KFontItalic>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontItalic _createKFontItalic = this._kRenderingFactory.createKFontItalic();
      _elvis = _createKFontItalic;
    }
    return _elvis;
  }
  
  public boolean getFontItalicValue(final KRendering rendering) {
    KFontItalic _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KFontItalic> _filter = Iterables.<KFontItalic>filter(_styles, KFontItalic.class);
    KFontItalic _last = IterableExtensions.<KFontItalic>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KFontItalic _createKFontItalic = this._kRenderingFactory.createKFontItalic();
      _elvis = _createKFontItalic;
    }
    return _elvis.isItalic();
  }
  
  public <T extends KRendering> T setFontItalic(final T rendering, final boolean italic) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KFontItalic> _filter = Iterables.<KFontItalic>filter(_styles_1, KFontItalic.class);
    List<KFontItalic> _list = IterableExtensions.<KFontItalic>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KFontItalic _createKFontItalic = this._kRenderingFactory.createKFontItalic();
    final Procedure1<KFontItalic> _function = new Procedure1<KFontItalic>() {
      @Override
      public void apply(final KFontItalic it) {
        it.setItalic(italic);
      }
    };
    KFontItalic _doubleArrow = ObjectExtensions.<KFontItalic>operator_doubleArrow(_createKFontItalic, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KTextUnderline getTextUnderline(final KRendering rendering) {
    KTextUnderline _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextUnderline> _filter = Iterables.<KTextUnderline>filter(_styles, KTextUnderline.class);
    KTextUnderline _last = IterableExtensions.<KTextUnderline>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextUnderline _createKTextUnderline = this._kRenderingFactory.createKTextUnderline();
      _elvis = _createKTextUnderline;
    }
    return _elvis;
  }
  
  public Underline getTextUnderlineValue(final KRendering rendering) {
    KTextUnderline _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextUnderline> _filter = Iterables.<KTextUnderline>filter(_styles, KTextUnderline.class);
    KTextUnderline _last = IterableExtensions.<KTextUnderline>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextUnderline _createKTextUnderline = this._kRenderingFactory.createKTextUnderline();
      _elvis = _createKTextUnderline;
    }
    return _elvis.getUnderline();
  }
  
  public KColor getTextUnderlineColorValue(final KRendering rendering) {
    KTextUnderline _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextUnderline> _filter = Iterables.<KTextUnderline>filter(_styles, KTextUnderline.class);
    KTextUnderline _last = IterableExtensions.<KTextUnderline>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextUnderline _createKTextUnderline = this._kRenderingFactory.createKTextUnderline();
      _elvis = _createKTextUnderline;
    }
    return _elvis.getColor();
  }
  
  public <T extends KRendering> T setTextUnderline(final T rendering, final Underline underline) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KTextUnderline> _filter = Iterables.<KTextUnderline>filter(_styles_1, KTextUnderline.class);
    List<KTextUnderline> _list = IterableExtensions.<KTextUnderline>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KTextUnderline _createKTextUnderline = this._kRenderingFactory.createKTextUnderline();
    final Procedure1<KTextUnderline> _function = new Procedure1<KTextUnderline>() {
      @Override
      public void apply(final KTextUnderline it) {
        it.setUnderline(underline);
      }
    };
    KTextUnderline _doubleArrow = ObjectExtensions.<KTextUnderline>operator_doubleArrow(_createKTextUnderline, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionTextUnderline(final T rendering, final Underline underline) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KTextUnderline> _filter_1 = Iterables.<KTextUnderline>filter(_filter, KTextUnderline.class);
    List<KTextUnderline> _list = IterableExtensions.<KTextUnderline>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KTextUnderline _createKTextUnderline = this._kRenderingFactory.createKTextUnderline();
    final Procedure1<KTextUnderline> _function = new Procedure1<KTextUnderline>() {
      @Override
      public void apply(final KTextUnderline it) {
        it.setSelection(true);
        it.setUnderline(underline);
      }
    };
    KTextUnderline _doubleArrow = ObjectExtensions.<KTextUnderline>operator_doubleArrow(_createKTextUnderline, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setTextUnderlineColor(final T rendering, final KColor color) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KTextUnderline _elvis = null;
        EList<KStyle> _styles = rendering.getStyles();
        Iterable<KTextUnderline> _filter = Iterables.<KTextUnderline>filter(_styles, KTextUnderline.class);
        KTextUnderline _last = IterableExtensions.<KTextUnderline>last(_filter);
        if (_last != null) {
          _elvis = _last;
        } else {
          KTextUnderline _createKTextUnderline = KRenderingExtensions.this._kRenderingFactory.createKTextUnderline();
          _elvis = _createKTextUnderline;
        }
        final Procedure1<KTextUnderline> _function = new Procedure1<KTextUnderline>() {
          @Override
          public void apply(final KTextUnderline it) {
            it.setColor(color);
          }
        };
        ObjectExtensions.<KTextUnderline>operator_doubleArrow(_elvis, _function);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  public <T extends KRendering> T setSelectionTextUnderlineColor(final T rendering, final KColor color) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KTextUnderline _elvis = null;
        EList<KStyle> _styles = rendering.getStyles();
        Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
        Iterable<KTextUnderline> _filter_1 = Iterables.<KTextUnderline>filter(_filter, KTextUnderline.class);
        KTextUnderline _last = IterableExtensions.<KTextUnderline>last(_filter_1);
        if (_last != null) {
          _elvis = _last;
        } else {
          KTextUnderline _createKTextUnderline = KRenderingExtensions.this._kRenderingFactory.createKTextUnderline();
          _elvis = _createKTextUnderline;
        }
        final Procedure1<KTextUnderline> _function = new Procedure1<KTextUnderline>() {
          @Override
          public void apply(final KTextUnderline it) {
            it.setSelection(true);
            it.setColor(color);
          }
        };
        ObjectExtensions.<KTextUnderline>operator_doubleArrow(_elvis, _function);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  public KTextStrikeout getTextStrikeout(final KRendering rendering) {
    KTextStrikeout _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextStrikeout> _filter = Iterables.<KTextStrikeout>filter(_styles, KTextStrikeout.class);
    KTextStrikeout _last = IterableExtensions.<KTextStrikeout>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextStrikeout _createKTextStrikeout = this._kRenderingFactory.createKTextStrikeout();
      _elvis = _createKTextStrikeout;
    }
    return _elvis;
  }
  
  public boolean getTextStrikeoutValue(final KRendering rendering) {
    KTextStrikeout _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextStrikeout> _filter = Iterables.<KTextStrikeout>filter(_styles, KTextStrikeout.class);
    KTextStrikeout _last = IterableExtensions.<KTextStrikeout>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextStrikeout _createKTextStrikeout = this._kRenderingFactory.createKTextStrikeout();
      _elvis = _createKTextStrikeout;
    }
    return (_elvis.getStruckOut()).booleanValue();
  }
  
  public KColor getTextStrikeoutColorValue(final KRendering rendering) {
    KTextStrikeout _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KTextStrikeout> _filter = Iterables.<KTextStrikeout>filter(_styles, KTextStrikeout.class);
    KTextStrikeout _last = IterableExtensions.<KTextStrikeout>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KTextStrikeout _createKTextStrikeout = this._kRenderingFactory.createKTextStrikeout();
      _elvis = _createKTextStrikeout;
    }
    return _elvis.getColor();
  }
  
  public <T extends KRendering> T setTextStrikeout(final T rendering, final boolean struckOut) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KTextStrikeout> _filter = Iterables.<KTextStrikeout>filter(_styles_1, KTextStrikeout.class);
    List<KTextStrikeout> _list = IterableExtensions.<KTextStrikeout>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KTextStrikeout _createKTextStrikeout = this._kRenderingFactory.createKTextStrikeout();
    final Procedure1<KTextStrikeout> _function = new Procedure1<KTextStrikeout>() {
      @Override
      public void apply(final KTextStrikeout it) {
        it.setStruckOut(Boolean.valueOf(struckOut));
      }
    };
    KTextStrikeout _doubleArrow = ObjectExtensions.<KTextStrikeout>operator_doubleArrow(_createKTextStrikeout, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setSelectionTextStrikeout(final T rendering, final boolean struckOut) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles_1, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
    Iterable<KTextStrikeout> _filter_1 = Iterables.<KTextStrikeout>filter(_filter, KTextStrikeout.class);
    List<KTextStrikeout> _list = IterableExtensions.<KTextStrikeout>toList(_filter_1);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KTextStrikeout _createKTextStrikeout = this._kRenderingFactory.createKTextStrikeout();
    final Procedure1<KTextStrikeout> _function = new Procedure1<KTextStrikeout>() {
      @Override
      public void apply(final KTextStrikeout it) {
        it.setSelection(true);
        it.setStruckOut(Boolean.valueOf(struckOut));
      }
    };
    KTextStrikeout _doubleArrow = ObjectExtensions.<KTextStrikeout>operator_doubleArrow(_createKTextStrikeout, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setTextStrikeoutColor(final T rendering, final KColor color) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KTextStrikeout _elvis = null;
        EList<KStyle> _styles = rendering.getStyles();
        Iterable<KTextStrikeout> _filter = Iterables.<KTextStrikeout>filter(_styles, KTextStrikeout.class);
        KTextStrikeout _last = IterableExtensions.<KTextStrikeout>last(_filter);
        if (_last != null) {
          _elvis = _last;
        } else {
          KTextStrikeout _createKTextStrikeout = KRenderingExtensions.this._kRenderingFactory.createKTextStrikeout();
          _elvis = _createKTextStrikeout;
        }
        final Procedure1<KTextStrikeout> _function = new Procedure1<KTextStrikeout>() {
          @Override
          public void apply(final KTextStrikeout it) {
            it.setColor(color);
          }
        };
        ObjectExtensions.<KTextStrikeout>operator_doubleArrow(_elvis, _function);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  public <T extends KRendering> T setSelectionTextStrikeoutColor(final T rendering, final KColor color) {
    final Procedure1<T> _function = new Procedure1<T>() {
      @Override
      public void apply(final T it) {
        KTextStrikeout _elvis = null;
        EList<KStyle> _styles = rendering.getStyles();
        Iterable<KStyle> _filter = IterableExtensions.<KStyle>filter(_styles, ((Function1<? super KStyle, Boolean>)KRenderingExtensions.IS_SELECTION));
        Iterable<KTextStrikeout> _filter_1 = Iterables.<KTextStrikeout>filter(_filter, KTextStrikeout.class);
        KTextStrikeout _last = IterableExtensions.<KTextStrikeout>last(_filter_1);
        if (_last != null) {
          _elvis = _last;
        } else {
          KTextStrikeout _createKTextStrikeout = KRenderingExtensions.this._kRenderingFactory.createKTextStrikeout();
          _elvis = _createKTextStrikeout;
        }
        final Procedure1<KTextStrikeout> _function = new Procedure1<KTextStrikeout>() {
          @Override
          public void apply(final KTextStrikeout it) {
            it.setColor(color);
          }
        };
        ObjectExtensions.<KTextStrikeout>operator_doubleArrow(_elvis, _function);
      }
    };
    return ObjectExtensions.<T>operator_doubleArrow(rendering, _function);
  }
  
  public final HorizontalAlignment H_LEFT = HorizontalAlignment.LEFT;
  
  public final HorizontalAlignment H_CENTRAL = HorizontalAlignment.CENTER;
  
  public final HorizontalAlignment H_RIGHT = HorizontalAlignment.RIGHT;
  
  public KHorizontalAlignment getHorizontalAlignment(final KRendering rendering) {
    KHorizontalAlignment _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KHorizontalAlignment> _filter = Iterables.<KHorizontalAlignment>filter(_styles, KHorizontalAlignment.class);
    KHorizontalAlignment _last = IterableExtensions.<KHorizontalAlignment>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KHorizontalAlignment _createKHorizontalAlignment = this._kRenderingFactory.createKHorizontalAlignment();
      final Procedure1<KHorizontalAlignment> _function = new Procedure1<KHorizontalAlignment>() {
        @Override
        public void apply(final KHorizontalAlignment it) {
          it.setHorizontalAlignment(KRenderingExtensions.this.H_CENTRAL);
        }
      };
      KHorizontalAlignment _doubleArrow = ObjectExtensions.<KHorizontalAlignment>operator_doubleArrow(_createKHorizontalAlignment, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public <T extends KRendering> T setHorizontalAlignment(final T rendering, final HorizontalAlignment ha) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KHorizontalAlignment> _filter = Iterables.<KHorizontalAlignment>filter(_styles_1, KHorizontalAlignment.class);
    List<KHorizontalAlignment> _list = IterableExtensions.<KHorizontalAlignment>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KHorizontalAlignment _createKHorizontalAlignment = this._kRenderingFactory.createKHorizontalAlignment();
    final Procedure1<KHorizontalAlignment> _function = new Procedure1<KHorizontalAlignment>() {
      @Override
      public void apply(final KHorizontalAlignment it) {
        it.setHorizontalAlignment(ha);
      }
    };
    KHorizontalAlignment _doubleArrow = ObjectExtensions.<KHorizontalAlignment>operator_doubleArrow(_createKHorizontalAlignment, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public final VerticalAlignment V_TOP = VerticalAlignment.TOP;
  
  public final VerticalAlignment V_CENTRAL = VerticalAlignment.CENTER;
  
  public final VerticalAlignment V_BOTTOM = VerticalAlignment.BOTTOM;
  
  public KVerticalAlignment getVerticalAlignment(final KRendering rendering) {
    KVerticalAlignment _elvis = null;
    EList<KStyle> _styles = rendering.getStyles();
    Iterable<KVerticalAlignment> _filter = Iterables.<KVerticalAlignment>filter(_styles, KVerticalAlignment.class);
    KVerticalAlignment _last = IterableExtensions.<KVerticalAlignment>last(_filter);
    if (_last != null) {
      _elvis = _last;
    } else {
      KVerticalAlignment _createKVerticalAlignment = this._kRenderingFactory.createKVerticalAlignment();
      final Procedure1<KVerticalAlignment> _function = new Procedure1<KVerticalAlignment>() {
        @Override
        public void apply(final KVerticalAlignment it) {
          it.setVerticalAlignment(KRenderingExtensions.this.V_CENTRAL);
        }
      };
      KVerticalAlignment _doubleArrow = ObjectExtensions.<KVerticalAlignment>operator_doubleArrow(_createKVerticalAlignment, _function);
      _elvis = _doubleArrow;
    }
    return _elvis;
  }
  
  public <T extends KRendering> T setVerticalAlignment(final T rendering, final VerticalAlignment va) {
    EList<KStyle> _styles = rendering.getStyles();
    EList<KStyle> _styles_1 = rendering.getStyles();
    Iterable<KVerticalAlignment> _filter = Iterables.<KVerticalAlignment>filter(_styles_1, KVerticalAlignment.class);
    List<KVerticalAlignment> _list = IterableExtensions.<KVerticalAlignment>toList(_filter);
    _styles.removeAll(_list);
    EList<KStyle> _styles_2 = rendering.getStyles();
    KVerticalAlignment _createKVerticalAlignment = this._kRenderingFactory.createKVerticalAlignment();
    final Procedure1<KVerticalAlignment> _function = new Procedure1<KVerticalAlignment>() {
      @Override
      public void apply(final KVerticalAlignment it) {
        it.setVerticalAlignment(va);
      }
    };
    KVerticalAlignment _doubleArrow = ObjectExtensions.<KVerticalAlignment>operator_doubleArrow(_createKVerticalAlignment, _function);
    _styles_2.add(_doubleArrow);
    return rendering;
  }
  
  public KPolyline from(final KPolyline polyline, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    final Procedure1<KPolyline> _function = new Procedure1<KPolyline>() {
      @Override
      public void apply(final KPolyline it) {
        EList<KPosition> _points = it.getPoints();
        KPosition _createKPosition = KRenderingExtensions.this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
        _points.add(_createKPosition);
      }
    };
    return ObjectExtensions.<KPolyline>operator_doubleArrow(polyline, _function);
  }
  
  public KPolyline to(final KPolyline polyline, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    final Procedure1<KPolyline> _function = new Procedure1<KPolyline>() {
      @Override
      public void apply(final KPolyline it) {
        EList<KPosition> _points = it.getPoints();
        KPosition _createKPosition = KRenderingExtensions.this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
        _points.add(_createKPosition);
      }
    };
    return ObjectExtensions.<KPolyline>operator_doubleArrow(polyline, _function);
  }
  
  public <T extends KRendering> T setAreaPlacementData(final T rendering, final KPosition topLeft, final KPosition bottomRight) {
    KAreaPlacementData _createKAreaPlacementData = this._kRenderingFactory.createKAreaPlacementData();
    final Procedure1<KAreaPlacementData> _function = new Procedure1<KAreaPlacementData>() {
      @Override
      public void apply(final KAreaPlacementData it) {
        it.setTopLeft(topLeft);
        it.setBottomRight(bottomRight);
      }
    };
    KAreaPlacementData _doubleArrow = ObjectExtensions.<KAreaPlacementData>operator_doubleArrow(_createKAreaPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  public KAreaPlacementData setAreaPlacementData(final KRendering rendering) {
    KAreaPlacementData _createKAreaPlacementData = this._kRenderingFactory.createKAreaPlacementData();
    final Procedure1<KAreaPlacementData> _function = new Procedure1<KAreaPlacementData>() {
      @Override
      public void apply(final KAreaPlacementData it) {
        rendering.setPlacementData(it);
      }
    };
    return ObjectExtensions.<KAreaPlacementData>operator_doubleArrow(_createKAreaPlacementData, _function);
  }
  
  public KAreaPlacementData from(final KAreaPlacementData placementData, final KPosition topLeft) {
    final Procedure1<KAreaPlacementData> _function = new Procedure1<KAreaPlacementData>() {
      @Override
      public void apply(final KAreaPlacementData it) {
        it.setTopLeft(topLeft);
      }
    };
    return ObjectExtensions.<KAreaPlacementData>operator_doubleArrow(placementData, _function);
  }
  
  public KAreaPlacementData from(final KAreaPlacementData placementData, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
    return this.from(placementData, _createKPosition);
  }
  
  public KRendering to(final KAreaPlacementData placementData, final KPosition bottomRight) {
    placementData.setBottomRight(bottomRight);
    EObject _eContainer = placementData.eContainer();
    return ((KRendering) _eContainer);
  }
  
  public KRendering to(final KAreaPlacementData placementData, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
    return this.to(placementData, _createKPosition);
  }
  
  public <T extends KRendering> T setSurroundingSpace(final T rendering, final float abs, final float rel) {
    KPosition _createKPosition = this.createKPosition(this.LEFT, abs, rel, this.TOP, abs, rel);
    KPosition _createKPosition_1 = this.createKPosition(this.RIGHT, abs, rel, this.BOTTOM, abs, rel);
    return this.<T>setAreaPlacementData(rendering, _createKPosition, _createKPosition_1);
  }
  
  public <T extends KRendering> T setSurroundingSpace(final T rendering, final float hAbs, final float hRel, final float vAbs, final float vRel) {
    KPosition _createKPosition = this.createKPosition(this.LEFT, hAbs, hRel, this.TOP, vAbs, vRel);
    KPosition _createKPosition_1 = this.createKPosition(this.RIGHT, hAbs, hRel, this.BOTTOM, vAbs, vRel);
    return this.<T>setAreaPlacementData(rendering, _createKPosition, _createKPosition_1);
  }
  
  /**
   * Adds a grid placement to the rendering element with the specified spacing.
   */
  public <T extends KRendering> T setGridPlacementData(final T rendering, final float minCellWidth, final float minCellHeight, final KPosition topLeft, final KPosition bottomRight) {
    KGridPlacementData _createKGridPlacementData = this._kRenderingFactory.createKGridPlacementData();
    final Procedure1<KGridPlacementData> _function = new Procedure1<KGridPlacementData>() {
      @Override
      public void apply(final KGridPlacementData it) {
        it.setMinCellWidth(minCellWidth);
        it.setMinCellHeight(minCellHeight);
        it.setTopLeft(topLeft);
        it.setBottomRight(bottomRight);
      }
    };
    KGridPlacementData _doubleArrow = ObjectExtensions.<KGridPlacementData>operator_doubleArrow(_createKGridPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  /**
   * Adds a grid placement to the rendering element with the specified spacing.
   * 
   * @example
   * rectangle.setGridPlacementData => [ cell |
   *  cell.from(LEFT, padding, 0, TOP, padding - 2, 0).to(RIGHT, padding, 0, BOTTOM, 0, 0)
   *  cell.minCellHeight = propBarHeight
   *  cell.flexibleHeight = false
   * ]
   */
  public KGridPlacementData setGridPlacementData(final KRendering rendering) {
    KGridPlacementData _createKGridPlacementData = this._kRenderingFactory.createKGridPlacementData();
    final Procedure1<KGridPlacementData> _function = new Procedure1<KGridPlacementData>() {
      @Override
      public void apply(final KGridPlacementData it) {
        rendering.setPlacementData(it);
      }
    };
    return ObjectExtensions.<KGridPlacementData>operator_doubleArrow(_createKGridPlacementData, _function);
  }
  
  /**
   * Adds a grid placement to the rendering element with the specified spacing.
   */
  public KGridPlacementData setGridPlacementData(final KRendering rendering, final float minCellWidth, final float minCellHeight) {
    KGridPlacementData _createKGridPlacementData = this._kRenderingFactory.createKGridPlacementData();
    final Procedure1<KGridPlacementData> _function = new Procedure1<KGridPlacementData>() {
      @Override
      public void apply(final KGridPlacementData it) {
        rendering.setPlacementData(it);
        it.setMinCellWidth(minCellWidth);
        it.setMinCellHeight(minCellHeight);
      }
    };
    return ObjectExtensions.<KGridPlacementData>operator_doubleArrow(_createKGridPlacementData, _function);
  }
  
  public <T extends KRendering> T setSurroundingSpaceGrid(final T rendering, final float abs, final float rel) {
    KPosition _createKPosition = this.createKPosition(this.LEFT, abs, rel, this.TOP, abs, rel);
    KPosition _createKPosition_1 = this.createKPosition(this.RIGHT, abs, rel, this.BOTTOM, abs, rel);
    return this.<T>setGridPlacementData(rendering, 0f, 0f, _createKPosition, _createKPosition_1);
  }
  
  public <T extends KRendering> T setSurroundingSpaceGrid(final T rendering, final float hAbs, final float hRel, final float vAbs, final float vRel) {
    KPosition _createKPosition = this.createKPosition(this.LEFT, hAbs, hRel, this.TOP, vAbs, vRel);
    KPosition _createKPosition_1 = this.createKPosition(this.RIGHT, hAbs, hRel, this.BOTTOM, vAbs, vRel);
    return this.<T>setGridPlacementData(rendering, 0f, 0f, _createKPosition, _createKPosition_1);
  }
  
  public KGridPlacement from(final KGridPlacement placement, final KPosition topLeft) {
    placement.setTopLeft(topLeft);
    return placement;
  }
  
  public KGridPlacement from(final KGridPlacement placement, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
    this.from(placement, _createKPosition);
    return placement;
  }
  
  public KContainerRendering to(final KGridPlacement placement, final KPosition bottomRight) {
    placement.setBottomRight(bottomRight);
    final EObject cont = placement.eContainer();
    KContainerRendering _xifexpression = null;
    if ((cont instanceof KContainerRendering)) {
      _xifexpression = ((KContainerRendering) cont);
    } else {
      _xifexpression = null;
    }
    return _xifexpression;
  }
  
  public KContainerRendering to(final KGridPlacement placement, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
    this.to(placement, _createKPosition);
    final EObject cont = placement.eContainer();
    KContainerRendering _xifexpression = null;
    if ((cont instanceof KContainerRendering)) {
      _xifexpression = ((KContainerRendering) cont);
    } else {
      _xifexpression = null;
    }
    return _xifexpression;
  }
  
  public <T extends KRendering> T setLeftTopAlignedPointPlacementData(final T rendering, final float leftMargin, final float topMargin, final float rightMargin, final float bottomMargin) {
    return this.<T>setLeftTopAlignedPointPlacementData(rendering, leftMargin, 0, topMargin, 0, rightMargin, bottomMargin, 0, 0);
  }
  
  public <T extends KRendering> T setLeftTopAlignedPointPlacementData(final T rendering, final float absoluteLR, final float relativeLR, final float absoluteTB, final float relativeTB, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    return this.<T>setPointPlacementData(rendering, 
      this.LEFT, absoluteLR, relativeLR, this.TOP, absoluteTB, relativeTB, this.H_LEFT, this.V_TOP, 0, 0, 0, 0);
  }
  
  public <T extends KRendering> T setLeftBottomAlignedPointPlacementData(final T rendering, final float leftMargin, final float topMargin, final float rightMargin, final float bottomMargin) {
    return this.<T>setLeftTopAlignedPointPlacementData(rendering, leftMargin, 0, bottomMargin, 0, rightMargin, topMargin, 0, 0);
  }
  
  public <T extends KRendering> T setLeftBottomAlignedPointPlacementData(final T rendering, final float absoluteLR, final float relativeLR, final float absoluteTB, final float relativeTB, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    return this.<T>setPointPlacementData(rendering, 
      this.LEFT, absoluteLR, relativeLR, this.BOTTOM, absoluteTB, relativeTB, this.H_LEFT, this.V_BOTTOM, 0, 0, 0, 0);
  }
  
  public <T extends KRendering> T setRightTopAlignedPointPlacementData(final T rendering, final float leftMargin, final float topMargin, final float rightMargin, final float bottomMargin) {
    return this.<T>setLeftTopAlignedPointPlacementData(rendering, rightMargin, 0, topMargin, 0, leftMargin, bottomMargin, 0, 0);
  }
  
  public <T extends KRendering> T setRightTopAlignedPointPlacementData(final T rendering, final float absoluteLR, final float relativeLR, final float absoluteTB, final float relativeTB, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    return this.<T>setPointPlacementData(rendering, 
      this.RIGHT, absoluteLR, relativeLR, this.TOP, absoluteTB, relativeTB, this.H_RIGHT, this.V_TOP, 0, 0, 0, 0);
  }
  
  public <T extends KRendering> T setRightBottomAlignedPointPlacementData(final T rendering, final float leftMargin, final float topMargin, final float rightMargin, final float bottomMargin) {
    return this.<T>setLeftTopAlignedPointPlacementData(rendering, rightMargin, 0, bottomMargin, 0, leftMargin, topMargin, 0, 0);
  }
  
  public <T extends KRendering> T setRightBottomAlignedPointPlacementData(final T rendering, final float absoluteLR, final float relativeLR, final float absoluteTB, final float relativeTB, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    return this.<T>setPointPlacementData(rendering, 
      this.RIGHT, absoluteLR, relativeLR, this.BOTTOM, absoluteTB, relativeTB, this.H_RIGHT, this.V_BOTTOM, 0, 0, 0, 0);
  }
  
  public <T extends KRendering> T setPointPlacementData(final T rendering, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPointPlacementData _createKPointPlacementData = this._kRenderingFactory.createKPointPlacementData();
    final Procedure1<KPointPlacementData> _function = new Procedure1<KPointPlacementData>() {
      @Override
      public void apply(final KPointPlacementData it) {
        KPosition _createKPosition = KRenderingExtensions.this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
        it.setReferencePoint(_createKPosition);
      }
    };
    KPointPlacementData _doubleArrow = ObjectExtensions.<KPointPlacementData>operator_doubleArrow(_createKPointPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> KPointPlacementData setPointPlacementData(final T rendering) {
    KPointPlacementData _createKPointPlacementData = this._kRenderingFactory.createKPointPlacementData();
    final Procedure1<KPointPlacementData> _function = new Procedure1<KPointPlacementData>() {
      @Override
      public void apply(final KPointPlacementData it) {
        rendering.setPlacementData(it);
      }
    };
    return ObjectExtensions.<KPointPlacementData>operator_doubleArrow(_createKPointPlacementData, _function);
  }
  
  public KPointPlacementData at(final KPointPlacementData ppd, final KPosition pos) {
    ppd.setReferencePoint(pos);
    return ppd;
  }
  
  public KPointPlacementData at(final KPointPlacementData ppd, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
    ppd.setReferencePoint(_createKPosition);
    return ppd;
  }
  
  public KPointPlacementData setAlignment(final KPointPlacementData ppd, final HorizontalAlignment horizontalAlignment, final VerticalAlignment verticalAlignment) {
    ppd.setHorizontalAlignment(horizontalAlignment);
    ppd.setVerticalAlignment(verticalAlignment);
    return ppd;
  }
  
  public KPointPlacementData setMargins(final KPointPlacementData ppd, final float horizontalMargin, final float verticalMargin) {
    ppd.setHorizontalMargin(horizontalMargin);
    ppd.setVerticalMargin(verticalMargin);
    return ppd;
  }
  
  public KPointPlacementData setMinimalSize(final KPointPlacementData ppd, final float minWidth, final float minHeight) {
    ppd.setMinWidth(minWidth);
    ppd.setMinHeight(minHeight);
    return ppd;
  }
  
  public KRendering finish(final KPointPlacementData ppd) {
    EObject _eContainer = ppd.eContainer();
    return ((KRendering) _eContainer);
  }
  
  public <T extends KRendering> T setPointPlacementData(final T rendering, final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB, final HorizontalAlignment horizontalAlignment, final VerticalAlignment verticalAlignment, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    KPointPlacementData _createKPointPlacementData = this._kRenderingFactory.createKPointPlacementData();
    final Procedure1<KPointPlacementData> _function = new Procedure1<KPointPlacementData>() {
      @Override
      public void apply(final KPointPlacementData it) {
        KPosition _createKPosition = KRenderingExtensions.this.createKPosition(px, absoluteLR, relativeLR, py, absoluteTB, relativeTB);
        it.setReferencePoint(_createKPosition);
        it.setHorizontalAlignment(horizontalAlignment);
        it.setVerticalAlignment(verticalAlignment);
        it.setHorizontalMargin(horizontalMargin);
        it.setVerticalMargin(verticalMargin);
        it.setMinWidth(minWidth);
        it.setMinHeight(minHeight);
      }
    };
    KPointPlacementData _doubleArrow = ObjectExtensions.<KPointPlacementData>operator_doubleArrow(_createKPointPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setPointPlacementData(final T rendering, final KPosition referencePoint, final HorizontalAlignment horizontalAlignment, final VerticalAlignment verticalAlignment, final float horizontalMargin, final float verticalMargin, final float minWidth, final float minHeight) {
    KPointPlacementData _createKPointPlacementData = this._kRenderingFactory.createKPointPlacementData();
    final Procedure1<KPointPlacementData> _function = new Procedure1<KPointPlacementData>() {
      @Override
      public void apply(final KPointPlacementData it) {
        it.setReferencePoint(referencePoint);
        it.setHorizontalAlignment(horizontalAlignment);
        it.setVerticalAlignment(verticalAlignment);
        it.setHorizontalMargin(horizontalMargin);
        it.setVerticalMargin(verticalMargin);
        it.setMinWidth(minWidth);
        it.setMinHeight(minHeight);
      }
    };
    KPointPlacementData _doubleArrow = ObjectExtensions.<KPointPlacementData>operator_doubleArrow(_createKPointPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  public <T extends KRendering> T setDecoratorPlacementData(final T rendering, final float width, final float height, final float posAbsolute, final float posRelative, final boolean rotateWithLine) {
    KDecoratorPlacementData _createKDecoratorPlacementData = this._kRenderingFactory.createKDecoratorPlacementData();
    final Procedure1<KDecoratorPlacementData> _function = new Procedure1<KDecoratorPlacementData>() {
      @Override
      public void apply(final KDecoratorPlacementData it) {
        it.setWidth(width);
        it.setHeight(height);
        it.setAbsolute(posAbsolute);
        it.setRelative(posRelative);
        it.setRotateWithLine(rotateWithLine);
        it.setXOffset(((-width) / 2));
        it.setYOffset(((-height) / 2));
      }
    };
    KDecoratorPlacementData _doubleArrow = ObjectExtensions.<KDecoratorPlacementData>operator_doubleArrow(_createKDecoratorPlacementData, _function);
    rendering.setPlacementData(_doubleArrow);
    return rendering;
  }
  
  public final PositionReferenceX LEFT = PositionReferenceX.LEFT;
  
  public final PositionReferenceX RIGHT = PositionReferenceX.RIGHT;
  
  public final PositionReferenceY TOP = PositionReferenceY.TOP;
  
  public final PositionReferenceY BOTTOM = PositionReferenceY.BOTTOM;
  
  public KPosition createKPosition(final PositionReferenceX px, final float absoluteLR, final float relativeLR, final PositionReferenceY py, final float absoluteTB, final float relativeTB) {
    KPosition _createKPosition = this._kRenderingFactory.createKPosition();
    final Procedure1<KPosition> _function = new Procedure1<KPosition>() {
      @Override
      public void apply(final KPosition it) {
        KXPosition<?> _switchResult = null;
        if (px != null) {
          switch (px) {
            case LEFT:
              _switchResult = KRenderingExtensions.this._kRenderingFactory.createKLeftPosition();
              break;
            case RIGHT:
              _switchResult = KRenderingExtensions.this._kRenderingFactory.createKRightPosition();
              break;
            default:
              break;
          }
        }
        final Procedure1<KXPosition<?>> _function = new Procedure1<KXPosition<?>>() {
          @Override
          public void apply(final KXPosition<?> it) {
            it.setAbsolute(absoluteLR);
            it.setRelative(relativeLR);
          }
        };
        KXPosition<?> _doubleArrow = ObjectExtensions.<KXPosition<?>>operator_doubleArrow(_switchResult, _function);
        it.setX(_doubleArrow);
        KYPosition<?> _switchResult_1 = null;
        if (py != null) {
          switch (py) {
            case TOP:
              _switchResult_1 = KRenderingExtensions.this._kRenderingFactory.createKTopPosition();
              break;
            case BOTTOM:
              _switchResult_1 = KRenderingExtensions.this._kRenderingFactory.createKBottomPosition();
              break;
            default:
              break;
          }
        }
        final Procedure1<KYPosition<?>> _function_1 = new Procedure1<KYPosition<?>>() {
          @Override
          public void apply(final KYPosition<?> it) {
            it.setAbsolute(absoluteTB);
            it.setRelative(relativeTB);
          }
        };
        KYPosition<?> _doubleArrow_1 = ObjectExtensions.<KYPosition<?>>operator_doubleArrow(_switchResult_1, _function_1);
        it.setY(_doubleArrow_1);
      }
    };
    return ObjectExtensions.<KPosition>operator_doubleArrow(_createKPosition, _function);
  }
  
  public KContainerRendering addInvisibleContainerRendering(final KGraphElement node) {
    if (node instanceof KNode) {
      return _addInvisibleContainerRendering((KNode)node);
    } else if (node instanceof KPort) {
      return _addInvisibleContainerRendering((KPort)node);
    } else if (node instanceof KLabel) {
      return _addInvisibleContainerRendering((KLabel)node);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node).toString());
    }
  }
  
  public KEllipse addEllipse(final KGraphElement node) {
    if (node instanceof KNode) {
      return _addEllipse((KNode)node);
    } else if (node instanceof KPort) {
      return _addEllipse((KPort)node);
    } else if (node instanceof KLabel) {
      return _addEllipse((KLabel)node);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node).toString());
    }
  }
  
  public KPolygon addPolygon(final KLabeledGraphElement node) {
    if (node instanceof KNode) {
      return _addPolygon((KNode)node);
    } else if (node instanceof KPort) {
      return _addPolygon((KPort)node);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node).toString());
    }
  }
  
  public KRectangle addRectangle(final KGraphElement node) {
    if (node instanceof KNode) {
      return _addRectangle((KNode)node);
    } else if (node instanceof KPort) {
      return _addRectangle((KPort)node);
    } else if (node instanceof KLabel) {
      return _addRectangle((KLabel)node);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node).toString());
    }
  }
  
  public KRoundedRectangle addRoundedRectangle(final KGraphElement node, final float cWidth, final float cHeight) {
    if (node instanceof KNode) {
      return _addRoundedRectangle((KNode)node, cWidth, cHeight);
    } else if (node instanceof KLabel) {
      return _addRoundedRectangle((KLabel)node, cWidth, cHeight);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node, cWidth, cHeight).toString());
    }
  }
  
  public KRoundedRectangle addRoundedRectangle(final KGraphElement node, final float cWidth, final float cHeight, final float lineWidth) {
    if (node instanceof KNode) {
      return _addRoundedRectangle((KNode)node, cWidth, cHeight, lineWidth);
    } else if (node instanceof KLabel) {
      return _addRoundedRectangle((KLabel)node, cWidth, cHeight, lineWidth);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(node, cWidth, cHeight, lineWidth).toString());
    }
  }
}
