/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.klighd.ui.view.syntheses;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import de.cau.cs.kieler.klighd.krendering.Colors;
import de.cau.cs.kieler.klighd.krendering.KAreaPlacementData;
import de.cau.cs.kieler.klighd.krendering.KGridPlacementData;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import de.cau.cs.kieler.klighd.ui.view.model.ErrorModel;
import de.cau.cs.kieler.klighd.ui.view.model.MessageModel;
import de.cau.cs.kieler.klighd.ui.view.syntheses.MessageModelSynthesis;
import de.cau.cs.kieler.klighd.ui.view.syntheses.action.ErrorModelShowExceptionAction;
import javax.inject.Inject;
import org.eclipse.elk.graph.KGraphData;
import org.eclipse.elk.graph.KNode;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Diagram synthesis for a {@link MessageModel}.
 * 
 * @author als
 * @kieler.design 2014-07-30 proposed
 * @kieler.rating 2014-07-30 proposed yellow
 */
@SuppressWarnings("all")
public class ErrorModelSynthesis extends AbstractDiagramSynthesis<ErrorModel> {
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  @Inject
  @Extension
  private MessageModelSynthesis _messageModelSynthesis;
  
  public final static String ID = "de.cau.cs.kieler.klighd.ui.view.syntheses.ErrorModelSynthesis";
  
  @Override
  public KNode transform(final ErrorModel model) {
    final KNode rootNode = this._messageModelSynthesis.transform(((MessageModel) model));
    boolean _and = false;
    boolean _notEquals = (!Objects.equal(rootNode, null));
    if (!_notEquals) {
      _and = false;
    } else {
      EList<KNode> _children = rootNode.getChildren();
      boolean _isEmpty = _children.isEmpty();
      boolean _not = (!_isEmpty);
      _and = _not;
    }
    if (_and) {
      EList<KNode> _children_1 = rootNode.getChildren();
      final KNode messageNode = IterableExtensions.<KNode>head(_children_1);
      EList<KGraphData> _data = messageNode.getData();
      Iterable<KRoundedRectangle> _filter = Iterables.<KRoundedRectangle>filter(_data, KRoundedRectangle.class);
      final KRoundedRectangle messageRect = IterableExtensions.<KRoundedRectangle>head(_filter);
      boolean _notEquals_1 = (!Objects.equal(messageRect, null));
      if (_notEquals_1) {
        EList<KRendering> _children_2 = messageRect.getChildren();
        KRendering _head = IterableExtensions.<KRendering>head(_children_2);
        this._kRenderingExtensions.setForeground(_head, Colors.RED);
        String _stackTrace = model.getStackTrace();
        boolean _notEquals_2 = (!Objects.equal(_stackTrace, null));
        if (_notEquals_2) {
          KText _addText = this._kContainerRenderingExtensions.addText(messageRect, "[Show Exception]");
          final Procedure1<KText> _function = new Procedure1<KText>() {
            @Override
            public void apply(final KText it) {
              ErrorModelSynthesis.this._kRenderingExtensions.setForeground(it, Colors.BLUE);
              ErrorModelSynthesis.this._kRenderingExtensions.setFontSize(it, 9);
              ErrorModelSynthesis.this._kRenderingExtensions.<KText>addSingleClickAction(it, ErrorModelShowExceptionAction.ID);
              ErrorModelSynthesis.this._kRenderingExtensions.<KText>addDoubleClickAction(it, ErrorModelShowExceptionAction.ID);
              KGridPlacementData _setGridPlacementData = ErrorModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it);
              KAreaPlacementData _from = ErrorModelSynthesis.this._kRenderingExtensions.from(_setGridPlacementData, ErrorModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, ErrorModelSynthesis.this._kRenderingExtensions.TOP, 4, 0);
              ErrorModelSynthesis.this._kRenderingExtensions.to(_from, ErrorModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, ErrorModelSynthesis.this._kRenderingExtensions.BOTTOM, 8, 0);
            }
          };
          ObjectExtensions.<KText>operator_doubleArrow(_addText, _function);
        }
      }
    }
    return rootNode;
  }
}
