/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.klighd.ui.view.syntheses;

import com.google.common.base.Objects;
import de.cau.cs.kieler.klighd.krendering.KAreaPlacementData;
import de.cau.cs.kieler.klighd.krendering.KGridPlacementData;
import de.cau.cs.kieler.klighd.krendering.KImage;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.KText;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KNodeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import de.cau.cs.kieler.klighd.syntheses.DiagramSyntheses;
import de.cau.cs.kieler.klighd.ui.view.model.MessageModel;
import javax.inject.Inject;
import org.eclipse.elk.graph.KNode;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Diagram synthesis for a {@link MessageModel}.
 * 
 * @author als
 * @kieler.design 2014-07-30 proposed
 * @kieler.rating 2014-07-30 proposed yellow
 */
@SuppressWarnings("all")
public class MessageModelSynthesis extends AbstractDiagramSynthesis<MessageModel> {
  @Inject
  @Extension
  private KNodeExtensions _kNodeExtensions;
  
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  public final static String ID = "de.cau.cs.kieler.klighd.ui.view.syntheses.MessageModelSynthesis";
  
  @Override
  public KNode transform(final MessageModel model) {
    final KNode rootNode = this._kNodeExtensions.createNode();
    EList<KNode> _children = rootNode.getChildren();
    KNode _createNode = this._kNodeExtensions.createNode(model);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode it) {
        KRectangle _addRectangle = MessageModelSynthesis.this._kRenderingExtensions.addRectangle(it);
        final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
          @Override
          public void apply(final KRectangle it) {
            MessageModelSynthesis.this._kRenderingExtensions.setInvisible(it, true);
            MessageModelSynthesis.this._kContainerRenderingExtensions.setGridPlacement(it, 1);
            boolean _hasIcon = model.hasIcon();
            if (_hasIcon) {
              KRectangle _addRectangle = MessageModelSynthesis.this._kContainerRenderingExtensions.addRectangle(it);
              final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
                @Override
                public void apply(final KRectangle it) {
                  MessageModelSynthesis.this._kRenderingExtensions.setInvisible(it, true);
                  int _iconSize = model.getIconSize();
                  int _iconSize_1 = model.getIconSize();
                  MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it, _iconSize, _iconSize_1);
                  String _iconPlugin = model.getIconPlugin();
                  String _iconPath = model.getIconPath();
                  KImage _addImage = MessageModelSynthesis.this._kContainerRenderingExtensions.addImage(it, _iconPlugin, _iconPath);
                  int _iconSize_2 = model.getIconSize();
                  int _iconSize_3 = model.getIconSize();
                  KImage _setPointPlacementData = MessageModelSynthesis.this._kRenderingExtensions.<KImage>setPointPlacementData(_addImage, MessageModelSynthesis.this._kRenderingExtensions.LEFT, 0, 0.5f, MessageModelSynthesis.this._kRenderingExtensions.TOP, 0, 0.5f, 
                    MessageModelSynthesis.this._kRenderingExtensions.H_CENTRAL, MessageModelSynthesis.this._kRenderingExtensions.V_CENTRAL, 0, 0, _iconSize_2, _iconSize_3);
                  MessageModelSynthesis.this._kContainerRenderingExtensions.addRectangularClip(_setPointPlacementData);
                }
              };
              ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
            }
            KRoundedRectangle _addRoundedRectangle = MessageModelSynthesis.this._kContainerRenderingExtensions.addRoundedRectangle(it, 7, 7);
            final Procedure1<KRoundedRectangle> _function_1 = new Procedure1<KRoundedRectangle>() {
              @Override
              public void apply(final KRoundedRectangle it) {
                MessageModelSynthesis.this._kContainerRenderingExtensions.setGridPlacement(it, 1);
                MessageModelSynthesis.this._kRenderingExtensions.setLineWidth(it, 2);
                String _title = model.getTitle();
                boolean _notEquals = (!Objects.equal(_title, null));
                if (_notEquals) {
                  String _title_1 = model.getTitle();
                  KText _addText = MessageModelSynthesis.this._kContainerRenderingExtensions.addText(it, _title_1);
                  final Procedure1<KText> _function = new Procedure1<KText>() {
                    @Override
                    public void apply(final KText it) {
                      MessageModelSynthesis.this._kRenderingExtensions.setFontSize(it, 12);
                      MessageModelSynthesis.this._kRenderingExtensions.setFontBold(it, true);
                      KGridPlacementData _setGridPlacementData = MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it);
                      KAreaPlacementData _from = MessageModelSynthesis.this._kRenderingExtensions.from(_setGridPlacementData, MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 8, 0);
                      MessageModelSynthesis.this._kRenderingExtensions.to(_from, MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 4, 0);
                      DiagramSyntheses.suppressSelectability(it);
                    }
                  };
                  ObjectExtensions.<KText>operator_doubleArrow(_addText, _function);
                }
                String _message = model.getMessage();
                boolean _notEquals_1 = (!Objects.equal(_message, null));
                if (_notEquals_1) {
                  String _message_1 = model.getMessage();
                  KText _addText_1 = MessageModelSynthesis.this._kContainerRenderingExtensions.addText(it, _message_1);
                  final Procedure1<KText> _function_1 = new Procedure1<KText>() {
                    @Override
                    public void apply(final KText it) {
                      MessageModelSynthesis.this._kRenderingExtensions.setFontSize(it, 12);
                      String _title = model.getTitle();
                      boolean _notEquals = (!Objects.equal(_title, null));
                      if (_notEquals) {
                        KGridPlacementData _setGridPlacementData = MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it);
                        KAreaPlacementData _from = MessageModelSynthesis.this._kRenderingExtensions.from(_setGridPlacementData, MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 0, 0);
                        MessageModelSynthesis.this._kRenderingExtensions.to(_from, MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 4, 0);
                      } else {
                        KGridPlacementData _setGridPlacementData_1 = MessageModelSynthesis.this._kRenderingExtensions.setGridPlacementData(it);
                        KAreaPlacementData _from_1 = MessageModelSynthesis.this._kRenderingExtensions.from(_setGridPlacementData_1, MessageModelSynthesis.this._kRenderingExtensions.LEFT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.TOP, 8, 0);
                        MessageModelSynthesis.this._kRenderingExtensions.to(_from_1, MessageModelSynthesis.this._kRenderingExtensions.RIGHT, 8, 0, MessageModelSynthesis.this._kRenderingExtensions.BOTTOM, 8, 0);
                      }
                      DiagramSyntheses.suppressSelectability(it);
                    }
                  };
                  ObjectExtensions.<KText>operator_doubleArrow(_addText_1, _function_1);
                }
              }
            };
            ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_addRoundedRectangle, _function_1);
          }
        };
        ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
      }
    };
    KNode _doubleArrow = ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
    _children.add(_doubleArrow);
    return rootNode;
  }
}
