/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2014 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.klighd.xtext.transformations;

import com.google.common.base.Objects;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Maps;
import com.google.inject.Inject;
import de.cau.cs.kieler.kiml.formats.gml.CollectionElement;
import de.cau.cs.kieler.kiml.formats.gml.Element;
import de.cau.cs.kieler.kiml.formats.gml.GMLModel;
import de.cau.cs.kieler.klighd.KlighdConstants;
import de.cau.cs.kieler.klighd.SynthesisOption;
import de.cau.cs.kieler.klighd.krendering.KBackground;
import de.cau.cs.kieler.klighd.krendering.KColor;
import de.cau.cs.kieler.klighd.krendering.KContainerRendering;
import de.cau.cs.kieler.klighd.krendering.KPolyline;
import de.cau.cs.kieler.klighd.krendering.KRectangle;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.klighd.krendering.KRoundedRectangle;
import de.cau.cs.kieler.klighd.krendering.extensions.KColorExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KContainerRenderingExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KEdgeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KNodeExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KPolylineExtensions;
import de.cau.cs.kieler.klighd.krendering.extensions.KRenderingExtensions;
import de.cau.cs.kieler.klighd.syntheses.AbstractDiagramSynthesis;
import de.cau.cs.kieler.klighd.util.KlighdProperties;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.function.Consumer;
import org.eclipse.elk.core.klayoutdata.KLayoutData;
import org.eclipse.elk.core.klayoutdata.KShapeLayout;
import org.eclipse.elk.core.math.ElkMath;
import org.eclipse.elk.core.math.KVector;
import org.eclipse.elk.core.math.KVectorChain;
import org.eclipse.elk.core.options.CoreOptions;
import org.eclipse.elk.core.util.ElkUtil;
import org.eclipse.elk.core.util.FixedLayoutProvider;
import org.eclipse.elk.graph.KEdge;
import org.eclipse.elk.graph.KGraphElement;
import org.eclipse.elk.graph.KLabel;
import org.eclipse.elk.graph.KNode;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Generic diagram synthesis for GML files.
 * 
 * @author uru
 */
@SuppressWarnings("all")
public class GMLDiagramSynthesis extends AbstractDiagramSynthesis<GMLModel> {
  @Inject
  @Extension
  private KNodeExtensions _kNodeExtensions;
  
  @Inject
  @Extension
  private KEdgeExtensions _kEdgeExtensions;
  
  @Inject
  @Extension
  private KRenderingExtensions _kRenderingExtensions;
  
  @Inject
  @Extension
  private KContainerRenderingExtensions _kContainerRenderingExtensions;
  
  @Inject
  @Extension
  private KColorExtensions _kColorExtensions;
  
  @Inject
  @Extension
  private KPolylineExtensions _kPolylineExtensions;
  
  private final HashMap<Integer, KNode> idMap = Maps.<Integer, KNode>newHashMap();
  
  private boolean hasPositions = false;
  
  public final static SynthesisOption SHOW_LABELS = SynthesisOption.createCheckOption(
    "Labels", Boolean.valueOf(true));
  
  @Override
  public List<SynthesisOption> getDisplayedSynthesisOptions() {
    return ImmutableList.<SynthesisOption>of(
      GMLDiagramSynthesis.SHOW_LABELS);
  }
  
  @Override
  public KNode transform(final GMLModel model) {
    KNode _createNode = this._kNodeExtensions.createNode();
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode root) {
        List<Element> _elements = model.getElements();
        final Consumer<Element> _function = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convert(it, root);
          }
        };
        _elements.forEach(_function);
        GMLDiagramSynthesis.this.centerNodes(root);
        if (GMLDiagramSynthesis.this.hasPositions) {
          GMLDiagramSynthesis.this.clipEdges(root);
        }
        if (GMLDiagramSynthesis.this.hasPositions) {
          GMLDiagramSynthesis.this.<KNode, String>setLayoutOption(root, CoreOptions.ALGORITHM, FixedLayoutProvider.ID);
        }
      }
    };
    return ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
  }
  
  private Iterable<Element> getElements(final Element d) {
    List<Element> _switchResult = null;
    boolean _matched = false;
    if (!_matched) {
      if (d instanceof CollectionElement) {
        _matched=true;
        List<Element> _xblockexpression = null;
        {
          final CollectionElement ce = ((CollectionElement) d);
          _xblockexpression = ce.getElements();
        }
        _switchResult = _xblockexpression;
      }
    }
    if (!_matched) {
      _switchResult = Collections.<Element>unmodifiableList(CollectionLiterals.<Element>newArrayList());
    }
    return _switchResult;
  }
  
  private void convert(final Element d, final KNode parent) {
    String _key = d.getKey();
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_key, "graph")) {
        _matched=true;
        Iterable<Element> _elements = this.getElements(d);
        final Function1<Element, Boolean> _function = new Function1<Element, Boolean>() {
          @Override
          public Boolean apply(final Element it) {
            String _key = it.getKey();
            return Boolean.valueOf(Objects.equal(_key, "node"));
          }
        };
        Iterable<Element> _filter = IterableExtensions.<Element>filter(_elements, _function);
        final Consumer<Element> _function_1 = new Consumer<Element>() {
          @Override
          public void accept(final Element e) {
            GMLDiagramSynthesis.this.createKNode(e, parent);
          }
        };
        _filter.forEach(_function_1);
        Iterable<Element> _elements_1 = this.getElements(d);
        final Function1<Element, Boolean> _function_2 = new Function1<Element, Boolean>() {
          @Override
          public Boolean apply(final Element it) {
            String _key = it.getKey();
            return Boolean.valueOf(Objects.equal(_key, "edge"));
          }
        };
        Iterable<Element> _filter_1 = IterableExtensions.<Element>filter(_elements_1, _function_2);
        final Consumer<Element> _function_3 = new Consumer<Element>() {
          @Override
          public void accept(final Element e) {
            GMLDiagramSynthesis.this.createKEdge(e);
          }
        };
        _filter_1.forEach(_function_3);
      }
    }
  }
  
  private void createKNode(final Element e, final KNode parent) {
    KNode _createNode = this._kNodeExtensions.createNode(e);
    final Procedure1<KNode> _function = new Procedure1<KNode>() {
      @Override
      public void apply(final KNode node) {
        EList<KNode> _children = parent.getChildren();
        _children.add(node);
        KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(node);
        KVector _kVector = new KVector();
        _layout.<KVector>setProperty(CoreOptions.POSITION, _kVector);
        KRectangle _addRectangle = GMLDiagramSynthesis.this._kRenderingExtensions.addRectangle(node);
        final Procedure1<KRectangle> _function = new Procedure1<KRectangle>() {
          @Override
          public void apply(final KRectangle it) {
            GMLDiagramSynthesis.this._kRenderingExtensions.setLineWidth(it, 1);
          }
        };
        ObjectExtensions.<KRectangle>operator_doubleArrow(_addRectangle, _function);
        Iterable<Element> _elements = GMLDiagramSynthesis.this.getElements(e);
        final Consumer<Element> _function_1 = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convertNode(it, node);
          }
        };
        _elements.forEach(_function_1);
      }
    };
    ObjectExtensions.<KNode>operator_doubleArrow(_createNode, _function);
  }
  
  private void createKEdge(final Element e) {
    KEdge _createEdge = this._kEdgeExtensions.createEdge(e);
    final Procedure1<KEdge> _function = new Procedure1<KEdge>() {
      @Override
      public void apply(final KEdge edge) {
        GMLDiagramSynthesis.this._kEdgeExtensions.addPolyline(edge);
        KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(edge);
        KVectorChain _kVectorChain = new KVectorChain();
        _layout.<KVectorChain>setProperty(CoreOptions.BEND_POINTS, _kVectorChain);
        Iterable<Element> _elements = GMLDiagramSynthesis.this.getElements(e);
        final Consumer<Element> _function = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convertEdge(it, edge);
          }
        };
        _elements.forEach(_function);
      }
    };
    ObjectExtensions.<KEdge>operator_doubleArrow(_createEdge, _function);
  }
  
  private void convertNode(final Element e, final KNode node) {
    String _key = e.getKey();
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_key, "id")) {
        _matched=true;
        String _value = e.getValue();
        Integer _int = this.toInt(_value);
        this.idMap.put(_int, node);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "label")) {
        _matched=true;
        boolean _booleanValue = this.getBooleanValue(GMLDiagramSynthesis.SHOW_LABELS);
        if (_booleanValue) {
          KLabel _createInitializedLabel = ElkUtil.createInitializedLabel(node);
          String _value_1 = e.getValue();
          String _stripQuotes = this.stripQuotes(_value_1);
          _createInitializedLabel.setText(_stripQuotes);
        }
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "graphics")) {
        _matched=true;
        Iterable<Element> _elements = this.getElements(e);
        final Consumer<Element> _function = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convertNode(it, node);
          }
        };
        _elements.forEach(_function);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "x")) {
        _matched=true;
        this.hasPositions = true;
        KLayoutData _layout = this.getLayout(node);
        final KVector v = _layout.<KVector>getProperty(CoreOptions.POSITION);
        String _value_2 = e.getValue();
        Float _float = this.toFloat(_value_2);
        v.x = (_float).floatValue();
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "y")) {
        _matched=true;
        this.hasPositions = true;
        KLayoutData _layout_1 = this.getLayout(node);
        final KVector v_1 = _layout_1.<KVector>getProperty(CoreOptions.POSITION);
        String _value_3 = e.getValue();
        Float _float_1 = this.toFloat(_value_3);
        v_1.y = (_float_1).floatValue();
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "w")) {
        _matched=true;
        String _value_4 = e.getValue();
        Float _float_2 = this.toFloat(_value_4);
        this._kNodeExtensions.setWidth(node, (_float_2).floatValue());
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "h")) {
        _matched=true;
        String _value_5 = e.getValue();
        Float _float_3 = this.toFloat(_value_5);
        this._kNodeExtensions.setHeight(node, (_float_3).floatValue());
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "line")) {
        _matched=true;
        KRendering _kRendering = this._kRenderingExtensions.getKRendering(node);
        String _value_6 = e.getValue();
        String _stripQuotes_1 = this.stripQuotes(_value_6);
        KColor _color = this._kColorExtensions.getColor(_stripQuotes_1);
        this._kRenderingExtensions.setForeground(_kRendering, _color);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "fill")) {
        _matched=true;
        KRendering _kRendering_1 = this._kRenderingExtensions.getKRendering(node);
        String _value_7 = e.getValue();
        String _stripQuotes_2 = this.stripQuotes(_value_7);
        KColor _color_1 = this._kColorExtensions.getColor(_stripQuotes_2);
        this._kRenderingExtensions.setBackground(_kRendering_1, _color_1);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "type")) {
        _matched=true;
        String _value_8 = e.getValue();
        String _stripQuotes_3 = this.stripQuotes(_value_8);
        boolean _matched_1 = false;
        if (!_matched_1) {
          if (Objects.equal(_stripQuotes_3, "oval")) {
            _matched_1=true;
            KContainerRendering _kContainerRendering = this._kRenderingExtensions.getKContainerRendering(node);
            this._kRenderingExtensions.setInvisible(_kContainerRendering, true);
            KContainerRendering _kContainerRendering_1 = this._kRenderingExtensions.getKContainerRendering(node);
            this._kContainerRenderingExtensions.addEllipse(_kContainerRendering_1);
          }
        }
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "node")) {
        _matched=true;
        this.createKNode(e, node);
        KLayoutData _layout_2 = this.getLayout(node);
        _layout_2.<Boolean>setProperty(KlighdProperties.EXPAND, Boolean.valueOf(false));
        final KContainerRendering ren = this._kRenderingExtensions.getKContainerRendering(node);
        Boolean _property = ren.<Boolean>getProperty(KlighdProperties.COLLAPSED_RENDERING);
        boolean _not = (!(_property).booleanValue());
        if (_not) {
          ren.<Boolean>setProperty(KlighdProperties.COLLAPSED_RENDERING, Boolean.valueOf(true));
          this._kRenderingExtensions.<KContainerRendering>addDoubleClickAction(ren, KlighdConstants.ACTION_COLLAPSE_EXPAND);
          KRoundedRectangle _addRoundedRectangle = this._kRenderingExtensions.addRoundedRectangle(node, 5, 5);
          final Procedure1<KRoundedRectangle> _function_1 = new Procedure1<KRoundedRectangle>() {
            @Override
            public void apply(final KRoundedRectangle er) {
              KColor _color = GMLDiagramSynthesis.this._kColorExtensions.getColor("#0000FF");
              GMLDiagramSynthesis.this._kRenderingExtensions.setBackground(er, _color);
              KBackground _background = GMLDiagramSynthesis.this._kRenderingExtensions.getBackground(er);
              _background.setAlpha(20);
              er.<Boolean>setProperty(KlighdProperties.EXPANDED_RENDERING, Boolean.valueOf(true));
              GMLDiagramSynthesis.this._kRenderingExtensions.<KRoundedRectangle>addDoubleClickAction(er, KlighdConstants.ACTION_COLLAPSE_EXPAND);
            }
          };
          ObjectExtensions.<KRoundedRectangle>operator_doubleArrow(_addRoundedRectangle, _function_1);
        }
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "edge")) {
        _matched=true;
        this.createKEdge(e);
      }
    }
  }
  
  private void convertEdge(final Element e, final KEdge edge) {
    String _key = e.getKey();
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(_key, "source")) {
        _matched=true;
        String _value = e.getValue();
        Integer _int = this.toInt(_value);
        final KNode src = this.idMap.get(_int);
        edge.setSource(src);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "target")) {
        _matched=true;
        String _value_1 = e.getValue();
        Integer _int_1 = this.toInt(_value_1);
        final KNode tgt = this.idMap.get(_int_1);
        edge.setTarget(tgt);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "graphics")) {
        _matched=true;
        Iterable<Element> _elements = this.getElements(e);
        final Consumer<Element> _function = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convertEdge(it, edge);
          }
        };
        _elements.forEach(_function);
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "arrow")) {
        _matched=true;
        String _value_2 = e.getValue();
        String _stripQuotes = this.stripQuotes(_value_2);
        boolean _matched_1 = false;
        if (!_matched_1) {
          if (Objects.equal(_stripQuotes, "head")) {
            _matched_1=true;
            KRendering _kRendering = this._kRenderingExtensions.getKRendering(edge);
            this._kPolylineExtensions.addHeadArrowDecorator(((KPolyline) _kRendering));
          }
        }
        if (!_matched_1) {
          if (Objects.equal(_stripQuotes, "tail")) {
            _matched_1=true;
            KRendering _kRendering_1 = this._kRenderingExtensions.getKRendering(edge);
            this._kPolylineExtensions.addTailArrowDecorator(((KPolyline) _kRendering_1));
          }
        }
        if (!_matched_1) {
          if (Objects.equal(_stripQuotes, "last")) {
            _matched_1=true;
            KRendering _kRendering_2 = this._kRenderingExtensions.getKRendering(edge);
            this._kPolylineExtensions.addHeadArrowDecorator(((KPolyline) _kRendering_2));
          }
        }
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "width")) {
        _matched=true;
        KRendering _kRendering_3 = this._kRenderingExtensions.getKRendering(edge);
        String _value_3 = e.getValue();
        Float _valueOf = Float.valueOf(_value_3);
        this._kRenderingExtensions.setLineWidth(_kRendering_3, (_valueOf).floatValue());
      }
    }
    if (!_matched) {
      if (Objects.equal(_key, "Line")) {
        _matched=true;
        Iterable<Element> _elements_1 = this.getElements(e);
        final Function1<Element, Boolean> _function_1 = new Function1<Element, Boolean>() {
          @Override
          public Boolean apply(final Element it) {
            String _key = it.getKey();
            return Boolean.valueOf(Objects.equal(_key, "point"));
          }
        };
        Iterable<Element> _filter = IterableExtensions.<Element>filter(_elements_1, _function_1);
        final Consumer<Element> _function_2 = new Consumer<Element>() {
          @Override
          public void accept(final Element it) {
            GMLDiagramSynthesis.this.convertEdgeBendpoints(it, edge);
          }
        };
        _filter.forEach(_function_2);
      }
    }
  }
  
  private boolean convertEdgeBendpoints(final Element bp, final KEdge e) {
    boolean _xblockexpression = false;
    {
      KLayoutData _layout = this.getLayout(e);
      final KVectorChain bps = _layout.<KVectorChain>getProperty(CoreOptions.BEND_POINTS);
      Iterable<Element> _elements = this.getElements(bp);
      final Function1<Element, Boolean> _function = new Function1<Element, Boolean>() {
        @Override
        public Boolean apply(final Element it) {
          String _key = it.getKey();
          return Boolean.valueOf(Objects.equal(_key, "x"));
        }
      };
      final Element x = IterableExtensions.<Element>findFirst(_elements, _function);
      Iterable<Element> _elements_1 = this.getElements(bp);
      final Function1<Element, Boolean> _function_1 = new Function1<Element, Boolean>() {
        @Override
        public Boolean apply(final Element it) {
          String _key = it.getKey();
          return Boolean.valueOf(Objects.equal(_key, "y"));
        }
      };
      final Element y = IterableExtensions.<Element>findFirst(_elements_1, _function_1);
      String _value = x.getValue();
      Double _valueOf = Double.valueOf(_value);
      String _value_1 = y.getValue();
      Double _valueOf_1 = Double.valueOf(_value_1);
      KVector _kVector = new KVector((_valueOf).doubleValue(), (_valueOf_1).doubleValue());
      _xblockexpression = bps.add(_kVector);
    }
    return _xblockexpression;
  }
  
  private void clipEdges(final KNode parent) {
    EList<KNode> _children = parent.getChildren();
    final Consumer<KNode> _function = new Consumer<KNode>() {
      @Override
      public void accept(final KNode n) {
        EList<KEdge> _outgoingEdges = n.getOutgoingEdges();
        final Consumer<KEdge> _function = new Consumer<KEdge>() {
          @Override
          public void accept(final KEdge e) {
            KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(e);
            final KVectorChain bps = _layout.<KVectorChain>getProperty(CoreOptions.BEND_POINTS);
            KNode _source = e.getSource();
            final KShapeLayout sShape = GMLDiagramSynthesis.this.getShapeLayout(_source);
            KNode _target = e.getTarget();
            final KShapeLayout tShape = GMLDiagramSynthesis.this.getShapeLayout(_target);
            boolean _isEmpty = bps.isEmpty();
            if (_isEmpty) {
              KNode _source_1 = e.getSource();
              KLayoutData _layout_1 = GMLDiagramSynthesis.this.getLayout(_source_1);
              KVector _property = _layout_1.<KVector>getProperty(CoreOptions.POSITION);
              KVector _clone = _property.clone();
              float _width = sShape.getWidth();
              float _divide = (_width / 2f);
              float _height = sShape.getHeight();
              float _divide_1 = (_height / 2f);
              final KVector src = _clone.add(_divide, _divide_1);
              KNode _target_1 = e.getTarget();
              KLayoutData _layout_2 = GMLDiagramSynthesis.this.getLayout(_target_1);
              KVector _property_1 = _layout_2.<KVector>getProperty(CoreOptions.POSITION);
              KVector _clone_1 = _property_1.clone();
              float _width_1 = tShape.getWidth();
              float _divide_2 = (_width_1 / 2f);
              float _height_1 = tShape.getHeight();
              float _divide_3 = (_height_1 / 2f);
              final KVector tgt = _clone_1.add(_divide_2, _divide_3);
              KVector _clone_2 = tgt.clone();
              KVector _sub = _clone_2.sub(src);
              KNode _source_2 = e.getSource();
              KVector _clip = GMLDiagramSynthesis.this.clip(_sub, _source_2);
              final KVector st = _clip.add(src);
              bps.add(0, st);
              KVector _clone_3 = src.clone();
              KVector _sub_1 = _clone_3.sub(tgt);
              KNode _target_2 = e.getTarget();
              KVector _clip_1 = GMLDiagramSynthesis.this.clip(_sub_1, _target_2);
              final KVector ts = _clip_1.add(tgt);
              bps.add(ts);
            } else {
              int _size = bps.size();
              boolean _greaterEqualsThan = (_size >= 2);
              if (_greaterEqualsThan) {
                final KVector src_1 = IterableExtensions.<KVector>head(bps);
                Iterable<KVector> _tail = IterableExtensions.<KVector>tail(bps);
                final KVector snd = IterableExtensions.<KVector>head(_tail);
                KVector _clone_4 = snd.clone();
                KVector _sub_2 = _clone_4.sub(src_1);
                KNode _source_3 = e.getSource();
                KVector _clip_2 = GMLDiagramSynthesis.this.clip(_sub_2, _source_3);
                src_1.add(_clip_2);
                Object _clone_5 = bps.clone();
                List<KVector> _reverse = ListExtensions.<KVector>reverse(((List<KVector>) _clone_5));
                final KVector tgt_1 = IterableExtensions.<KVector>head(_reverse);
                Object _clone_6 = bps.clone();
                List<KVector> _reverse_1 = ListExtensions.<KVector>reverse(((List<KVector>) _clone_6));
                Iterable<KVector> _tail_1 = IterableExtensions.<KVector>tail(_reverse_1);
                final KVector penult = IterableExtensions.<KVector>head(_tail_1);
                KVector _clone_7 = penult.clone();
                KVector _sub_3 = _clone_7.sub(tgt_1);
                KNode _target_3 = e.getTarget();
                KVector _clip_3 = GMLDiagramSynthesis.this.clip(_sub_3, _target_3);
                tgt_1.add(_clip_3);
              }
            }
          }
        };
        _outgoingEdges.forEach(_function);
        GMLDiagramSynthesis.this.clipEdges(n);
      }
    };
    _children.forEach(_function);
  }
  
  private KVector clip(final KVector v, final KNode n) {
    KShapeLayout _shapeLayout = this.getShapeLayout(n);
    float _width = _shapeLayout.getWidth();
    KShapeLayout _shapeLayout_1 = this.getShapeLayout(n);
    float _height = _shapeLayout_1.getHeight();
    return ElkMath.clipVector(v, _width, _height);
  }
  
  private double minX = Double.MAX_VALUE;
  
  private double minY = Double.MAX_VALUE;
  
  /**
   * KGraph's node positions refer to the top-left corner
   * while GML specifies center positions. Here we move
   * the node by half its width and height towards the
   * top left position.
   */
  private void centerNodes(final KNode parent) {
    EList<KNode> _children = parent.getChildren();
    final Consumer<KNode> _function = new Consumer<KNode>() {
      @Override
      public void accept(final KNode n) {
        KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(n);
        final KVector pos = _layout.<KVector>getProperty(CoreOptions.POSITION);
        boolean _notEquals = (!Objects.equal(pos, null));
        if (_notEquals) {
          float _width = GMLDiagramSynthesis.this._kNodeExtensions.getWidth(n);
          float _divide = (_width / 2);
          double _minus = (pos.x - _divide);
          double _min = Math.min(GMLDiagramSynthesis.this.minX, _minus);
          GMLDiagramSynthesis.this.minX = _min;
          float _height = GMLDiagramSynthesis.this._kNodeExtensions.getHeight(n);
          float _divide_1 = (_height / 2);
          double _minus_1 = (pos.y - _divide_1);
          double _min_1 = Math.min(GMLDiagramSynthesis.this.minY, _minus_1);
          GMLDiagramSynthesis.this.minY = _min_1;
        }
        GMLDiagramSynthesis.this.centerNodes(n);
      }
    };
    _children.forEach(_function);
    KLayoutData _layout = this.getLayout(parent);
    final Float borderSpacing = _layout.<Float>getProperty(CoreOptions.SPACING_BORDER);
    EList<KNode> _children_1 = parent.getChildren();
    final Consumer<KNode> _function_1 = new Consumer<KNode>() {
      @Override
      public void accept(final KNode n) {
        KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(n);
        final KVector pos = _layout.<KVector>getProperty(CoreOptions.POSITION);
        boolean _notEquals = (!Objects.equal(pos, null));
        if (_notEquals) {
          float _width = GMLDiagramSynthesis.this._kNodeExtensions.getWidth(n);
          float _divide = (_width / 2);
          double _minus = (((-GMLDiagramSynthesis.this.minX) + pos.x) - _divide);
          double _plus = (_minus + (borderSpacing).floatValue());
          pos.x = _plus;
          float _height = GMLDiagramSynthesis.this._kNodeExtensions.getHeight(n);
          float _divide_1 = (_height / 2);
          double _minus_1 = (((-GMLDiagramSynthesis.this.minY) + pos.y) - _divide_1);
          double _plus_1 = (_minus_1 + (borderSpacing).floatValue());
          pos.y = _plus_1;
        }
        EList<KEdge> _outgoingEdges = n.getOutgoingEdges();
        final Consumer<KEdge> _function = new Consumer<KEdge>() {
          @Override
          public void accept(final KEdge e) {
            KLayoutData _layout = GMLDiagramSynthesis.this.getLayout(e);
            final KVectorChain bps = _layout.<KVectorChain>getProperty(CoreOptions.BEND_POINTS);
            boolean _notEquals = (!Objects.equal(bps, null));
            if (_notEquals) {
              final Consumer<KVector> _function = new Consumer<KVector>() {
                @Override
                public void accept(final KVector bp) {
                  bp.x = (bp.x - GMLDiagramSynthesis.this.minX);
                  bp.y = (bp.y - GMLDiagramSynthesis.this.minY);
                }
              };
              bps.forEach(_function);
            }
          }
        };
        _outgoingEdges.forEach(_function);
        EList<KLabel> _labels = n.getLabels();
        final Consumer<KLabel> _function_1 = new Consumer<KLabel>() {
          @Override
          public void accept(final KLabel l) {
            KShapeLayout _shapeLayout = GMLDiagramSynthesis.this.getShapeLayout(l);
            float _width = GMLDiagramSynthesis.this._kNodeExtensions.getWidth(n);
            float _divide = (_width / 2);
            _shapeLayout.setXpos(_divide);
            KShapeLayout _shapeLayout_1 = GMLDiagramSynthesis.this.getShapeLayout(l);
            float _height = GMLDiagramSynthesis.this._kNodeExtensions.getHeight(n);
            float _divide_1 = (_height / 2);
            _shapeLayout_1.setYpos(_divide_1);
          }
        };
        _labels.forEach(_function_1);
      }
    };
    _children_1.forEach(_function_1);
  }
  
  private Integer toInt(final String s) {
    return Integer.valueOf(s);
  }
  
  private Float toFloat(final String s) {
    return Float.valueOf(s);
  }
  
  private String stripQuotes(final String s) {
    String _xifexpression = null;
    boolean _and = false;
    boolean _startsWith = s.startsWith("\"");
    if (!_startsWith) {
      _and = false;
    } else {
      boolean _endsWith = s.endsWith("\"");
      _and = _endsWith;
    }
    if (_and) {
      int _length = s.length();
      int _minus = (_length - 1);
      _xifexpression = s.substring(1, _minus);
    } else {
      _xifexpression = s;
    }
    return _xifexpression;
  }
  
  private KLayoutData getLayout(final KGraphElement e) {
    return e.<KLayoutData>getData(KLayoutData.class);
  }
  
  private KShapeLayout getShapeLayout(final KGraphElement e) {
    return e.<KShapeLayout>getData(KShapeLayout.class);
  }
}
