/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2013 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.ptolemy.klighd.transformation;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.MarkerExtensions;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.MiscellaneousExtensions;
import java.util.List;
import org.eclipse.elk.core.klayoutdata.KLayoutData;
import org.eclipse.elk.core.util.ElkUtil;
import org.eclipse.elk.graph.KEdge;
import org.eclipse.elk.graph.KNode;
import org.eclipse.elk.graph.KPort;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * Optional optimization that flattens the graph by eliminating all composite actors.
 * 
 * @author msp
 */
@SuppressWarnings("all")
public class Flattener {
  /**
   * Access to marked nodes.
   */
  @Inject
  @Extension
  private MarkerExtensions _markerExtensions;
  
  /**
   * Miscellaneous stuff to make my life easier.
   */
  @Inject
  @Extension
  private MiscellaneousExtensions _miscellaneousExtensions;
  
  /**
   * Flatten all composite actors in the given graph.
   */
  public void flatten(final KNode node) {
    final List<KNode> compositeNodes = CollectionLiterals.<KNode>newArrayList();
    EList<KNode> _children = node.getChildren();
    for (final KNode child : _children) {
      boolean _and = false;
      boolean _and_1 = false;
      EList<KNode> _children_1 = child.getChildren();
      boolean _isEmpty = _children_1.isEmpty();
      boolean _not = (!_isEmpty);
      if (!_not) {
        _and_1 = false;
      } else {
        boolean _isMarkedAsState = this._markerExtensions.isMarkedAsState(child);
        boolean _not_1 = (!_isMarkedAsState);
        _and_1 = _not_1;
      }
      if (!_and_1) {
        _and = false;
      } else {
        boolean _isMarkedAsStateMachineContainer = this._markerExtensions.isMarkedAsStateMachineContainer(child);
        boolean _not_2 = (!_isMarkedAsStateMachineContainer);
        _and = _not_2;
      }
      if (_and) {
        compositeNodes.add(child);
      }
    }
    for (final KNode child_1 : compositeNodes) {
      {
        this.flatten(child_1);
        this.eliminateHierarchy(node, child_1);
      }
    }
  }
  
  /**
   * Eliminate the hierarchy level introduced by the given child node.
   * 
   * @param parent the parent node
   * @param child the child node
   */
  public void eliminateHierarchy(final KNode parent, final KNode child) {
    boolean _or = false;
    EList<KEdge> _incomingEdges = child.getIncomingEdges();
    final Function1<KEdge, Boolean> _function = new Function1<KEdge, Boolean>() {
      @Override
      public Boolean apply(final KEdge e) {
        KNode _source = e.getSource();
        KNode _parent = _source.getParent();
        return Boolean.valueOf(Objects.equal(_parent, child));
      }
    };
    boolean _exists = IterableExtensions.<KEdge>exists(_incomingEdges, _function);
    if (_exists) {
      _or = true;
    } else {
      EList<KEdge> _outgoingEdges = child.getOutgoingEdges();
      final Function1<KEdge, Boolean> _function_1 = new Function1<KEdge, Boolean>() {
        @Override
        public Boolean apply(final KEdge e) {
          KNode _target = e.getTarget();
          KNode _parent = _target.getParent();
          return Boolean.valueOf(Objects.equal(_parent, child));
        }
      };
      boolean _exists_1 = IterableExtensions.<KEdge>exists(_outgoingEdges, _function_1);
      _or = _exists_1;
    }
    boolean _not = (!_or);
    if (_not) {
      return;
    }
    EList<KPort> _ports = child.getPorts();
    for (final KPort port : _ports) {
      {
        final List<KEdge> incoming = CollectionLiterals.<KEdge>newArrayList();
        final List<KEdge> outgoing = CollectionLiterals.<KEdge>newArrayList();
        EList<KEdge> _edges = port.getEdges();
        for (final KEdge edge : _edges) {
          {
            KPort _sourcePort = edge.getSourcePort();
            boolean _equals = Objects.equal(_sourcePort, port);
            if (_equals) {
              outgoing.add(edge);
            }
            KPort _targetPort = edge.getTargetPort();
            boolean _equals_1 = Objects.equal(_targetPort, port);
            if (_equals_1) {
              incoming.add(edge);
            }
          }
        }
        for (final KEdge edge1 : incoming) {
          for (final KEdge edge2 : outgoing) {
            boolean _and = false;
            KNode _source = edge1.getSource();
            boolean _notEquals = (!Objects.equal(_source, child));
            if (!_notEquals) {
              _and = false;
            } else {
              KNode _target = edge2.getTarget();
              boolean _notEquals_1 = (!Objects.equal(_target, child));
              _and = _notEquals_1;
            }
            if (_and) {
              KEdge _createInitializedEdge = ElkUtil.createInitializedEdge();
              final Procedure1<KEdge> _function_2 = new Procedure1<KEdge>() {
                @Override
                public void apply(final KEdge e) {
                  KNode _source = edge1.getSource();
                  e.setSource(_source);
                  KPort _sourcePort = edge1.getSourcePort();
                  e.setSourcePort(_sourcePort);
                  KNode _target = edge2.getTarget();
                  e.setTarget(_target);
                  KPort _targetPort = edge2.getTargetPort();
                  e.setTargetPort(_targetPort);
                  KLayoutData _layoutEx = Flattener.this._miscellaneousExtensions.getLayoutEx(e);
                  KLayoutData _layoutEx_1 = Flattener.this._miscellaneousExtensions.getLayoutEx(edge1);
                  _layoutEx.copyProperties(_layoutEx_1);
                }
              };
              ObjectExtensions.<KEdge>operator_doubleArrow(_createInitializedEdge, _function_2);
            }
          }
        }
        for (final KEdge edge1_1 : incoming) {
          {
            edge1_1.setSource(null);
            edge1_1.setSourcePort(null);
          }
        }
        for (final KEdge edge2_1 : outgoing) {
          {
            edge2_1.setTarget(null);
            edge2_1.setTargetPort(null);
          }
        }
      }
    }
    while ((!child.getChildren().isEmpty())) {
      {
        EList<KNode> _children = child.getChildren();
        final KNode grandChild = _children.get(0);
        boolean _or_1 = false;
        boolean _isMarkedAsDirector = this._markerExtensions.isMarkedAsDirector(grandChild);
        if (_isMarkedAsDirector) {
          _or_1 = true;
        } else {
          boolean _isMarkedAsParameterNode = this._markerExtensions.isMarkedAsParameterNode(grandChild);
          _or_1 = _isMarkedAsParameterNode;
        }
        if (_or_1) {
          grandChild.setParent(null);
        } else {
          grandChild.setParent(parent);
        }
      }
    }
    child.setParent(null);
    EList<KEdge> _incomingEdges_1 = child.getIncomingEdges();
    for (final KEdge edge : _incomingEdges_1) {
      {
        edge.setSource(null);
        edge.setSourcePort(null);
      }
    }
    EList<KEdge> _outgoingEdges_1 = child.getOutgoingEdges();
    for (final KEdge edge_1 : _outgoingEdges_1) {
      {
        edge_1.setTarget(null);
        edge_1.setTargetPort(null);
      }
    }
  }
}
