/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://www.informatik.uni-kiel.de/rtsys/kieler/
 * 
 * Copyright 2013 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 * See the file epl-v10.html for the license text.
 */
package de.cau.cs.kieler.ptolemy.klighd.transformation;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import de.cau.cs.kieler.klighd.krendering.KRendering;
import de.cau.cs.kieler.ptolemy.klighd.transformation.ImageLoadWorker;
import de.cau.cs.kieler.ptolemy.klighd.transformation.PtolemyInterface;
import java.awt.EventQueue;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Extension;
import ptolemy.kernel.Entity;

/**
 * Provides methods to retrieve KRenderings for the SVG graphics that specify how Ptolemy objects look.
 * 
 * @author ckru
 * @author cds
 */
@SuppressWarnings("all")
public final class PtolemyFigureInterface {
  /**
   * Instantiating Ptolemy entities.
   */
  @Inject
  @Extension
  private PtolemyInterface _ptolemyInterface;
  
  /**
   * The object that does the image loading in another thread.
   */
  @Inject
  private ImageLoadWorker imageLoadWorker;
  
  /**
   * Tries to return a KRendering of actors with the given class name.
   * 
   * @param className class name of the actor.
   * @return the actor's KRendering representation or {@code null} if there was a problem.
   */
  public KRendering createPtolemyFigureRendering(final String className) {
    try {
      Entity entity = null;
      try {
        Entity _instantiatePtolemyEntity = this._ptolemyInterface.instantiatePtolemyEntity(className);
        entity = _instantiatePtolemyEntity;
      } catch (final Throwable _t) {
        if (_t instanceof Exception) {
          final Exception e = (Exception)_t;
          return null;
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      }
      this.imageLoadWorker.reset(entity);
      while (Objects.equal(this.imageLoadWorker.getResult(), null)) {
        EventQueue.invokeAndWait(this.imageLoadWorker);
      }
      return this.imageLoadWorker.getResult();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
