/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 */
package de.cau.cs.kieler.ptolemy.klighd.transformation.comments;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.AnnotationExtensions;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.LabelExtensions;
import de.cau.cs.kieler.ptolemy.klighd.transformation.util.TransformationConstants;
import org.eclipse.elk.core.comments.IExplicitAttachmentProvider;
import org.eclipse.elk.graph.KGraphElement;
import org.eclipse.elk.graph.KNode;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Extension;
import org.ptolemy.moml.PropertyType;

/**
 * Extracts explicit attachments made between comments and actors in Ptolemy.
 * 
 * @author cds
 */
@SuppressWarnings("all")
public final class ExplicitPtolemyAttachmentProvider implements IExplicitAttachmentProvider {
  @Inject
  @Extension
  private AnnotationExtensions _annotationExtensions;
  
  @Inject
  @Extension
  private LabelExtensions _labelExtensions;
  
  /**
   * {@inheritDoc}
   */
  @Override
  public KGraphElement findExplicitAttachment(final KNode comment) {
    final PropertyType location = this._annotationExtensions.getAnnotation(comment, TransformationConstants.ANNOTATION_LOCATION);
    PropertyType _annotation = null;
    if (location!=null) {
      _annotation=this._annotationExtensions.getAnnotation(location, TransformationConstants.ANNOTATION_RELATIVE_TO);
    }
    final PropertyType relativeTo = _annotation;
    PropertyType _annotation_1 = null;
    if (location!=null) {
      _annotation_1=this._annotationExtensions.getAnnotation(location, TransformationConstants.ANNOTATION_RELATIVE_TO_ELEMENT_NAME);
    }
    final PropertyType relativeToElementName = _annotation_1;
    boolean _and = false;
    boolean _and_1 = false;
    boolean _and_2 = false;
    boolean _notEquals = (!Objects.equal(relativeTo, null));
    if (!_notEquals) {
      _and_2 = false;
    } else {
      boolean _notEquals_1 = (!Objects.equal(relativeToElementName, null));
      _and_2 = _notEquals_1;
    }
    if (!_and_2) {
      _and_1 = false;
    } else {
      String _value = relativeToElementName.getValue();
      boolean _notEquals_2 = (!Objects.equal(_value, null));
      _and_1 = _notEquals_2;
    }
    if (!_and_1) {
      _and = false;
    } else {
      String _value_1 = relativeToElementName.getValue();
      boolean _equals = _value_1.equals("entity");
      _and = _equals;
    }
    if (_and) {
      KNode _parent = comment.getParent();
      EList<KNode> _children = _parent.getChildren();
      for (final KNode sibling : _children) {
        String _name = this._labelExtensions.getName(sibling);
        String _value_2 = relativeTo.getValue();
        boolean _equals_1 = _name.equals(_value_2);
        if (_equals_1) {
          return sibling;
        }
      }
    }
    return null;
  }
}
