/**
 * KIELER - Kiel Integrated Environment for Layout Eclipse RichClient
 * 
 * http://rtsys.informatik.uni-kiel.de/kieler
 * 
 * Copyright 2015 by
 * + Kiel University
 *   + Department of Computer Science
 *     + Real-Time and Embedded Systems Group
 * 
 * This code is provided under the terms of the Eclipse Public License (EPL).
 */
package de.cau.cs.kieler.ptolemy.klighd.transformation.comments;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import de.cau.cs.kieler.klighd.microlayout.Bounds;
import de.cau.cs.kieler.klighd.microlayout.PlacementUtil;
import de.cau.cs.kieler.ptolemy.klighd.PtolemyProperties;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.AnnotationExtensions;
import de.cau.cs.kieler.ptolemy.klighd.transformation.extensions.MarkerExtensions;
import de.cau.cs.kieler.ptolemy.klighd.transformation.util.TransformationConstants;
import java.awt.geom.Rectangle2D;
import java.util.List;
import org.eclipse.elk.core.comments.IBoundsProvider;
import org.eclipse.elk.core.klayoutdata.KLayoutData;
import org.eclipse.elk.graph.KNode;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Extension;
import org.ptolemy.moml.PropertyType;

/**
 * Retrieves the position of the given node in the original Ptolemy diagram, if any, as well as
 * its size as given by the node's shape layout. The shape layout's size fields are expected to
 * have been correctly set during the visualization step of the diagram synthesis. It's a good
 * idea to wrap this bounds provider in a caching bounds provider prior to use, as calculating
 * bounds for Ptolemy objects can be quite expensive.
 * 
 * @author cds
 */
@SuppressWarnings("all")
public final class PtolemyBoundsProvider implements IBoundsProvider {
  @Inject
  @Extension
  private AnnotationExtensions _annotationExtensions;
  
  @Inject
  @Extension
  private MarkerExtensions _markerExtensions;
  
  /**
   * {@inheritDoc}
   */
  @Override
  public Rectangle2D.Double boundsFor(final KNode node) {
    final Rectangle2D.Double bounds = new Rectangle2D.Double();
    bounds.x = 2e20;
    bounds.y = 2e20;
    PropertyType locationAnnotation = this._annotationExtensions.getAnnotation(node, TransformationConstants.ANNOTATION_LOCATION);
    String locationAnnotationValue = null;
    boolean _equals = Objects.equal(locationAnnotation, null);
    if (_equals) {
      KLayoutData _layout = this._annotationExtensions.getLayout(node);
      String locationSpecialAnnotation = _layout.<String>getProperty(PtolemyProperties.PT_LOCATION);
      boolean _equals_1 = Objects.equal(locationSpecialAnnotation, null);
      if (_equals_1) {
        return bounds;
      } else {
        locationAnnotationValue = locationSpecialAnnotation;
      }
    } else {
      String _value = locationAnnotation.getValue();
      locationAnnotationValue = _value;
    }
    final String locationString = locationAnnotationValue.replaceAll("[\\s\\[\\]{}]+", "");
    final String[] locationArray = locationString.split(",");
    int _size = ((List<String>)Conversions.doWrapArray(locationArray)).size();
    boolean _equals_2 = (_size == 2);
    if (_equals_2) {
      try {
        String _get = locationArray[0];
        Double _valueOf = Double.valueOf(_get);
        bounds.x = (_valueOf).doubleValue();
        String _get_1 = locationArray[1];
        Double _valueOf_1 = Double.valueOf(_get_1);
        bounds.y = (_valueOf_1).doubleValue();
        final Bounds estimatedSize = PlacementUtil.estimateSize(node);
        float _width = estimatedSize.getWidth();
        bounds.width = _width;
        float _height = estimatedSize.getHeight();
        bounds.height = _height;
      } catch (final Throwable _t) {
        if (_t instanceof NumberFormatException) {
          final NumberFormatException e = (NumberFormatException)_t;
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      }
    }
    String _xifexpression = null;
    boolean _isMarkedAsComment = this._markerExtensions.isMarkedAsComment(node);
    if (_isMarkedAsComment) {
      _xifexpression = "northwest";
    } else {
      _xifexpression = "center";
    }
    final String anchorDefault = _xifexpression;
    final PropertyType anchorAnnotation = this._annotationExtensions.getAnnotation(node, TransformationConstants.ANNOTATION_ANCHOR);
    String _elvis = null;
    String _value_1 = null;
    if (anchorAnnotation!=null) {
      _value_1=anchorAnnotation.getValue();
    }
    if (_value_1 != null) {
      _elvis = _value_1;
    } else {
      _elvis = anchorDefault;
    }
    final String anchorString = _elvis;
    boolean _matched = false;
    if (!_matched) {
      if (Objects.equal(anchorString, "north")) {
        _matched=true;
        bounds.x = (bounds.y - (bounds.width / 2));
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "south")) {
        _matched=true;
        bounds.x = (bounds.x - (bounds.width / 2));
        bounds.y = (bounds.y - bounds.height);
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "west")) {
        _matched=true;
        bounds.y = (bounds.y - (bounds.height / 2));
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "east")) {
        _matched=true;
        bounds.x = (bounds.x - bounds.width);
        bounds.y = (bounds.y - (bounds.height / 2));
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "northwest")) {
        _matched=true;
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "northeast")) {
        _matched=true;
        bounds.x = (bounds.x - bounds.width);
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "southwest")) {
        _matched=true;
        bounds.y = (bounds.y - bounds.height);
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "sountheast")) {
        _matched=true;
        bounds.x = (bounds.x - bounds.width);
        bounds.y = (bounds.y - bounds.height);
      }
    }
    if (!_matched) {
      if (Objects.equal(anchorString, "southeast")) {
        _matched=true;
        bounds.x = (bounds.x - bounds.width);
        bounds.y = (bounds.y - bounds.height);
      }
    }
    if (!_matched) {
      {
        bounds.x = (bounds.x - (bounds.width / 2));
        bounds.y = (bounds.y - (bounds.height / 2));
      }
    }
    return bounds;
  }
}
